/***************************************************************************
 *   Copyright (C) 2005 by Sean Harmer                                     *
 *   sh@rama.homelinux.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#ifndef KFILEACLPROPSPLUGIN_H
#define KFILEACLPROPSPLUGIN_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qdialog.h>
#include <qwidget.h>
#include <kfileitem.h>

#include <sys/acl.h>

class QCheckBox;
class QComboBox;
class QFrame;
class QGroupBox;
class QLabel;
class QPushButton;

class KComboBox;
class KACLListView;
class KACLListViewItem;

/**
 * Access Control List Entry that uses tri-state perms which are
 * needed for the case where we are modifying ACLs on files and not
 * all files have the same ACL entries and/or the ACL entries have
 * different perms.
 */
class KACLEntry
{
public:
    KACLEntry( acl_entry_t = 0, KFileItem* pItem = 0 );
    ~KACLEntry();

    acl_tag_t entryType() const { return m_type; }

    void mergePermissions( acl_entry_t aclEntry );

    const QString& mergeQualifier( acl_entry_t aclEntry, KFileItem* pItem );

    void setPartialEntry( bool b ) { m_bPartialEntry = b; }
    bool isPartialEntry() const { return m_bPartialEntry; }

    QString debug() const;

private:
    acl_tag_t m_type;
    acl_perm_t m_perms;
    acl_perm_t m_partialPerms;
    QString m_qualifier;
    bool m_bPartialEntry;

    friend class KACL;
    friend class KACLListView;
    friend class KACLListViewItem;
};


/**
 * Access Control List representation
 */
class KACL
{
public:
    KACL( acl_type_t aclType = ACL_TYPE_ACCESS );
    ~KACL();

    const QPtrList<KACLEntry>& entries() const { return m_entries; }
    QPtrList<KACLEntry>& entries() { return m_entries; }

    acl_type_t aclType() const { return m_aclType; }

    /**
     * Adds ACL entries to this ACL from the ACLs on a list of files.
     * If an ACL entry does not exist in all of the files, then it is
     * marked as a partial entry. Also, if the perms on a given ACL entry
     * are differ between any two files, then the appropriate perms are
     * marked as partial.
     *
     * \returns The number of ACL entries added.
     */
    int addEntries( KFileItemList items );

    int mergeEntries( KFileItem* pItem );

    int entryCount() const { return m_entries.count(); }

    bool removeEntry( KACLEntry* pEntry ){ return m_entries.removeRef( pEntry ); }

    void clearEntries() { return m_entries.clear(); }

    void removeExtendedEntries();

    /**
     * Searches for an entry of type entryType.
     *
     * \returns TRUE is an entry of type ACL_MASK exists, FALSE otherwise.
     */
    bool hasEntryOfType( acl_tag_t entryType ) const;

protected:
    acl_type_t m_aclType;
    bool m_bHasEmptyACL;
    QPtrList<KACLEntry> m_entries;
};


class KACLEntryDlg : public QDialog
{
Q_OBJECT

public:
    KACLEntryDlg( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~KACLEntryDlg();

    QLabel* textLabel1;
    QLabel* textLabel2;
    QComboBox* m_comboType;
    QComboBox* m_comboQualifier;
    QGroupBox* groupBox1;
    QCheckBox* m_cbRead;
    QCheckBox* m_cbWrite;
    QCheckBox* m_cbExecute;
    QPushButton* m_pbHelp;
    QPushButton* m_pbOK;
    QPushButton* m_pbCancel;
};


/**
 * 'Access Control List' plugin
 * In this plugin you can modify acls and change
 * the owner/group of a file.
 * @internal
 */
class KFileACLPropsPlugin : public QWidget
{
    Q_OBJECT
public:
    enum PermissionsMode
    {
        PermissionsOnlyFiles = 0,
        PermissionsOnlyDirs = 1,
        PermissionsOnlyLinks = 2,
        PermissionsMixed = 3
    };

    /**
     * Constructor
     */
    KFileACLPropsPlugin( KFileItemList items, QWidget* parent = 0, const char* name = 0 );
    virtual ~KFileACLPropsPlugin();

    //virtual void applyChanges();

    /**
     * Tests whether the file specified by _items needs an 'ACL' plugin.
     */
    //static bool supports( KFileItemList _items );

public slots:
    void slotAccessListChanged( int index );
    void slotSelectedEntriesChanged();
    void removeSelectedEntries();
    void showAddEntryDlg();
    void showModifyEntryDlg();
    void deleteACL();
    void updateDeleteACLButton();

private:
    // GUI variables
    QFrame* m_frame;
    QGroupBox* m_gbACL;
    QLabel* m_lblControlList;
    KComboBox* m_comboControlList;
    KACLListView* m_lvACL;
    QPushButton* m_pbAdd;
    QPushButton* m_pbModify;
    QPushButton* m_pbRemove;
    QPushButton* m_pbDeleteACL;
    QGroupBox* m_gbSpecial;
    QCheckBox* m_cbSetUID;
    QCheckBox* m_cbSetGID;
    QCheckBox* m_cbSticky;
    QCheckBox* m_cbRecursive;
    QPushButton* m_pbHelp;
    QPushButton* m_pbOK;
    QPushButton* m_pbCancel;

    // Working variables
    KACL m_aclAccess;
    KACL m_aclDefault;
    KACL* m_pACL; // Pointer to currently selected control list

    PermissionsMode m_pmode;
    QString m_strOwner;
    QString m_strGroup;
    mode_t m_permissions;
    mode_t m_partialPermissions;
    bool m_bCanChangePermissions;
    bool m_bIsIrregular;

    class KFileACLPropsPluginPrivate;
    KFileACLPropsPluginPrivate *d;
};

#endif
