/*
 * KDE Composite Manager, 
 * Copyright (C) 2003-2005 Fredrik Höglund <fredrik@kde.org>
 *
 * Based on xcompmgr,
 * Copyright (C) 2003 Keith Packard <keithp@keithp.com>
 *
 * Internal structure and naming style inspired by KWin,
 * Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 * Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Fredrik Höglund not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Fredrik Höglund makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * FREDRIK HÖGLUND DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL FREDRIK HÖGLUND BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include "animation.h"
#include "settings.h"
#include "client.h"

#include <kglobal.h>


Animation::Animation( Client *client, AnimationType type, uint flags )
	: mClient( client ), mType( type ), mFlags( flags ), mDone( false )
{
	// Mark the time we were created
	mTime.start();

	switch ( type )
	{
		case FadeIn:
			// Set the initial opacity for FadeIn animations
			client->setAnimationOpacity( 0.0 );
			mIncrement = (1000. / Settings::fadeInTime()) / 1000.;
			break;

		case FadeOut:
			mIncrement = (1000. / Settings::fadeOutTime()) / 1000.;
			break;
	}
}


Animation::~Animation()
{
	if ( flags() & HideClientWhenDone )
		client()->hide();

	if ( flags() & DeleteClientWhenDone )
		Workspace::instance()->deleteClient( client() );
}


void Animation::setType( AnimationType type )
{
	mType = type;

	switch ( type )
	{
		case FadeIn:
			mIncrement = (1000. / Settings::fadeInTime()) / 1000.;
			break;

		case FadeOut:
			mIncrement = (1000. / Settings::fadeOutTime()) / 1000.;
			break;
	}
}


void Animation::update()
{
	double opacity = client()->animationOpacity();
	double delta = mIncrement * mTime.restart();

	switch ( mType )
	{
		case FadeIn:
			opacity = qMin( opacity + delta, 1.0 );
			break;

		case FadeOut:
			opacity = qMax( opacity - delta, 0.0 );
			break;
	};

	// Make sure the opacity change is large enough to warrant a repaint
	bool needRepaint = ( int(client()->animationOpacity() * 0xff) != int(opacity * 0xff) );
	client()->setAnimationOpacity( opacity );

	// Damage the area the client occupies
	if ( needRepaint && client()->isPainted() ) {
		XserverRegion region = client()->createRegion( WindowAndShadow );
		Workspace::instance()->addDamage( region ); // Destroys region
	}

	// This will cause the animation to be deleted 
	if ( ( mType == FadeIn && opacity == 1.0 ) || ( mType == FadeOut && opacity == 0.0 ) )
		setIsDone( true );
}

