/*
 * KDE Composite Manager, 
 * Copyright (C) 2003-2005 Fredrik Höglund <fredrik@kde.org>
 *
 * Based on xcompmgr,
 * Copyright (C) 2003 Keith Packard <keithp@keithp.com>
 *
 * Internal structure and naming style inspired by KWin,
 * Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 * Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Fredrik Höglund not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Fredrik Höglund makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * FREDRIK HÖGLUND DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL FREDRIK HÖGLUND BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */


#include "client.h"
#include "workspace.h"

#include <X11/Xatom.h>
#include <X11/extensions/Xcomposite.h>
#include <X11/Xmu/WinUtil.h>

#include "shadowpainter.h"
#include "settings.h"
#include "utils.h"

#include <iostream>

extern Display *dpy;

// Declared and interned in main.cpp
extern Atom net_wm_window_type;
extern Atom net_wm_window_type_desktop;
extern Atom net_wm_window_type_dock;
extern Atom net_wm_window_type_toolbar;
extern Atom net_wm_window_type_menu;
extern Atom net_wm_window_type_utility;
extern Atom net_wm_window_type_splash;
extern Atom net_wm_window_type_dialog;
extern Atom net_wm_window_type_normal;
extern Atom net_wm_window_type_override; // KDE extension for no-border windows
extern Atom net_wm_window_type_topmenu;  // KDE extension for mac style menubar
extern Atom net_wm_window_opacity;
extern Atom net_wm_desktop;


Client::Client( Window win, const XWindowAttributes &attr )
	: mId( win ),
      mClientId( None ),
	  mTransient( None ),
	  mPixmap( None ),
	  mPicture( None ),
	  mShadowPict( None ),
	  mAlphaMask( None ),
	  mOpacity( 1.0 ),
	  mOpacityHint( 1.0 ),
	  mDamage( None ),
	  mGeometry( 0, 0, 0, 0 ),
	  mBorderWidth( 0 ),
	  mVisibleRegion( None ),
	  mShapeRegion( None ),
	  mSavedClipRegion( None ),
	  mDesktop( -1 ),
	  mVisible( attr.map_state == IsViewable ),
	  mPainted( false ),
	  mActive( false ),
	  mShapeClipValid( false ),
	  mAlphaChannel( false ),
	  mOverrideRedirect( attr.override_redirect ),
	  mInputOnly( attr.c_class == InputOnly ),
	  mWindowTypeValid( false ),
	  mAlphaMaskValid( false ),
	  mOpacityHintValid( false )
{
	if ( !isInputOnly() )
	{
		// Find the window pixel format
		mFormat = XRenderFindVisualFormat( dpy, attr.visual );

		// Check if the window has an alpha channel
		if ( mFormat && mFormat->type == PictTypeDirect && mFormat->direct.alphaMask )
			setHasAlphaChannel( true );

		// Create the damage object
		mDamage = XDamageCreate( dpy, winId(), XDamageReportNonEmpty );

		// Find the client window in the frame
		if ( !isOverrideRedirect() )
		{
			mClientId = XmuClientWindow( dpy, winId() );

			// Get the value of the _NET_WM_DESKTOP hint
			readCardinalProperty( clientId(), net_wm_desktop, &mDesktop );

			// Get the value of the WM_TRANSIENT_FOR hint
			XGetTransientForHint( dpy, clientId(), &mTransient );
		} else
			mClientId = winId();

		// Track changes to the opacity hint
		XSelectInput( dpy, winId(), PropertyChangeMask );

		// Monitor changes to the EWMH hints
		if ( winId() != clientId() )
			XSelectInput( dpy, clientId(), PropertyChangeMask );
	
		// Make sure we get notified when the window shape is changed
		XShapeSelectInput( dpy, winId(), ShapeNotifyMask );

		mShadowRadius = Settings::inactiveShadowRadius();
	}

	// Initialize the geometry
	mGeometry = QRect( attr.x, attr.y, attr.width, attr.height );
	mBorderWidth = attr.border_width;

	// Make the window visible if it's mapped
	if ( isVisible() )
		show();
}


Client::~Client()
{
	releaseRegion( mVisibleRegion );
	releaseRegion( mShapeRegion );
	releaseRegion( mSavedClipRegion );

	releasePixmap( mPixmap );
	releasePicture( mPicture );
	releasePicture( mAlphaMask );
	releasePicture( mShadowPict );

	// Stop listening for events
	XSelectInput( dpy, winId(), NoEventMask );

	if ( winId() != clientId() )
		XSelectInput( dpy, clientId(), NoEventMask );

	if ( mDamage )
		XDamageDestroy( dpy, mDamage );
}


// Creates the window picture
Picture Client::createWindowPicture() const
{
	// Increase the ref count for the backing pixmap
	if ( !mPixmap )
		mPixmap = XCompositeNameWindowPixmap( dpy, winId() );

	// Create the window picture
	return XRenderCreatePicture( dpy, mPixmap, mFormat, 0, 0 );
}


// Creates the shadow picture
Picture Client::createShadowPicture() const
{
	QRect r = geometry( Shadow );
	return ShadowPainter::createShadowPicture( r.size(), mShadowRadius );
}


// This function gets called when the window geometry is changed.
void Client::geometryChanged( const QRect &newGeometry, int newBorderWidth )
{
	// Destroy the cached shape region since it's no longer valid
	releaseRegion( mShapeRegion );

	// If the window has been resized
	if ( width() != newGeometry.width() || height() != newGeometry.height()
			|| borderWidth() != newBorderWidth )
	{
		// Deref the now stale backing pixmap, and free the picture
		releasePixmap( mPixmap );
		releasePicture( mPicture );
		releasePicture( mShadowPict );

		mShapeClipValid = false;
	}

	mGeometry    = newGeometry;
	mBorderWidth = newBorderWidth;

	if ( !isVisible() )
		return;

	// Create an empty region and initialize it with the current visible region
	XserverRegion damage = XFixesCreateRegion( dpy, 0, 0  );
	if ( mVisibleRegion != None )
		XFixesCopyRegion( dpy, damage, mVisibleRegion );

	// Create a region with the new geometry, and union it with the old
	XserverRegion region = createRegion( WindowAndShadow );
	XFixesUnionRegion( dpy, damage, damage, region );
	XFixesDestroyRegion( dpy, region );

	// Add the region to the workspace's damage
	Workspace::instance()->addDamage( damage );
}


// This function gets called when the windows shape is changed
// (by using the XShape extension).
void Client::shapeChanged()
{
	// Destroy the cached shape region since it's no longer valid
	releaseRegion( mShapeRegion );

	mShapeClipValid = false;

	// Damage the whole window (destroys mVisibleRegion)
	Workspace::instance()->addDamage( mVisibleRegion );
}


// Sets the clients shadow radius to the specified value, and damages the
// shadow region.
void Client::setShadowRadius( double radius )
{
	XserverRegion damage = None;

	// If the new radius is greater than the current one, we'll
	// update the radius before computing the damage region.
	// (createRegion() uses mShadowRadius)
	if ( radius > mShadowRadius )
	{
		mShadowRadius = radius;

		if ( isPainted() && mShadowPict )
			damage = createRegion( Shadow );
	}

	// If not we'll do it in the opposite order, to make sure the
	// damage covers the whole updated region.
	else if ( radius < mShadowRadius )
	{
		if ( isPainted() && mShadowPict )
			damage = createRegion( Shadow );

		mShadowRadius = radius;
	}

	// If the shadow radius was changed, destroy the cached shadow picture so
	// it will be recreated with the new radius.
	if ( damage )
	{
		releasePicture( mShadowPict );
		Workspace::instance()->addDamage( damage ); // Destroys damage
	}
}


// Called when the client gets focus (becomes the active window)
void Client::focusIn()
{
	mActive = true;

	// Don't change the shadow radius for panels and desktop widgets
	if ( windowType() != NET::Dock && windowType() != NET::Desktop )
		setShadowRadius( Settings::activeShadowRadius() );
}


// Called when the window loses focus (becomes inactive)
void Client::focusOut()
{
	mActive = false;

	// Don't change the shadow radius for panels and desktop widgets
	if ( windowType() != NET::Dock && windowType() != NET::Desktop )
		setShadowRadius( Settings::inactiveShadowRadius() );
}


// Returns the window geometry of the specified extents
QRect Client::geometry( ShapeType type ) const
{
	QRect r;

	switch ( type )
	{
		case WindowAndBorder:
			r = QRect( x(), y(), width() + borderWidth() * 2, height() + borderWidth() * 2 );
			break;

		case Shadow: {
			int xOffset = isActive() ? Settings::activeShadowOffsetX()
						: Settings::inactiveShadowOffsetX();
			int yOffset = isActive() ? Settings::activeShadowOffsetY()
						: Settings::inactiveShadowOffsetY();

			r = ShadowPainter::shadowGeometry( geometry( WindowAndBorder ), mShadowRadius );
			r.moveBy( xOffset, yOffset );
			break;
		}

		case WindowAndShadow:
			r = geometry( Shadow ) | geometry( WindowAndBorder );
			break;

		case WindowOnly:
			r = QRect( x() + borderWidth(), y() + borderWidth(), width(), height() );
			break;

		default:
			r = QRect();
	}

	return r;
}


// Creates and returns a region object of the specified type:
// WindowOnly       matches the window shape precisely
// WindowAndBorder  matches the window shape + X border
// WindowAndShadow  matches the window shape + X border and shadow.
// Shadow           contains the shadow geometry only
XserverRegion Client::createRegion( ShapeType type ) const
{
	XserverRegion region;

	switch ( type )
	{
		// Region matching the window shape, minus the X border
		case WindowOnly:
			region = XFixesCreateRegionFromWindow( dpy, winId(), WindowRegionBounding );
			XFixesTranslateRegion( dpy, region, x() + borderWidth(), y() + borderWidth() );
			break;

		// All other region types
		default:
			region = createRegionFromQRect( geometry( type ) );
	}

	return region;
}


// Returns a cached copy of the windows' shape region, creating it if necessary.
XserverRegion Client::shape() const
{
	if ( mShapeRegion == None )
		mShapeRegion = createRegion( WindowOnly );

	return mShapeRegion;
}


// Updates or initializes the region with the bounding rectangle for the window and its shadow,
// as they appear on the screen. This function is only called when updating the root window.
void Client::updateOnScreenRegion()
{
	// TODO: Optimize: only update the region if it's actually invalid
	// to reduce X server traffic

	if ( mVisibleRegion != None )
		XFixesDestroyRegion( dpy, mVisibleRegion );

	mVisibleRegion = createRegion( WindowAndShadow );
}


// Marks the window as visible so it will be drawn the next time the root window is updated.
// This function doesn't damage the region the window occupies since we'll receive a damage
// notify event as soon as the window is drawn by the client.
void Client::show()
{
	mVisible = true;
	mPainted = false;

	// Read the window type and opacity hints immediately
	if ( !mWindowTypeValid ) {
		mWindowType = getWindowType();
		mWindowTypeValid = true;
	}

	if ( !mOpacityHintValid || !mAlphaMaskValid )
		updateAlphaMask();
}


// Marks the window as hidden so it won't be drawn when the root window is updated
void Client::hide()
{
	if ( !isVisible() )
		return;

	mVisible         = false;
	mPainted         = false;
	mShapeClipValid  = false;
	//mAlphaMaskValid  = false;

	releasePixmap( mPixmap );      // Deref the backing pixmap
	releasePicture( mPicture );    // Free the window picture
	releasePicture( mShadowPict ); // Free the shadow picture
	//releasePicture( mAlphaMask );  // Free the opacity mask

	// When a window is hidden, add the windows' visible region to the workspace's
	// damage so the underlying windows (and overlying translucent windows) are redrawn.
	// This is necessary since they won't receive expose events when this happens.
	// (Due to the composite extension providing backing-store.)
	if ( mVisibleRegion != None )
		Workspace::instance()->addDamage( mVisibleRegion ); // Destroys mVisibleRegion
}


void Client::reconfigure()
{
	double radius = isActive() ? Settings::activeShadowRadius()
			: Settings::inactiveShadowRadius();

	// If the shadow radius setting has changed, destroy the cached picture
	// so it will be recreated with the new radius.
	if ( radius != mShadowRadius ) {
		mShadowRadius = radius;
		releasePicture( mShadowPict );
	}

	// If shadows are disabled, make sure we don't keep any cached shadow
	// pictures around.
	if ( !Settings::drawShadows() )
		releasePicture( mShadowPict );

	// We have to do this since a client will have its opacity set to 0.0
	// when it's hidden if animations were enabled.
	if ( !Settings::fadeWindows() )
		setAnimationOpacity( 1.0 );
}


void Client::renewWindowPixmap()
{
	releasePicture( mPicture );
	releasePixmap( mPixmap );
}


// Reads the _NET_WM_WINDOW_TYPE property
NET::WindowType Client::getWindowType() const
{
	NET::WindowType type = NET::Unknown;

	int nitems = 0;
	void *data = readXProperty( clientId(), net_wm_window_type, XA_ATOM, &nitems );

	if ( data && nitems > 0 )
	{
		Atom *types = reinterpret_cast<Atom*>( data );

		for ( int i = 0; i < nitems; i++ )
		{
			if ( types[i] == net_wm_window_type_normal )
				type = NET::Normal;
			else if ( types[i] == net_wm_window_type_dialog )
				type = NET::Dialog;
			else if ( types[i] == net_wm_window_type_desktop )
				type = NET::Desktop;
			else if ( types[i] == net_wm_window_type_splash )
				type = NET::Splash;
			else if ( types[i] == net_wm_window_type_dock )
				type = NET::Dock;
			else if ( types[i] == net_wm_window_type_utility )
				type = NET::Utility;
			else if ( types[i] == net_wm_window_type_toolbar )
				type = NET::Toolbar;
			else if ( types[i] == net_wm_window_type_menu )
				type = NET::Menu;
			else if ( types[i] == net_wm_window_type_topmenu )
				type = NET::TopMenu;
			else if ( types[i] == net_wm_window_type_override )
				type = NET::Override;

			if ( type != NET::Unknown )
				break;
		}

		XFree( data );
	}

	return type;
}


double Client::getOpacityHint() const
{
	ulong value = 0xffffffff;

	// Try to read the hint from the window frame first (it has precedense)
	bool success = readCardinalProperty( winId(), net_wm_window_opacity, (int*)&value );

	// If that failed, look at the embedded window
	if ( !success && winId() != clientId() )
		readCardinalProperty( clientId(), net_wm_window_opacity, (int*)&value );

	return double( value ) / 0xffffffff;
}


void Client::updateAlphaMask() const
{
	int alpha = int( opacity() * 0xffff );

	// No alpha mask is needed when the client is fully opaque
	if ( alpha == 0xffff && mAlphaMask != None )
		releasePicture( mAlphaMask );

	else if ( alpha < 0xffff )
	{
		if ( mAlphaMask == None )
			mAlphaMask = createPicture( 1, 1, PictStandardA8, Repeating );

		XRenderColor col;
		col.alpha = alpha;
		XRenderFillRectangle( dpy, PictOpSrc, mAlphaMask, &col, 0, 0, 1, 1 );
	}

	mAlphaMaskValid = true;
}
