/* This file is part of the KDE project
   Copyright (C) 2005 Ramakrishna R <coolrama@gmail.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef _lbapi_h_
#define _lbapi_h_

#include<qstring.h>
#include<qstringlist.h>
#include<string>

using namespace std;

/** Namespace KLB collects all the KLabelManager functionality. */
namespace KLB
{
/**
* This enum is used for specifying type of labels HIDDEN meaning not visible for viewing by default.
* and NOT_HIDDEN meaning visible.
**/
enum hiddenStatus {HIDDEN=1, NOT_HIDDEN=0};
/**
* LabelManager API  provides methods for application and retrievals of labels to paths
* (file path, html links, or other protocols supported by konqueror)
* This class is used in an application where access to these labels is required.
* This api uses mysql database(to provide speed and efficient storage. It is being used to 
* support complex seatch operations to be implemented in near future. 
* mysqld deamon must be running when applications use this API.
* A simple application which uses this api is shown below. Let us name this application as "bookmark".
* This simple application applies label "bookmark" to html links passed on the command line. 
*
* @code
* using namespace std;
* using namespace KLB;
* int main(int argc, char* argv[])
* {
* 	if(argc < 2)
*	{
*		cout << " usage: bookmark website1 website2 ..." <<endl;
*		exit(0);
*	}
*	LabelManager * mgr = new LabelManager();
* 	QString labelName = "Bookmarks";
*	int i;
*	for(i=1; i<argc ; ++i)
*	{
*		mgr->applyLabel(labelName,argv[i]);
*	}
* 	return 0;
* }
* @endcode
* Please note that all labels and paths are case sensitive.
**/
	class LabelManager
	{
	public :
/**
* Constructor is responsible for setting up connections with the database
* 
**/
		LabelManager();
//		bool createLabel(const char* labelName);
/**
* This function is used to create a brand new Label name.
* The new label name will be immediately visible/accessible to/by  the other methods of the api.
* @param labelName The name of the new Label to be created.
* @param hidden This field can be used by applications which want to hide labels 
* created by applications( not users).
**/
		bool createLabel(const QString &labelName,hiddenStatus hidden=KLB::NOT_HIDDEN);
/**
* This is a convenience function that behaves like the above, 
* only @p comment is also added to the label so that you can comment a label for later reference. 
**/
		bool createLabel(const QString &labelName,const QString &comment,hiddenStatus hidden=KLB::NOT_HIDDEN);
/**
* Associate given label to given path
* @param labelName the label name which is already created.
* @param path the absolute path of a given file or link.
**/
		bool applyLabel(const QString &labelName, const QString &path);
		
//		bool getLabels(const char* path,string &labelList);
//		bool getLabels(const QString & path,QString &labelList);
/**
* This function is used to retrieve all the Labels applied for a particular file or link. 
* @param path The absoulute path for which label list is to be retrieved.
* @param labelList This QStringList will contain the label names associated with the file specified.
**/
		bool getLabels(const QString & path,QStringList &labelList);
//		bool getPaths(const char* labelName,string &pathList);
//		bool getPaths(const QString & labelName,QString &pathList);
/**
* This function is used to retrieve the absolute paths for a given label name
* @param labelName The Label Name for which the path list is to be retrieved.
* @param pathList The QStringList will contain the path list after execution of the method.
**/
		bool getPaths(const QString & labelName,QStringList &pathList);
		
//		bool getAllLabels(QString &labelList);
/**
* This function will return all the labels currently created.
* @param labelList This QStringList variable will contain the list of all Labels 
* after the execution of the method
* @param hidden This field can be set to KLB::HIDDEN if u want to get even the hidden labels
**/
		bool getAllLabels(QStringList &labelList, hiddenStatus hidden=KLB::NOT_HIDDEN);
/**
* This function is used to associate keywords with a particular label. 
* These keywords will be used when automatically applying labels to file based on content.
* @param labelName This is the label name to be associated.
* @param keywordList This list contains keywords for which the labelname is to be associated with.
**/
		bool addKeywords(const QString & labelName,const QStringList &keywordList);
/**
* This function is used to automatatically apply label to a file based on the content.
* The content has to be passed to this function in the form of QSTringList containing words of file.
* This function does not scan the file passed in path field. 
* It only searches for keywords in the QStrignList words.
* @param path the path of  the file whose content is passed in words.
* @param words The content of the file in the form of List of words.
**/		
		bool scanKeywordsAndAutomaticApply(const QString & path , const QStringList &words);
/**
* This function is used to search for links for which a combination of labels apply.
* You can search for all links with either labelled as say "important" or labelled as "personal".
* In that case the @p searchQuery would be "names="important" or names="personal""  
* @param pathList This QStringList will contain all the paths after the search operation
* @param searchQuery This is a query string in the format names=labelName1 or names=labelname2 or ...
**/
		bool searchLabels(QStringList & pathList ,const QString &searchQuery);
/**
* This function is used to delete a label. It will return false if no such label is present.
* @param labelName The label which is to be deleted
**/
		bool deleteLabel(const QString &labelName);
/**
* This function is used to rename a label. It will return false if no such label is present.
* @param oldLabelName The label which is to be renamed
* @param newLabelName This the new name for the Label
**/
		bool renameLabel(const QString &oldLabelName,const QString &newLabelName);
		
/**
* This function is used to remove a label for a particular path
* It will return false if no such label is present for the given path
* @param LabelName The label which is to be removed
* @param path This path for which the labelling is to be removed
**/
		bool removeLabel(const QString &LabelName,const QString &path);
/**
* This function is used to get the entire information in bulk in XML format. 
* This function could be useful for applications which need all the information at once
* The information is stored in the following format 
* &lt;klb&gt;
* &lt;AllLabels&gt;
* &lt;Label&gt;
* &lt;LabelId&gt;1&lt;/LabelId&gt;
* &lt;name&gt;'Important'&lt;/name&gt;
* &lt;/Label&gt;
* &lt;Label&gt;
* &lt;LabelId&gt;2&lt;/LabelId&gt;
* &lt;name&gt;'MyProjects'&lt;/name&gt;
* &lt;/Label&gt;
* &lt;/AllLabels&gt;
* &lt;Allpath&gt;
* &lt;pathLabels&gt;
* &lt;path&gt;'file:/root/LabelBrowser/'&lt;/path&gt;
* &lt;Labeld&gt;1&lt;LabelId&gt; &lt;Labeld&gt;2&lt;LabelId&gt; 
* &lt;/pathLabels&gt;
* &lt;/Allpath&gt;
* &lt;/klb&gt;
* From the above data we can deduce that the path 'file:/root/LabelBrowser/' has 2 labels associated with it
* They are LabelId 1,2 which are 'Important' and 'MyProjects' respectively.
* @param xmlData This is where all the information gets stored after execution
**/	
		bool getAllXML(QString &xmlData);
	};
}

#endif

