/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "backupwizardpages.h"
#include "backupwizard.h"

//FIXME: Make the paths absolute
#include "../common/identifierwidget.h"
#include "../common/mergerwidget.h"

#include <KDebug>
#include <KLineEdit>

#include <QtGui/QVBoxLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QLineEdit>

Nepomuk::IntroPage::IntroPage(QWidget* parent)
    : QWizardPage( parent )
{
    setupUi( this );
    
    setTitle("Nepomuk Backup");
    setSubTitle("Please choose one of the following options");

    registerField("systemBackup", m_systemBackup);
    registerField("custonBackup", m_customBackup);
    registerField("backupLocation", static_cast<QLineEdit*>( m_urlRequester->lineEdit() ) );
}

int Nepomuk::IntroPage::nextId() const
{
    if( m_backup->isChecked() )
        return BackupWizard::Id_BackupOptionsPage;
    else if( m_restore->isChecked() )
        return BackupWizard::Id_RestorePage;
    else
        return BackupWizard::Id_CheckpointPage;
}


//
// Backup Options Page
//
Nepomuk::BackupOptionsPage::BackupOptionsPage(QWidget* parent)
    : QWizardPage(parent)
{
    setupUi( this );
    
    setTitle("Nepomuk Backup");
    setSubTitle("Please choose one of the following options");

    registerField("iterativeBackup", m_iterativeBackup);
}


int Nepomuk::BackupOptionsPage::nextId() const
{
    return BackupWizard::Id_BackupPage;
}


//
// Backup Page
//
Nepomuk::BackupPage::BackupPage(QWidget* parent): QWizardPage(parent)
{
    setupUi( this );
    setTitle("Nepomuk Backup");
    setSubTitle("Performing backup");

    m_backupManager = new BackupManager( QLatin1String("org.kde.nepomuk.services.nepomukbackupsync"),
                                         "/backupmanager",
                                         QDBusConnection::sessionBus(), this);
    connect( m_backupManager, SIGNAL(backupDone()), this, SLOT(slotBackupDone()) );
    connect( m_backupManager, SIGNAL(backupNotRequired(QDateTime)), this, SLOT(slotBackupNotRequired(QDateTime)) );
}

void Nepomuk::BackupPage::initializePage()
{
    QString backupUrl;
    if( field("custonBackup").toBool() )
        backupUrl = field("backupLocation").toString();
    
    if( field("iterativeBackup").toBool() )
        m_backupManager->backup( backupUrl );
    else
        m_backupManager->freshBackup( backupUrl );

    m_status->setText("Performing backup ..");
}

int Nepomuk::BackupPage::nextId() const
{
    return -1;
}

void Nepomuk::BackupPage::slotBackupDone()
{
    m_status->setText("Backup completed successfully");
    m_progressBar->setMaximum( 100 );
    m_progressBar->setValue( 100 );
}

void Nepomuk::BackupPage::slotBackupNotRequired(QDateTime dt)
{
    m_progressBar->setDisabled( true );
    m_progressBar->setMaximum( 100 );
    m_progressBar->setValue( 0 );
    m_status->setText("Backup not required. Last backup performed on " + dt.toString());
}


Nepomuk::CheckpointPage::CheckpointPage(QWidget* parent): QWizardPage(parent)
{
    setupUi( this );

    m_backupManager = new BackupManager( QLatin1String("org.kde.nepomuk.services.nepomukbackupsync"),
                                         "/backupmanager",
                                         QDBusConnection::sessionBus(), this);
    connect( m_revert, SIGNAL(clicked(bool)), this, SLOT(revert()) );
}

void Nepomuk::CheckpointPage::initializePage()
{
    QString backupUrl;
    if( field("custonBackup").toBool() )
        backupUrl = field("backupLocation").toString();

    QDBusPendingReply< QStringList > reply = m_backupManager->checkpoints( backupUrl );
    reply.waitForFinished();
    if( reply.isValid() ) {
        QStringList list = reply.value();
        foreach( const QString & check, list ) 
            m_checkpointList->addItem( check );
    }
}

void Nepomuk::CheckpointPage::revert()
{
    QString backupUrl;
    if( field("custonBackup").toBool() )
        backupUrl = field("backupLocation").toString();
    
    QList< QListWidgetItem* > items = m_checkpointList->selectedItems();
    if( items.isEmpty() )
        return;

    if( items.size() == 1 ) {
        QString dt = items.first()->text();
        m_backupManager->revertSingleCheckpoint( dt, backupUrl );
        return;
    }

    QString min = items.first()->text();
    foreach( const QListWidgetItem * item, items ) {
        if( item->text() < min )
            min = item->text();
    }
    m_backupManager->revertCheckpointsTill( min, backupUrl );
}


int Nepomuk::CheckpointPage::nextId() const
{
    return -1;
}

//
// Restore Page
//
Nepomuk::RestorePage::RestorePage(QWidget* parent): QWizardPage(parent)
{
    QHBoxLayout * layout = new QHBoxLayout( this );
    setLayout( layout );

    m_identifier = new IdentifierWidget( this );
    m_merger = new MergerWidget( this );

    layout->addWidget( m_identifier );
    layout->addWidget( m_merger );

    // Page Properties
    setTitle("Restoring Backup");
    setSubTitle("The backup is being restored .. ");
    setFinalPage( true );

    m_backupManager = new BackupManager( QLatin1String("org.kde.nepomuk.services.nepomukbackupsync"),
                                         "/backupmanager",
                                         QDBusConnection::sessionBus(), this);
}


void Nepomuk::RestorePage::initializePage()
{
    QString backupUrl;
    if( field("custonBackup").toBool() )
        backupUrl = field("backupLocation").toString();
    
    m_backupManager->restore( backupUrl );
}

int Nepomuk::RestorePage::nextId() const
{
    return -1;
}

