/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "backupmanager.h"
#include "backupstorage.h"
#include "identifier.h"
#include "merger.h"
#include "backupmanageradaptor.h"
#include "logstorage.h"

#include "changelog.h"
#include "syncfile.h"
#include "identificationset.h"

#include <QtDBus/QDBusConnection>
#include <QtCore/QListIterator>

#include <KDebug>

Nepomuk::BackupManager::BackupManager(Nepomuk::Identifier* ident, Nepomuk::Merger* merger, QObject* parent)
    : QObject( parent ),
      m_identifier( ident),
      m_merger( merger)
{
    new BackupManagerAdaptor( this );
    // Register via DBUs
    QDBusConnection con = QDBusConnection::sessionBus();
    con.registerObject( QLatin1String("/backupmanager"), this );
}


Nepomuk::BackupManager::~BackupManager()
{
}


void Nepomuk::BackupManager::backup(const QString& url)
{
    kDebug();
    BackupStorage storage( url );

    if( !storage.exists() ) {
        kDebug() << "Doesn't exist - creating";
        storage.create();
        emit backupDone();
        return;
    }

    // Load all the records after the lastBackup
    QDateTime lastBackup = storage.lastBackup();
    kDebug() << "Last Backup : " << lastBackup;
    ChangeLog log = LogStorage::instance()->getChangeLog( lastBackup );

    if( !log.empty() ) {
        kDebug() << "Performing backup";
        storage.add( log );
        emit backupDone();
    }
    else {
        emit backupNotRequired( lastBackup );
        kDebug() << "The backup is upto date.";
    }
}


void Nepomuk::BackupManager::restore(const QString& url)
{
    BackupStorage storage( url );
    QList<SyncFile> syncFiles = storage.syncFiles();

    //
    // Combine all the changeLogs together. It's better to send them in one go
    // instead of doing it multiple times. If they were done one after another then the
    // system's changeLog would get modidified after each one, and that would cause problems
    // for the next one.
    //
    ChangeLog combinedChangeLog;
    foreach( const SyncFile& sf, syncFiles ) {
        combinedChangeLog << sf.changeLog();
    }

    //
    // Combine all the IdentificationSets together. This way we avoid identifying
    // the same resources multiple times. Plus we don't ask the user for the same info mulitple
    // times.
    IdentificationSet combinedIdentSet;
    foreach( const SyncFile& sf, syncFiles ) {
        // Don't worry about duplictes. The Identifier should handle them.
        combinedIdentSet << sf.identificationSet();
    }

    m_identifier->process( SyncFile( combinedChangeLog, combinedIdentSet) );
}


QStringList Nepomuk::BackupManager::checkpoints(const QString& url)
{
    BackupStorage storage( url );
    QList<QDateTime> dateTimeList = storage.syncFileDates();
    QStringList list;
    foreach( const QDateTime & dt, dateTimeList )
        list << dt.toString( ChangeLog::dateTimeFormat() );

    return list;
}


void Nepomuk::BackupManager::revertCheckpointsTill(const QString& checkPointString, const QString& url)
{
    BackupStorage storage( url );
    QDateTime checkPoint = QDateTime::fromString( checkPointString, ChangeLog::dateTimeFormat() );

    QList<QDateTime> cpList = storage.syncFileDates();
    QListIterator<QDateTime> it( cpList );
    it.toBack();

    ChangeLog combinedChangeLog;
    IdentificationSet combinedIdentSet;
    while( it.hasPrevious() ) {
        const QDateTime & dt = it.previous();

        if( dt < checkPoint )
            break;

        const SyncFile& sf = storage.syncFile( dt );

        combinedChangeLog << sf.changeLog();
        combinedIdentSet << sf.identificationSet();
    }

    m_identifier->process( SyncFile( combinedChangeLog, combinedIdentSet ) );
}


void Nepomuk::BackupManager::revertSingleCheckpoint(const QString& checkPointString, const QString& url)
{
    BackupStorage storage( url );
    QDateTime checkPoint = QDateTime::fromString( checkPointString, ChangeLog::dateTimeFormat() );

    SyncFile sf = storage.syncFile( checkPoint );
    m_identifier->process( sf );
}


void Nepomuk::BackupManager::freshBackup(const QString& url)
{
    BackupStorage storage( url );
    kDebug();
    storage.remove();
    storage.create();
    emit backupDone();
}


#include "backupmanager.moc"
