/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "propertyeditwidget.h"

#include "resourcecreateandsearchwidget.h"

#include <limits>

#include <QLabel>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QUrl>
#include <KDebug>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include <QCheckBox>
#include <QClipboard>
#include <QKeySequence>
#include <QMouseEvent>
#include <QFocusEvent>

#include <KDatePicker>
#include <KDateTimeWidget>
#include <KLineEdit>
#include <KPushButton>
#include <KLocale>
#include <KGlobal>
#include <KStandardGuiItem>
#include <KIconLoader>
#include <KApplication>
#include <KAction>
#include <KStandardAction>

#include <Nepomuk/Types/Class>
#include <Nepomuk/Types/Literal>
#include <Nepomuk/Query/ResourceTypeTerm>

class Nepomuk::PropertyEditWidget::Private
{
public:
    Private(const PropertyEditWidget * parent = 0)
        : qq( parent ),
        type( Private::UndefinedType )
    {}
    enum Type { UndefinedType, DateType, DateTimeType, StringType, IntType, DoubleType, BoolType, ResourceType };

    PropertyEditWidget const *qq;
    Nepomuk::Types::Property property;
    Nepomuk::Variant value;
    Nepomuk::Variant originalValue;
    QWidget *editor;
    bool editingEnabled;
    Private::Type type;
    bool multipleValues;

    void initResourceType();
};

Nepomuk::PropertyEditWidget::PropertyEditWidget(QWidget *parent,
        const Nepomuk::Types::Property &property,
        const Nepomuk::Variant &value) :
        QWidget(parent),
        d(new Private(this))
{
    //layout and editor
    d->editor = new QLabel(this);
    new QVBoxLayout(this);
    layout()->addWidget(d->editor);
    layout()->setContentsMargins(10, 1, 10, 1);

    d->editingEnabled = false;

    //value and type
    setNepomukProperty(property);
    setValue(value);    

    //copy actions
    KAction *copyLabel = KStandardAction::copy(this, SLOT(copyLabelToClipboard()), this);
    copyLabel->setText("Copy label");
    copyLabel->setShortcut(QKeySequence());
    addAction(copyLabel);

    KAction *copyValue = KStandardAction::copy(this, SLOT(copyValueToClipboard()), this);
    copyValue->setText("Copy value");
    copyValue->setShortcut(QKeySequence());
    addAction(copyValue);

    setContextMenuPolicy(Qt::ActionsContextMenu);
    setFocusPolicy(Qt::StrongFocus);
}

Nepomuk::PropertyEditWidget::~PropertyEditWidget()
{
    delete d;
}


/*************************** public functions **************************/

void Nepomuk::PropertyEditWidget::setNepomukProperty(const Nepomuk::Types::Property &property)
{
    if (d->property.uri() != property.uri()) { //reinitialize on property change
        d->property = property;
        d->initResourceType();
        d->value = Nepomuk::Variant();
        setEditingEnabled(false);
    }
}

void Nepomuk::PropertyEditWidget::setValue(const Nepomuk::Variant& value)
{
    if(d->value == value) {
        return;
    }

    if (value.variant().isValid()) {
        d->value = value;
        d->originalValue = value;
    }
    //setEditingEnabled(false);

    //refresh values
    if(d->editingEnabled) {
        edit();
    }
    else {
        display();
    }
}

Nepomuk::Variant Nepomuk::PropertyEditWidget::value() const
{
    return d->value;
}

Nepomuk::Types::Property Nepomuk::PropertyEditWidget::nepomukProperty() const
{
    return d->property;
}

void Nepomuk::PropertyEditWidget::setEditingEnabled(bool enabled)
{
    if (enabled) {
        edit();
    } else {
        display();
    }

    d->editingEnabled = enabled;
}

/*************************** protected events **************************/
void Nepomuk::PropertyEditWidget::mouseDoubleClickEvent(QMouseEvent *event)
{
    QWidget::mouseDoubleClickEvent(event);
    event->accept();
    setEditingEnabled(true);
}

void Nepomuk::PropertyEditWidget::focusOutEvent(QFocusEvent *event)
{    
    QWidget::focusOutEvent(event);
    event->accept();


    QWidget *widget = QApplication::focusWidget();    
    //kDebug() << "************* new focus widget:" << widget << event->reason();
    QObject *parent = widget;
    bool descendant = false;
    while(parent) {
        parent = parent->parent();
        if(parent == this) {
            descendant = true;
        }
    }
    if(descendant) { //if widget is a descendant, watch for
        //kDebug() << "************* widget is a descendant";
        widget->installEventFilter(this);
    }
    else if ( widget != 0 ) { //disable editing if the newly focused widget is not a descendant
        //kDebug() << "*************  widget is NOT a descendant";
        setEditingEnabled(false);
    }
    else { //if widget is null, focus on editor and watch for FocusOut
        //kDebug() << "*************  setting focus on editor";
        //FIXME: if someone else steals focus on a different thread (e.g. a child widget of editor), FocusOut event will probably not be activated
        d->editor->setFocus(Qt::OtherFocusReason);
        d->editor->installEventFilter(this);
    }
}

bool Nepomuk::PropertyEditWidget::eventFilter(QObject *obj, QEvent *event)
{
    if(event->type() == QEvent::FocusOut) { //if watched object loses focus, remove event filter and fire the event processing
        if(obj) {
            //kDebug() << "***** eventFilter FocusOut:" << obj;
            //WARNING: by the time we get here, this widget could've deleted obj (in the focusOutEvent method) on a different thread
            obj->removeEventFilter(this);
        }
        focusOutEvent(new QFocusEvent(QFocusEvent::FocusOut));
        return true;

    }
    //FIXME: handling Close and Hide is probably not needed
    else if( event->type() == QEvent::Close || event->type() == QEvent::Hide) { //if object is closed or hidden
            //kDebug() << "************* CLOSE or HIDE event" << obj;
            if(obj) {
                //kDebug() << "***** eventFilter close or hide:" << obj;
                //obj->removeEventFilter(this);
            }
            setFocus(Qt::OtherFocusReason);
            return true;
    }

    return false;
}


/*************************** private slots **************************/

void Nepomuk::PropertyEditWidget::display()
{
    if (d->editingEnabled == true) { //delete editor
        layout()->removeWidget(d->editor);
        delete d->editor;
        d->editor = new QLabel(this);
        layout()->addWidget(d->editor);
    }

    qobject_cast<QLabel*>(d->editor)->setText(labelsToString());
    qobject_cast<QLabel*>(d->editor)->setToolTip(valuesToString());
}


void Nepomuk::PropertyEditWidget::edit()
{
    if(d->editor) {
        d->editor->deleteLater();
    }

    if(d->multipleValues) { //multiple values - add editors with a remove button, and an add button at the bottom
        QVBoxLayout *vLayout = new QVBoxLayout(d->editor);
        vLayout->setContentsMargins(0, 0, 0, 0);//to reduce space
        vLayout->setSpacing(0);

        QList<Nepomuk::Variant> values = d->value.toVariantList();       

        foreach(Nepomuk::Variant value, values) {
            QWidget *editor;

            editor = createNewEditor(value);

            //add a REMOVE button to each editor
            QHBoxLayout *hLayout = new QHBoxLayout();
            hLayout->setContentsMargins(0, 0, 0, 0);
            KPushButton *remBtn = new KPushButton(KStandardGuiItem::remove().icon(), "");
            remBtn->setMaximumSize(KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium);
            remBtn->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
            connect(remBtn, SIGNAL(clicked()), hLayout, SLOT(deleteLater()));
            connect(remBtn, SIGNAL(clicked()), editor, SLOT(deleteLater()));
            connect(remBtn, SIGNAL(clicked()), remBtn, SLOT(deleteLater()));
            connect(hLayout, SIGNAL(destroyed()), this, SLOT(updateValue()));

            hLayout->addWidget(editor);
            hLayout->addWidget(remBtn);
            vLayout->addLayout(hLayout);
        }

        //add an ADD button at the bottom for adding new items
        KPushButton *addBtn = new KPushButton(KStandardGuiItem::add().icon(), "");        
        connect(addBtn, SIGNAL(clicked()), this, SLOT(addMultipleValueEditor()));
        addBtn->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        addBtn->setMaximumSize(KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium);
        vLayout->addWidget(addBtn);
        d->editor = new QWidget();
        d->editor->setLayout(vLayout);
    }
    else { //single value - create an editor for the value
        d->editor = createNewEditor(d->value);
    }
    layout()->addWidget(d->editor);
}

void Nepomuk::PropertyEditWidget::updateValue()
{
    if(d->multipleValues) { //multiple values - extract values from layouts
        QVBoxLayout *editorLayout = qobject_cast<QVBoxLayout*>(d->editor->layout());
        if(!editorLayout) {
            //kDebug() << "No layout on multi-value editor";
            return;
        }
        QList<Nepomuk::Variant> values;

        //retrieve values
        for(int i=0; i<editorLayout->count(); i++) {
            //kDebug() << "At item: " << i;

            QHBoxLayout *hLayout = qobject_cast<QHBoxLayout*>(editorLayout->itemAt(i)->layout());
            if(!hLayout) {
                continue;
            }
            QWidget *editorWidget = qobject_cast<QWidget*>(hLayout->itemAt(0)->widget());
            if(!editorWidget) {
                continue;
            }

            Nepomuk::Variant value = valueFromEditor(editorWidget);

            if(value.isValid()) {
                //kDebug() << "Appending: " << value;
                values.append(value);
            }
            else {
                //kDebug() << "NOT appending: " << value;
            }
        }//for
        d->value = values;
        //kDebug() << "*** d->value:" << d->value;
    }
    else {
        d->value = valueFromEditor(d->editor);
    }
}

void Nepomuk::PropertyEditWidget::addMultipleValueEditor()
{
    QVBoxLayout *editorLayout = qobject_cast<QVBoxLayout*>(d->editor->layout());
    if(!editorLayout) {
        //kDebug() << "Error: no layout on multi-value editor";
        return;
    }

    Nepomuk::Variant invalidValue;
    QWidget *editor = createNewEditor(invalidValue);

    QHBoxLayout *hLayout = new QHBoxLayout();
    hLayout->setContentsMargins(0, 0, 0, 0);

    KPushButton *remBtn = new KPushButton(KStandardGuiItem::remove().icon(), "");
    connect(remBtn, SIGNAL(clicked()), hLayout, SLOT(deleteLater()));
    connect(remBtn, SIGNAL(clicked()), editor, SLOT(deleteLater()));
    connect(remBtn, SIGNAL(clicked()), remBtn, SLOT(deleteLater()));
    connect(hLayout, SIGNAL(destroyed()), this, SLOT(updateValue()));
    remBtn->setMaximumSize(KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium);
    remBtn->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
    hLayout->addWidget(editor);
    hLayout->addWidget(remBtn);

    editorLayout->insertLayout(editorLayout->count()-1, hLayout);//add it before the Add button

    updateGeometry();
    emit geometryChanged();
}


/*************************** private utility functions **************************/

Nepomuk::Variant Nepomuk::PropertyEditWidget::valueFromEditor(QWidget *editorWidget)
{
    Nepomuk::Variant value;

    switch(d->type) {
    //Resource
    case Private::ResourceType: {
        Nepomuk::Resource resource;
        if(ResourceCreateAndSearchWidget *w = qobject_cast<ResourceCreateAndSearchWidget*>(editorWidget)) {
            resource = w->resource();
        }
        if(resource.isValid()) {
            value = Nepomuk::Variant(resource);
        }
    }
    //Literal
    case Private::DateType:
        if(KDatePicker *w = qobject_cast<KDatePicker*>(editorWidget)) {
            value = Nepomuk::Variant(QVariant(w->date()));
        }
        break;
    case Private::DateTimeType:
        if(KDateTimeWidget *w = qobject_cast<KDateTimeWidget*>(editorWidget)) {
            value = Nepomuk::Variant(QVariant(w->dateTime()));
        }
        break;
    case Private::StringType:
        if(KLineEdit *w = qobject_cast<KLineEdit*>(editorWidget)) {
            value = Nepomuk::Variant(QVariant(w->text()));
        }
        break;
    case Private::IntType:
        if(QSpinBox *w = qobject_cast<QSpinBox*>(editorWidget)) {
            value = Nepomuk::Variant(QVariant(w->value()));
        }
        break;
    case Private::DoubleType:
        if(QDoubleSpinBox *w = qobject_cast<QDoubleSpinBox*>(editorWidget)) {
            value = Nepomuk::Variant(QVariant(w->value()));
        }
        break;
    case Private::BoolType:
        if(QCheckBox *w = qobject_cast<QCheckBox*>(editorWidget)) {
            value = Nepomuk::Variant(QVariant(w->isChecked()));
        }
        break;
    default:
        break;
    }

    return value;
}

QString Nepomuk::PropertyEditWidget::labelFromValue(const Nepomuk::Variant &variant)
{
    QString label;
    //WARNING: KLocale only works within a KApplication

    if(variant.isValid()) {
        switch(d->type) {
        //non-literal
        case Private::ResourceType:
            label = variant.toResource().genericLabel();
            break;
        //literal
        case Private::DateType:
            label = KGlobal::locale()->formatDate(variant.toDate());
            break;
        case Private::DateTimeType:
            label = KGlobal::locale()->formatDateTime(variant.toDateTime());
            break;
        case Private::StringType:
            label = variant.toString();
            break;
        case Private::IntType:
            label = KGlobal::locale()->formatLong(variant.toInt());
            break;
        case Private::DoubleType:
            label = KGlobal::locale()->formatNumber(variant.toDouble());
            break;
        case Private::BoolType:
            label = variant.toBool();
            break;
        case Private::UndefinedType:
            break;
        }
    }

    return label;
}


QWidget *Nepomuk::PropertyEditWidget::createNewEditor(Nepomuk::Variant &value)
{
    QWidget *editor;

    switch(d->type) {
    //Resource
    case Private::ResourceType:{
            ResourceCreateAndSearchWidget *createAndSearchWidget = new ResourceCreateAndSearchWidget();
            createAndSearchWidget->setType(d->property.range());
            createAndSearchWidget->setResource(value.toResource());
            connect(createAndSearchWidget, SIGNAL(valueChanged()),this, SLOT(updateValue()));
            editor = createAndSearchWidget;
            break;
    }
    //Literals
    case Private::DateType: {
        KDatePicker *widget = new KDatePicker();
        widget->setDate(value.toDate());
        connect(widget, SIGNAL(dateChanged(QDate)),
                this, SLOT(updateValue()));
        editor = widget;
        break;
    }
    case Private::DateTimeType: {
        KDateTimeWidget *widget = new KDateTimeWidget();
        widget->setDateTime(value.toDateTime());
        connect(widget, SIGNAL(valueChanged(QDateTime)),
                this, SLOT(updateValue()));
        editor = widget;
        break;
    }
    case Private::StringType: {
        KLineEdit *widget = new KLineEdit();
        widget->setText(value.toString());
        connect(widget, SIGNAL(textChanged(QString)),
                this, SLOT(updateValue()));
        editor = widget;
        break;
    }
    case  Private::IntType: {
        QSpinBox *widget = new QSpinBox();
        widget->setRange(std::numeric_limits<int>::min(), std::numeric_limits<int>::max());
        widget->setValue(value.toInt());
        connect(widget, SIGNAL(valueChanged(int)),
                this, SLOT(updateValue()));
        editor = widget;
        break;
    }
    case Private::DoubleType: {
        QDoubleSpinBox *widget = new QDoubleSpinBox();
        widget->setRange(std::numeric_limits<float>::min(), std::numeric_limits<float>::max());
        widget->setValue(value.toDouble());
        connect(widget, SIGNAL(valueChanged(double)),
                this, SLOT(updateValue()));
        editor = widget;
        break;
    }
    case Private::BoolType: {
        QCheckBox *widget = new QCheckBox();
        widget->setChecked(value.toBool());
        connect(widget, SIGNAL(stateChanged(int)),
                this, SLOT(updateValue()));
        editor = widget;
        break;
    }
    case Private::UndefinedType: {
        break;
    }
    }

    return editor;
}

void Nepomuk::PropertyEditWidget::copyLabelToClipboard()
{
    KApplication::clipboard()->setText(labelsToString());
}

void Nepomuk::PropertyEditWidget::copyValueToClipboard()
{
    KApplication::clipboard()->setText(valuesToString());
}

QString Nepomuk::PropertyEditWidget::labelsToString()
{
    QString label;
    if(d->multipleValues) {//multiple values - take each value and add its label
        QList<Nepomuk::Variant> values = d->value.toVariantList();
        foreach(Nepomuk::Variant value, values) {
            label += labelFromValue(value);
            label += "\n";
        }
        label.truncate(label.size()-1);//remove trailing newline character
    }
    else {//single value - add its label
            label = labelFromValue(d->value);
    }
    return label;
}

QString Nepomuk::PropertyEditWidget::valuesToString()
{
    QString toolTip;
    if(d->multipleValues) {//multiple values - take each value and add its string representation
        QList<Nepomuk::Variant> values = d->value.toVariantList();
        foreach(Nepomuk::Variant value, values) {
            toolTip += value.toString();
            toolTip += "\n";
        }
        toolTip.truncate(toolTip.size()-1);//remove trailing newline character
    }
    else {//single value - add its string representation
            toolTip = d->value.toString();
    }
    return toolTip;
}

/*************************** Private class functions **************************/

void Nepomuk::PropertyEditWidget::Private::initResourceType()
{
    //TODO? add exceptions
    //e.g. nie:url has a resource range, but should be handled as a string

    if (property.literalRangeType().isValid()) { //Literal

        if (property.literalRangeType().dataType() == QVariant::Date) {
            type = DateType;
        }

        else if (property.literalRangeType().dataType() == QVariant::DateTime) {
            type = DateTimeType;
        }

        else if (property.literalRangeType().dataType() == QVariant::String) {
            type = StringType;
        }

        else if (property.literalRangeType().dataType() == QVariant::Int) {
            type = IntType;
        }
        
        else if (property.literalRangeType().dataType() == QVariant::Double) {
            type = DoubleType;
        }
        
        else if (property.literalRangeType().dataType() == QVariant::Bool) {
            type = BoolType;
        }

    } else if(property.range().isValid()) { //Resource
        type = ResourceType;
    } else { //Invalid
        type = UndefinedType;
        //kDebug() << "*** Undefined Type";
    }

    if(property.maxCardinality() == 1) {
        multipleValues = false;
    }
    else {
        multipleValues = true;
        //kDebug() << "*** Property range with multiple values";
    }
}
