/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>
Copyright (C) 2010 Sebastian Trueg <trueg@kde.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "resourceitemview.h"
#include "resourceitemdelegate.h"

#include <QWidget>
#include <QModelIndex>
#include <QClipboard>
#include <QKeySequence>
#include <QLayout>

#include <KDebug>
#include <KAction>
#include <KStandardAction>
#include <KApplication>
#include <KMessageBox>
#include <KLocale>

#include <Nepomuk/Resource>
#include <Nepomuk/Variant>

Nepomuk::ResourceItemView::ResourceItemView(QWidget *parent) :
        QListView(parent)
{
    connect(this, SIGNAL(activated(QModelIndex)), this, SLOT(itemActivated(QModelIndex)));
    setSelectionMode(QAbstractItemView::SingleSelection);
    setSelectionBehavior(QAbstractItemView::SelectRows);
    setEditTriggers(QAbstractItemView::NoEditTriggers);//editing handled by itemActivated
    this->setVerticalScrollMode(QListView::ScrollPerPixel);//otherwise it won't smoothly scroll over the editor widget

    //copy actions
    KAction *copyLabel = KStandardAction::copy(this, SLOT(copyLabelToClipboard()), this);
    copyLabel->setText(i18n("Copy label"));
    copyLabel->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_C));
    addAction(copyLabel);

    KAction *copyUri = KStandardAction::copy(this, SLOT(copyUriToClipboard()), this);
    copyUri->setText(i18n("Copy URI"));
    copyUri->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_C));
    addAction(copyUri);

    KAction *deleteResource = KStandardAction::close(this, SLOT(deleteResource()), this);
    deleteResource->setText(i18n("Delete resource"));
    deleteResource->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_D));
    addAction(deleteResource);

    this->setContextMenuPolicy(Qt::ActionsContextMenu);
}

void Nepomuk::ResourceItemView::itemActivated(const QModelIndex &index)
{
    if (index.isValid()) {
        ResourceItemDelegate *delegate = qobject_cast<ResourceItemDelegate*>(itemDelegate());
        if (delegate->isExtended(index)) {
            delegate->contractItem(index);
        } else {
            QWidget *editor = delegate->createEditor(this, QStyleOptionViewItem(), index);
            delegate->extendItem(editor, index);
            //editor->updateGeometry();
            //refreshGeometries();
            //FIXME: if the editor's geometry is changed, the view is NOT updated
            //To reproduce: 1) extend and start editing an item above another one that is extended, 2) add new resource to a multiple valued property 3) sometimes event by just expanding an item
        }
    }
}

void Nepomuk::ResourceItemView::reset()
{
    ResourceItemDelegate *delegate = qobject_cast<ResourceItemDelegate*>(itemDelegate());

    for (int i = 0; i < model()->rowCount(); i++) {
        delegate->contractItem(model()->index(i, 0));
    }

    QListView::reset();
}

void Nepomuk::ResourceItemView::refreshGeometries()
{
    //updateGeometries();
    setRootIndex(rootIndex());

    //ResourceItemDelegate *delegate = qobject_cast<ResourceItemDelegate*>(itemDelegate());
    //delegate->emitGeometryChanged();
    doItemsLayout();

    for (int i = 0; i < model()->rowCount(); i++) {
        update(model()->index(i, 0));

    }
}

void Nepomuk::ResourceItemView::copyLabelToClipboard()
{
    KApplication::clipboard()->setText(model()->data(currentIndex(), Qt::DisplayRole).toString());
}

void Nepomuk::ResourceItemView::copyUriToClipboard()
{
    KApplication::clipboard()->setText(model()->data(currentIndex(), ResourceItemDelegate::ResourceUriRole).toString());
}

void Nepomuk::ResourceItemView::deleteResource()
{
    Nepomuk::Resource resource = Nepomuk::Variant(model()->data(currentIndex(), Qt::EditRole)).toResource();

    if(KMessageBox::warningContinueCancel(this,
                                          "This will <b>permanently</b> remove the resource from the system. Are you sure?")
        == KMessageBox::Continue) {
        model()->removeRow(currentIndex().row());
        resource.remove();
    }
}
