/*
   Copyright (c) 2010 Sebastian Trueg <trueg@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QtCore/QCoreApplication>
#include <QtCore/QHash>
#include <QtCore/QUrl>
#include <QtCore/QSet>

#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <KDE/KLocale>
#include <KComponentData>
#include <KUrl>

#include <Nepomuk/Resource>
#include <Nepomuk/Variant>
#include <Nepomuk/Types/Property>
#include <Nepomuk/Types/Class>
#include <Nepomuk/ResourceManager>

namespace {
    QStringList variantToN3( const Nepomuk::Variant& variant )
    {
        QStringList sl;
        if ( variant.isList() ) {
            foreach( const Nepomuk::Variant& v, variant.toVariantList() ) {
                sl << variantToN3( v );
            }
        }
        else {
            if ( variant.isResource() ||
                 variant.isUrl() ) {
                sl << Soprano::Node::resourceToN3( variant.toUrl() );
            }
            else {
                sl << Soprano::Node::literalToN3( Soprano::LiteralValue( variant.variant() ) );
            }
        }
        return sl;
    }

    QList<Nepomuk::Resource> extractExistingResources( const Nepomuk::Variant& variant )
    {
        QList<Nepomuk::Resource> rl;
        if ( variant.isList() ) {
            foreach( const Nepomuk::Variant& v, variant.toVariantList() ) {
                rl += extractExistingResources( v );
            }
        }
        else {
            if ( variant.isResource() ||
                 variant.isUrl() ) {
                rl << variant.toResource();
            }
        }
        return rl;
    }

    QStringList typesToString( const QList<QUrl>& types, bool hr )
    {
        QStringList tl;
        foreach( const QUrl& type, types ) {
            if ( hr )
                tl << Nepomuk::Types::Class( type ).label();
            else
                tl << Soprano::Node::resourceToN3( type );
        }
        return tl;
    }

    void printResourceHeader( const Nepomuk::Resource& res, bool hr )
    {
        QTextStream std( stdout );
        std << "Resource: " << Soprano::Node::resourceToN3( res.resourceUri() ) << endl;
        QStringList types = typesToString( res.types(), hr );
        std << "Types:    " << types[0] << endl;
        if ( types.count() > 1 ) {
            for ( int i = 1; i < types.count(); ++i ) {
                std << "          " << types[i] << endl;
            }
        }
    }

    void printResource( const Nepomuk::Resource& res, bool hr, int depth )
    {
        QTextStream std( stdout );

        // gather table data
        QHash<QUrl, Nepomuk::Variant> props = res.properties();
        QSet<Nepomuk::Resource> children;
        QStringList propLabels;
        QList<QStringList> vals;
        int propLen = 0;
        for ( QHash<QUrl, Nepomuk::Variant>::ConstIterator it = props.constBegin();
              it != props.constEnd(); ++it ) {
            if ( hr ) {
                propLabels << Nepomuk::Types::Property( it.key() ).label();
                vals << QStringList( it.value().toString() );
            }
            else {
                propLabels << Soprano::Node::resourceToN3( it.key() );
                vals << variantToN3( it.value() );
            }
            propLen = qMax( propLen, propLabels.last().length() );

            children += extractExistingResources( it.value() ).toSet();
        }

        // space between label and value
        propLen += 1;

        // output
        printResourceHeader( res, hr );
        for ( int i = 0; i < propLabels.count(); ++i ) {
            const QString label = propLabels[i];
            const QStringList val = vals[i];
            QString spacer;
            spacer.fill( ' ', propLen - label.length() );
            std << label << spacer << val[0] << endl;
            if ( val.count() > 1 ) {
                spacer.fill( ' ', propLen );
                for ( int j = 1; j < val.count(); ++j ) {
                    std << spacer << val[j] << endl;
                }
            }
        }

        // print children
        if ( depth > 0 ) {
            foreach( const Nepomuk::Resource& child, children ) {
                std << endl;
                printResource( child, hr, depth-1 );
            }
        }
    }
}

int main( int argc, char *argv[] )
{
    KAboutData aboutData( "dumpres",
                          "dumpres",
                          ki18n("Nepomuk DumpRes"),
                          "0.1",
                          ki18n("The Nepomuk Resource Dumper - A debugging tool"),
                          KAboutData::License_GPL,
                          ki18n("(c) 2010, Sebastian Trg"),
                          KLocalizedString(),
                          "http://nepomuk.kde.org" );
    aboutData.addAuthor(ki18n("Sebastian Trg"),ki18n("Maintainer"), "trueg@kde.org");
    aboutData.setProgramIconName( "nepomuk" );

    KCmdLineArgs::init( argc, argv, &aboutData );

    KCmdLineOptions options;
	options.add("h", ki18n("Use a more human readable output form."));
	options.add("depth <depth>", ki18n("How many child resources should be printed."), "0");
	options.add("+resource", ki18n("The resource URI, a file URL, or a file path."));
    KCmdLineArgs::addCmdLineOptions( options );

    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();

    QCoreApplication app( argc, argv );
    KComponentData comp( aboutData );

    Nepomuk::ResourceManager::instance()->init();

    QTextStream err( stdout );
    bool hr = args->isSet( "h" );
    int depth = args->getOption( "depth" ).toInt();
    Nepomuk::Resource res( args->url( 0 ) );
    if ( !res.exists() ) {
        err << "Resource does not exist: " << args->url( 0 ).url() << endl;
        return 1;
    }
    else {
        printResource( res, hr, depth );
        return 0;
    }
}
