/***************************************************************************
 *   Copyright (C) 2010 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *   Copyright (C) 2007 by Robert Knight <robertknight@gmail.com>          *
 *   Copyright (C) 2008 by Alexis Ménard <darktears31@gmail.com>           *
 *   Copyright (C) 2008 by Marco Martin <notmart@gmail.com>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "tasks_view_item.h"

#include "context_menu_p.h"
#include "tasks_roles.h"
#include "dataengines_tasks.h"

#include <KIconEffect>
#include <KIconLoader>
#include <KGlobalSettings>
#include <KColorUtils>
#include <Plasma/FrameSvg>
#include <Plasma/PaintUtils>
#include <Plasma/Service>

#include <QMenu>
#include <QGraphicsSceneContextMenuEvent>
#include <QPainter>
#include <QStyleOptionGraphicsItem>
#include <QTextLayout>
#include <QPropertyAnimation>
#include <QGraphicsScene>
#include <QGraphicsView>

TasksViewItem::TasksViewItem(const QPersistentModelIndex &modelIndex,
                             Plasma::DataEngine *tasksEngine,
                             Plasma::DataEngine *statusNotifierItemEngine,
                             DataEnginesTasks *applet,
                             QGraphicsItem *parent)
    : QGraphicsWidget(parent), m_currentTaskFlags(0), m_applet(applet),
      m_tasksEngine(tasksEngine), m_statusNotifierItemEngine(statusNotifierItemEngine),
      m_modelIndex(modelIndex), m_activateAction(NULL),
      m_clickStartPos(), m_cachedShadow(), m_alpha(1), m_oldBackgroundPrefix(),
      m_backgroundPrefix("normal"), m_activeRect(), m_lastGeometryUpdate(),
      m_lastUpdate(), m_activateTimerId(0), m_updateGeometryTimerId(0),
      m_updateTimerId(0), m_attentionTimerId(0), m_attentionTicks(0),
      m_layoutAnimation(NULL), m_backgroundFadeAnim(NULL), m_lastViewId(),
      m_showText(true), m_layoutAnimationLock(false), m_firstGeometryUpdate(false)
{
    Q_ASSERT(m_modelIndex.isValid());

    setContentsMargins(2, 2, 2, 2);

    m_layoutAnimation = new QPropertyAnimation(this, "animationPos", this);
    m_layoutAnimation->setEasingCurve(QEasingCurve::InOutQuad);
    m_layoutAnimation->setDuration(250);

    setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
    setAcceptsHoverEvents(true);
    setFocusPolicy(Qt::StrongFocus);

    dataChanged();

    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(syncActiveRect()));
    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(queueUpdate()));
}

void TasksViewItem::triggerAction(const QVariantMap &actionDescription, const QPoint &pos)
{
    QVariantMap activateExtraParameters;
    activateExtraParameters["x"] = pos.x();
    activateExtraParameters["y"] = pos.y();

    QVariantMap actionParameters = actionDescription;
    actionParameters["extraOperationParameters"] = activateExtraParameters;

    Plasma::Service *taskService = NULL;
    const QVariant &taskSourceName(m_modelIndex.data(TasksModel::TaskSourceName));
    if (taskSourceName.canConvert<QString>()) {
        taskService = m_tasksEngine->serviceForSource(m_modelIndex.data(TasksModel::TaskSourceName).toString());
        kDebug() << "taskService" << taskService->name();
    }

    Plasma::Service *statusItemService = NULL;
    const QVariant &statusItemSourceNames(m_modelIndex.data(TasksModel::StatusItemSourceNames));
    if (statusItemSourceNames.canConvert<QList<QVariant> >() && !statusItemSourceNames.toList().empty()) {
        statusItemService = m_statusNotifierItemEngine->serviceForSource(statusItemSourceNames.toList().first().toString());
        kDebug() << "statusItemService" << statusItemService->name();
    }

    if (m_activateAction) {
        delete m_activateAction;
        m_activateAction = NULL;
    }
    m_activateAction = new ServiceAction(this, taskService, statusItemService, actionParameters);
    m_activateAction->trigger();
}

void TasksViewItem::updateModelIndex(const QPersistentModelIndex &modelIndex)
{
    m_modelIndex = modelIndex;
    if (m_modelIndex.isValid()) {
        dataChanged();
    }
}

void TasksViewItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
    Q_ASSERT(m_modelIndex.isValid());

    const QVariant &contextMenuData(m_modelIndex.data(TasksModel::ContextMenuRole));
    if (contextMenuData.canConvert<QList<QVariant> >()) {
        QMenu *myContextMenu = new ContextMenu(m_modelIndex.data(Qt::DisplayRole).toString(),
                                               m_modelIndex.data(Qt::DecorationRole).value<QIcon>(),
                                               m_modelIndex.data(TasksModel::ContextMenuRole).toList(),
                                               m_modelIndex, m_tasksEngine, m_statusNotifierItemEngine, NULL);
        myContextMenu->exec(event->screenPos());
        delete myContextMenu;
    } else if (contextMenuData.canConvert<QVariantMap>()) {
        triggerAction(contextMenuData.toMap(), event->screenPos());
    }
}

const QPersistentModelIndex& TasksViewItem::getIndex() const
{
    return m_modelIndex;
}

void TasksViewItem::dataChanged()
{
    Q_ASSERT(m_modelIndex.isValid());
    updateTaskFlags();
    queueUpdate();
}

void TasksViewItem::updateTaskFlags()
{
    if (((m_currentTaskFlags & TaskWantsAttention) != 0) != m_modelIndex.data(TasksModel::IsDemandingAttention).toBool()) {
        if (m_modelIndex.data(TasksModel::IsDemandingAttention).toBool()) {
            m_applet->needsVisualFocus();
            // start attention getting
            if (!m_attentionTimerId) {
                m_attentionTimerId = startTimer(500);
            }
        } else if (m_attentionTimerId) {
            killTimer(m_attentionTimerId);
            m_attentionTimerId = 0;
            // stop attention getting
        }
    }

    if (m_modelIndex.data(TasksModel::IsDemandingAttention).toBool()) {
        m_currentTaskFlags |= TaskWantsAttention;
    } else {
        m_currentTaskFlags &= ~TaskWantsAttention;
    }

    QString newBackground;
    if (m_modelIndex.data(TasksModel::IsMinimizedRole).toBool()) {
        m_currentTaskFlags |= TaskIsMinimized;
        m_currentTaskFlags &= ~TaskHasFocus;
        newBackground = "minimized";
    } else {
        m_currentTaskFlags &= ~TaskIsMinimized;
        if (m_modelIndex.data(TasksModel::IsActive).toBool()) {
            m_currentTaskFlags |= TaskHasFocus;
            newBackground = "focus";
        } else {
            m_currentTaskFlags &= ~TaskHasFocus;
            newBackground = "normal";
        }
    }

    if (newBackground != m_backgroundPrefix) {
        fadeBackground(newBackground, 100);
    }
}

void TasksViewItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    if (m_modelIndex.isValid()) {
        painter->setRenderHint(QPainter::Antialiasing);

        // draw background
        drawBackground(painter, option, widget);

        // draw icon and text
        drawTask(painter, option, widget);
    }
}

void TasksViewItem::drawBackground(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
{
    // Do not paint with invalid sizes, the happens when the layout is being initialized
    if (!option->rect.isValid()) {
        return;
    }

    /*FIXME -could be done more elegant with caching in tasks in a qhash <size,svg>.
    -do not use size() directly because this introduces the blackline syndrome.
    -This line is only needed when we have different items in the taskbar because of an expanded group for example. otherwise the resizing in the resizeEvent is sufficient
    */
    Plasma::FrameSvg *itemBackground = m_applet->itemBackground();

    //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
    if (size().toSize() != itemBackground->frameSize()) {
        resizeBackground(size().toSize());
    }

    if (~option->state & QStyle::State_Sunken && 
        (!m_backgroundFadeAnim || m_backgroundFadeAnim->state() != QAbstractAnimation::Running)) {
        itemBackground->setElementPrefix(m_backgroundPrefix);
        if (itemBackground->frameSize() == m_activeRect.size().toSize()) {
            itemBackground->paintFrame(painter, m_activeRect.topLeft());
        } else {
            itemBackground->paintFrame(painter);
        }
        //itemBackground->paintFrame(painter, backgroundPosition);
        return;
    }

    itemBackground->setElementPrefix(m_oldBackgroundPrefix);
    QPixmap oldBackground;

    if (option->state & QStyle::State_Sunken) {
        oldBackground = QPixmap(m_activeRect.size().toSize());
        oldBackground.fill(Qt::transparent);
        m_alpha = 0.4;
    } else {
        oldBackground = itemBackground->framePixmap();
    }

    itemBackground->setElementPrefix(m_backgroundPrefix);
    QPixmap result = Plasma::PaintUtils::transition(oldBackground, itemBackground->framePixmap(), m_alpha);

    if (result.size() == m_activeRect.size().toSize()) {
        painter->drawPixmap(m_activeRect.topLeft(), result);
    } else {
        painter->drawPixmap(QPoint(0,0), result);
    }

}

void TasksViewItem::drawTask(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
{
    Q_UNUSED(option)

    QRectF bounds = boundingRect().adjusted(m_applet->itemLeftMargin(), m_applet->itemTopMargin(), -m_applet->itemRightMargin(), -m_applet->itemBottomMargin());

    QGraphicsWidget *busyWidget = NULL;
    const QRectF iconR = iconRect(bounds);

    if (busyWidget) {
        busyWidget->setGeometry(iconR);
        busyWidget->show();
    } else {
        const bool fadingBg = m_backgroundFadeAnim && m_backgroundFadeAnim->state() == QAbstractAnimation::Running;
        if ((!fadingBg && !(option->state & QStyle::State_MouseOver)) ||
            (m_oldBackgroundPrefix != "hover" && m_backgroundPrefix != "hover")) {
            m_modelIndex.data(Qt::DecorationRole).value<QIcon>().paint(painter, iconR.toRect());
        } else {
            KIconEffect *effect = KIconLoader::global()->iconEffect();
            QPixmap result = m_modelIndex.data(Qt::DecorationRole).value<QIcon>().pixmap(iconR.toRect().size());

            if (effect->hasEffect(KIconLoader::Desktop, KIconLoader::ActiveState)) {
                if (qFuzzyCompare(qreal(1.0), m_alpha)) {
                    result = effect->apply(result, KIconLoader::Desktop, KIconLoader::ActiveState);
                } else {
                    result = Plasma::PaintUtils::transition(result,
                                        effect->apply(result, KIconLoader::Desktop,
                                        KIconLoader::ActiveState), m_backgroundPrefix != "hover" ? 1 - m_alpha : m_alpha);
                }
            }

            painter->drawPixmap(iconR.topLeft(), result);
        }
    }

    painter->setPen(QPen(textColor(), 1.0));

    if (m_showText) {
        QRect rect = textRect(bounds).toRect();
        if (rect.height() > 20) {
            rect.adjust(2, 2, -2, -2); // Create a text margin
        }
        QTextLayout layout;
        layout.setFont(KGlobalSettings::taskbarFont());
        layout.setTextOption(textOption());

        layoutText(layout, m_modelIndex.data(Qt::DisplayRole).toString(), rect.size());
        drawTextLayout(painter, layout, rect);
    }

    if (false) {
        QFont font(KGlobalSettings::smallestReadableFont());
        QFontMetrics fm(font);
        QRectF rect(expanderRect(bounds));

        Plasma::FrameSvg *itemBackground = m_applet->itemBackground();

        if (itemBackground->hasElement(expanderElement())) {
            QSizeF arrowSize(itemBackground->elementSize(expanderElement()));
            QRectF arrowRect(rect.center()-QPointF(arrowSize.width()/2, arrowSize.height()+fm.xHeight()/2), arrowSize);
            itemBackground->paint(painter, arrowRect, expanderElement());

            painter->setFont(font);
            rect.setTop(arrowRect.bottom());
            painter->drawText(rect, Qt::AlignHCenter|Qt::AlignTop, QString::number(m_modelIndex.model()->rowCount(m_modelIndex)));
        } else {
            painter->setFont(font);
            painter->drawText(rect, Qt::AlignCenter, QString::number(m_modelIndex.model()->rowCount(m_modelIndex)));
        }
    }
}

void TasksViewItem::resizeBackground(const QSize &size)
{
    Plasma::FrameSvg *itemBackground = m_applet->itemBackground();

    itemBackground->setElementPrefix("focus");
    m_applet->resizeItemBackground(size);
    itemBackground->setElementPrefix("normal");
    m_applet->resizeItemBackground(size);
    itemBackground->setElementPrefix("minimized");
    m_applet->resizeItemBackground(size);
    itemBackground->setElementPrefix("attention");
    m_applet->resizeItemBackground(size);
    itemBackground->setElementPrefix("hover");
    m_applet->resizeItemBackground(size);

    //restore the prefix
    itemBackground->setElementPrefix(m_backgroundPrefix);
}

QRectF TasksViewItem::iconRect(const QRectF &b) const
{
    QRectF bounds(b);
    const int right = bounds.right();

    if (m_showText) {
        //leave enough space for the text. useful in vertical panel
        bounds.setWidth(qMax(bounds.width() / 3, qMin(minimumSize().height(), bounds.width())));
    }

    //restore right position if the layout is RTL
    if (QApplication::layoutDirection() == Qt::RightToLeft) {
        bounds.moveRight(right);
    }

    QSize iconSize = m_modelIndex.data(Qt::DecorationRole).value<QIcon>().actualSize(bounds.size().toSize());

    if (iconSize.width() == iconSize.height()) {
        if (iconSize.width() > KIconLoader::SizeSmall && iconSize.width() < KIconLoader::SizeSmallMedium) {
            iconSize = QSize(KIconLoader::SizeSmall, KIconLoader::SizeSmall);
        } else if (iconSize.width() > KIconLoader::SizeSmallMedium && iconSize.width() < KIconLoader::SizeMedium) {
            iconSize = QSize(KIconLoader::SizeSmallMedium, KIconLoader::SizeSmallMedium);
        } else if (iconSize.width() > KIconLoader::SizeMedium && iconSize.width() < KIconLoader::SizeLarge) {
            iconSize = QSize(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
        }
    }

    return QStyle::alignedRect(QApplication::layoutDirection(),
                               (m_showText ? Qt::AlignLeft : Qt::AlignCenter) | Qt::AlignVCenter,
                               iconSize, bounds.toRect());
}

QColor TasksViewItem::textColor() const
{
    QColor color;
    qreal bias;
    Plasma::Theme *theme = Plasma::Theme::defaultTheme();

    if ((m_oldBackgroundPrefix == "attention" || m_backgroundPrefix == "attention") &&
         m_applet->itemBackground()->hasElement("hint-attention-button-color")) {
        bool animatingBg = m_backgroundFadeAnim && m_backgroundFadeAnim->state() == QAbstractAnimation::Running;
        if (animatingBg) {
            if (m_oldBackgroundPrefix == "attention") {
                bias = 1 - m_alpha;
            } else {
                bias = m_alpha;
            }

            color = KColorUtils::mix(theme->color(Plasma::Theme::TextColor),
                                     theme->color(Plasma::Theme::ButtonTextColor), bias);
        } else if (m_backgroundPrefix != "attention") {
                color = theme->color(Plasma::Theme::TextColor);
        } else {
            color = theme->color(Plasma::Theme::ButtonTextColor);
        }
    } else {
        color = theme->color(Plasma::Theme::TextColor);
    }

    if (m_modelIndex.data(TasksModel::IsMinimizedRole).toBool()) {
        color.setAlphaF(0.85);
    }

    return color;
}

QRectF TasksViewItem::textRect(const QRectF &bounds) const
{
    QSize size(bounds.size().toSize());
    QRectF effectiveBounds(bounds);

    size.rwidth() -= int(iconRect(bounds).width()) + qMax(0, IconTextSpacing - 2);
    if (false) {
        size.rwidth() -= int(expanderRect(bounds).width()) + qMax(0, IconTextSpacing - 2);

        if (QApplication::layoutDirection() == Qt::RightToLeft) {
            effectiveBounds.setLeft(expanderRect(bounds).right());
        } else {
            effectiveBounds.setRight(expanderRect(bounds).left());
        }
    }

    return QStyle::alignedRect(QApplication::layoutDirection(), Qt::AlignRight | Qt::AlignVCenter,
                                     size, effectiveBounds.toRect());
}

QTextOption TasksViewItem::textOption() const
{
    Qt::LayoutDirection direction = QApplication::layoutDirection();
    Qt::Alignment alignment = QStyle::visualAlignment(direction, Qt::AlignLeft | Qt::AlignVCenter);

    QTextOption option;
    option.setTextDirection(direction);
    option.setAlignment(alignment);

    return option;
}

QSize TasksViewItem::layoutText(QTextLayout &layout, const QString &text,
                                const QSize &constraints) const
{
    QFontMetrics metrics(layout.font());
    int leading     = metrics.leading();
    int height      = 0;
    int maxWidth    = constraints.width();
    int widthUsed   = 0;
    int lineSpacing = metrics.lineSpacing();
    QTextLine line;

    layout.setText(text);

    layout.beginLayout();
    while ((line = layout.createLine()).isValid()) {
        height += leading;

        // Make the last line that will fit infinitely long.
        // drawTextLayout() will handle this by fading the line out
        // if it won't fit in the constraints.
        if (height + 2 * lineSpacing > constraints.height()) {
            line.setPosition(QPoint(0, height));
            break;
        }

        line.setLineWidth(maxWidth);
        line.setPosition(QPoint(0, height));

        height += int(line.height());
        widthUsed = int(qMax(qreal(widthUsed), line.naturalTextWidth()));
    }
    layout.endLayout();

    return QSize(widthUsed, height);
}

void TasksViewItem::drawTextLayout(QPainter *painter, const QTextLayout &layout, const QRect &rect)
{
    if (rect.width() < 1 || rect.height() < 1) {
        return;
    }

    QPixmap pixmap(rect.size());
    pixmap.fill(Qt::transparent);

    QPainter p(&pixmap);
    p.setPen(painter->pen());

    // Create the alpha gradient for the fade out effect
    QLinearGradient alphaGradient(0, 0, 1, 0);
    alphaGradient.setCoordinateMode(QGradient::ObjectBoundingMode);
    if (layout.textOption().textDirection() == Qt::LeftToRight)
    {
        alphaGradient.setColorAt(0, QColor(0, 0, 0, 255));
        alphaGradient.setColorAt(1, QColor(0, 0, 0, 0));
    } else
    {
        alphaGradient.setColorAt(0, QColor(0, 0, 0, 0));
        alphaGradient.setColorAt(1, QColor(0, 0, 0, 255));
    }

    QFontMetrics fm(layout.font());
    int textHeight = layout.lineCount() * fm.lineSpacing();

    QPointF position(0, (rect.height() - textHeight) / 2 + (fm.tightBoundingRect("M").height() - fm.xHeight())/2);
    QList<QRect> fadeRects;
    int fadeWidth = 30;

    // Draw each line in the layout
    for (int i = 0; i < layout.lineCount(); i++)
    {
        QTextLine line = layout.lineAt(i);
        line.draw(&p, position);

        // Add a fade out rect to the list if the line is too long
        if (line.naturalTextWidth() > rect.width())
        {
            int x = int(qMin(line.naturalTextWidth(), (qreal)pixmap.width())) - fadeWidth;
            int y = int(line.position().y() + position.y());
            QRect r = QStyle::visualRect(layout.textOption().textDirection(), pixmap.rect(),
                                         QRect(x, y, fadeWidth, int(line.height())));
            fadeRects.append(r);
        }
    }

    // Reduce the alpha in each fade out rect using the alpha gradient
    if (!fadeRects.isEmpty())
    {
        p.setCompositionMode(QPainter::CompositionMode_DestinationIn);
        foreach (const QRect &rect, fadeRects) {
            p.fillRect(rect, alphaGradient);
        }
    }

    p.end();


    QColor shadowColor;
    if (qGray(textColor().rgb()) > 192) {
        shadowColor = Qt::black;
    } else {
        shadowColor = Qt::white;
    }

    if (m_cachedShadow.isNull()) {
        QImage shadow = pixmap.toImage();
        Plasma::PaintUtils::shadowBlur(shadow, 2, shadowColor);
        m_cachedShadow = QPixmap(shadow.size());
        m_cachedShadow.fill(Qt::transparent);
        QPainter buffPainter(&m_cachedShadow);
        buffPainter.drawImage(QPoint(0,0), shadow);
    }

    if (shadowColor == Qt::white) {
        painter->drawPixmap(rect.topLeft(), m_cachedShadow);
    } else {
        painter->drawPixmap(rect.topLeft() + QPoint(1,2), m_cachedShadow);
    }
    painter->drawPixmap(rect.topLeft(), pixmap);
}

QRectF TasksViewItem::expanderRect(const QRectF &bounds) const
{
    if (false) {
        return QRectF();
    }

    QFontMetrics fm(KGlobalSettings::smallestReadableFont());
    Plasma::FrameSvg *itemBackground = m_applet->itemBackground();

    QSize expanderSize(qMax(fm.width(QString::number(m_modelIndex.model()->rowCount(m_modelIndex))),
                       itemBackground->elementSize(expanderElement()).width()),
                       size().height());

    return QStyle::alignedRect(QApplication::layoutDirection(), Qt::AlignRight | Qt::AlignVCenter,
                               expanderSize, bounds.toRect());
}

QString TasksViewItem::expanderElement() const
{
    switch (m_applet->location()) {
    case Plasma::TopEdge:
        return "group-expander-top";
    case Plasma::RightEdge:
        return "group-expander-right";
    case Plasma::LeftEdge:
        return "group-expander-left";
    case Plasma::BottomEdge:
    default:
        return "group-expander-bottom";
    }
}

void TasksViewItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
    Q_UNUSED(event)
    fadeBackground("hover", 175);
}

void TasksViewItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
    Q_UNUSED(event)

    QString backgroundPrefix;
    if (m_modelIndex.data(TasksModel::IsDemandingAttention).toBool()) {
        backgroundPrefix = "attention";
    } else if (m_modelIndex.data(TasksModel::IsMinimizedRole).toBool()) {
        backgroundPrefix = "minimized";
    } else {
        backgroundPrefix = "normal";
    }

    fadeBackground(backgroundPrefix, 150);
}

void TasksViewItem::queueUpdate()
{
    if (m_updateTimerId || m_attentionTimerId) {
        return;
    }

    if (m_lastUpdate.elapsed() < 200) {
        m_updateTimerId = startTimer(200);
        return;
    }

    update();
    m_lastUpdate.restart();
}

void TasksViewItem::fadeBackground(const QString &newBackground, int duration)
{
    Q_ASSERT(m_modelIndex.isValid());
    if (false) {
        return;
    }

    m_oldBackgroundPrefix = m_backgroundPrefix;
    m_backgroundPrefix = newBackground;

    if (m_oldBackgroundPrefix.isEmpty()) {
        update();
    } else {
        if (!m_backgroundFadeAnim) {
            m_backgroundFadeAnim = new QPropertyAnimation(this);
            m_backgroundFadeAnim->setDuration(duration);
            m_backgroundFadeAnim->setEasingCurve(QEasingCurve::InQuad);
            m_backgroundFadeAnim->setPropertyName("backgroundFadeAlpha");
            m_backgroundFadeAnim->setTargetObject(this);
            m_backgroundFadeAnim->setStartValue(0);
            m_backgroundFadeAnim->setEndValue(1);
        }

        m_backgroundFadeAnim->start();
    }
}

void TasksViewItem::timerEvent(QTimerEvent *event)
{
    if (event->timerId() == m_activateTimerId) {
        killTimer(m_activateTimerId);
        m_activateTimerId = 0;
        if (!m_modelIndex.data(TasksModel::IsActive).toBool()) {
            triggerAction(m_modelIndex.data(TasksModel::ActivateActionRole).toMap(), QPoint());
        }
    } else if (event->timerId() == m_updateGeometryTimerId) {
        killTimer(m_updateGeometryTimerId);
        m_updateGeometryTimerId = 0;
        m_firstGeometryUpdate = true;
        publishIconGeometry();
    } else if (event->timerId() == m_updateTimerId) {
        killTimer(m_updateTimerId);
        m_updateTimerId = 0;
        update();
    } else if (event->timerId() == m_attentionTimerId) {
        ++m_attentionTicks;
        if (m_attentionTicks > 6) {
            killTimer(m_attentionTimerId);
            m_attentionTimerId = 0;
            m_attentionTicks = 0;
        }

        if (m_attentionTicks % 2 == 0) {
            fadeBackground("attention", 100);
        } else {
            fadeBackground("normal", 150);
        }
    } else {
        QGraphicsWidget::timerEvent(event);
    }
}

void TasksViewItem::publishIconGeometry() const
{
    publishIconGeometry(iconGeometry());
}

void TasksViewItem::publishIconGeometry(const QRect &rect) const
{
    Q_ASSERT(m_modelIndex.isValid());
    Plasma::Service *theService = m_tasksEngine->serviceForSource(m_modelIndex.data(TasksModel::TaskSourceName).toString());
    KConfigGroup params = theService->operationDescription("publishIconGeometry");
    params.writeEntry("geometry", rect);
    theService->startOperationCall(params);
}

QRect TasksViewItem::iconGeometry() const
{
    if (!scene() || !boundingRect().isValid()) {
        return QRect();
    }

    QGraphicsView *parentView = 0;
    QGraphicsView *possibleParentView = 0;
    // The following was taken from Plasma::Applet, it doesn't make sense to make the item an applet, and this was the easiest way around it.
    foreach (QGraphicsView *view, scene()->views()) {
        if (view->sceneRect().intersects(sceneBoundingRect()) ||
            view->sceneRect().contains(scenePos())) {
            if (view->isActiveWindow()) {
                parentView = view;
                break;
            } else {
                possibleParentView = view;
            }
        }
    }

    if (!parentView) {
        parentView = possibleParentView;

        if (!parentView) {
            return QRect();
        }
    }

    QRect rect = parentView->mapFromScene(mapToScene(boundingRect())).boundingRect().adjusted(0, 0, 1, 1);
    rect.moveTopLeft(parentView->mapToGlobal(rect.topLeft()));
    return rect;
}

qreal TasksViewItem::backgroundFadeAlpha() const
{
    return m_alpha;
}

void TasksViewItem::setBackgroundFadeAlpha(qreal progress)
{
    m_alpha = progress;
    update();
}

void TasksViewItem::setAnimationPos(const QPointF &pos)
{
    m_layoutAnimationLock = true;
    setPos(pos);
    m_layoutAnimationLock = false;
}

QPointF TasksViewItem::animationPos() const
{
    return pos();
}

void TasksViewItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {
        triggerAction(m_modelIndex.data(TasksModel::ActivateActionRole).toMap(), event->screenPos());
    } else if (event->button() == Qt::MidButton) {
        const QVariant &secondaryActivateAction(m_modelIndex.data(TasksModel::SecondaryActivateActionRole));
        if (!secondaryActivateAction.isNull() && secondaryActivateAction.canConvert<QVariantMap>()) {
            triggerAction(secondaryActivateAction.toMap(), event->screenPos());
        }
    }
}

void TasksViewItem::mousePressEvent(QGraphicsSceneMouseEvent *)
{
    update();
}

void TasksViewItem::syncActiveRect()
{
    Plasma::FrameSvg *itemBackground = m_applet->itemBackground();
    itemBackground->setElementPrefix("normal");

    qreal left, top, right, bottom;
    itemBackground->getMargins(left, top, right, bottom);

    itemBackground->setElementPrefix("focus");
    qreal activeLeft, activeTop, activeRight, activeBottom;
    itemBackground->getMargins(activeLeft, activeTop, activeRight, activeBottom);

    m_activeRect = QRectF(QPointF(0, 0), size());
    m_activeRect.adjust(left - activeLeft, top - activeTop,
                        -(right - activeRight), -(bottom - activeBottom));

    itemBackground->setElementPrefix(m_backgroundPrefix);

    // check to see if there is enough room!
    QFontMetrics fm(font());
    const int minimumWidth = left + 8 + IconTextSpacing + right;
    m_showText = (size().width() >= fm.width("M") * 6 + minimumWidth);
}
