/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "kerenconfigdialog.h"
#include <KStandardDirs>
#include <KPageWidgetItem>
#include <KDebug>
#include <QLabel>
#include <QtGui>
#include <gst/gst.h>
#include "renderer/videowidget.h"
#include "videocapturebin.h"
#include "libqtgstreamer/qgstpipeline.h"
#include "libqtgstreamer/qgstelement.h"
#include "libqtgstreamer/qgstelementfactory.h"
#include "libkgstdevices/devicemanager.h"
#include "libkgstdevices/devicesmodel.h"
#include "audioinputdevice.h"

namespace QtGstreamer
{
    class QGstPipeline;        
    typedef QSharedPointer<QGstPipeline> QGstPipelinePtr;

    class QGstElement;
    typedef QSharedPointer<QGstElement> QGstElementPtr;
}

void KerenConfigDialog::initAudioDevice()
{
    using namespace QtGstreamer;
    
    mAudioPipeline = QtGstreamer::QGstPipeline::newPipeline("audio-capture-test");
    mAudioInputDriver = "autoaudiosrc";
    mAudioSinkDriver = "autoaudiosink";
    mAudioInputPath = "";
    mAudioInputDevice = AudioInputDevice::newDevice(mAudioInputDriver,mAudioInputPath);
    QGstElementPtr audioSink = QGstElementFactory::make(mAudioSinkDriver.toLatin1());
    
    mAudioPipeline->add(mAudioInputDevice);
    mAudioPipeline->add(audioSink);
    QGstElement::link(mAudioInputDevice,audioSink);
}

void KerenConfigDialog::initVideoDevice()
{
    using namespace QtGstreamer;
    
    mVideoFrameSize = QSize(320,240);
    mVideoFrameRate = QSize(30,1);
    mVideoInputPath = "";
    mVideoInputDriver = "autovideosrc";

    mVideoPipeline = QGstPipeline::newPipeline("config_dialog_video_pipeline");
    mVideoCaptureBin = VideoCaptureBin::createVideoCaptureBin();

    QGstElementPtr videoSink= QGstElement::fromGstElement(mVideoWidget->videoBin());

    mVideoPipeline->add(mVideoCaptureBin);
    mVideoPipeline->add(videoSink);
    QGstElement::link(mVideoCaptureBin,videoSink);

    mVideoCaptureBin->newDevice(mVideoInputDriver,mVideoInputPath);
    mVideoCaptureBin->queryDeviceData();
}

KerenConfigDialog::KerenConfigDialog(QWidget *parent)
    : KDialog(parent)
{
    setButtons(KDialog::None);
    audioSettings = new QWidget();
    audioSettingsUi.setupUi(audioSettings);

    videoSettings = new QWidget();
    videoSettingsUi.setupUi(videoSettings);

    QString monitorPath = KStandardDirs::locate("data",  "kcontrol/pics/monitor.png");
    // Size of monitor image: 200x186
    // Geometry of "display" part of monitor image: (23,14)-[151x115]
    videoSettingsUi.previewLabel->setPixmap(QPixmap(monitorPath).scaled(300,279));
    mVideoWidget = new VideoWidget(videoSettingsUi.previewLabel,"software");
    mVideoWidget->setGeometry(34.5, 21, 226.5, 172.5);
    mVideoWidget->show();

    mDeviceManager = new KGstDevices::DeviceManager(this);
    mAudioInputModel = new KGstDevices::DevicesModel(this);
    mAudioInputModel->populate(mDeviceManager,KGstDevices::DeviceManager::AudioInput);
    audioSettingsUi.deviceComboBox->setModel(mAudioInputModel);
    
    mVideoInputModel = new KGstDevices::DevicesModel(this);
    mVideoInputModel->populate(mDeviceManager,KGstDevices::DeviceManager::VideoInput);
    videoSettingsUi.deviceComboBox->setModel(mVideoInputModel);
    
    initAudioDevice();
    initVideoDevice();
    updateResolutionList();
    updateFrameRateList();

    connect(audioSettingsUi.testCaptureButton, SIGNAL(clicked()),this,SLOT(slotAudioTestCapture()));
    connect(parent,SIGNAL(currentPageChanged(KPageWidgetItem*,KPageWidgetItem*)),this,SLOT(slotPageChanged(KPageWidgetItem*,KPageWidgetItem*)));
    connect(audioSettingsUi.deviceComboBox, SIGNAL(activated(int)), this, SLOT(slotAudioInputDeviceActivated(int)));
    connect(videoSettingsUi.deviceComboBox, SIGNAL(activated(int)), this, SLOT(slotVideoDeviceActivated(int)));
    connect(videoSettingsUi.resolutionComboBox, SIGNAL(activated(int)), this, SLOT(slotVideoResolutionActivated(int)));
    connect(videoSettingsUi.frameRateComboBox, SIGNAL(activated(int)), this, SLOT(slotVideoFrameRateActivated(int)));
    connect(videoSettingsUi.brightnessSlider, SIGNAL(valueChanged(int)), this, SLOT(slotBrightnessSliderChanged(int)));
    connect(videoSettingsUi.contrastSlider, SIGNAL(valueChanged(int)), this, SLOT(slotContrastSliderChanged(int)));
    connect(videoSettingsUi.saturationSlider, SIGNAL(valueChanged(int)), this, SLOT(slotSaturationSliderChanged(int)));
    connect(videoSettingsUi.hueSlider, SIGNAL(valueChanged(int)), this, SLOT(slotHueSliderChanged(int)));    
}

KerenConfigDialog::~KerenConfigDialog()
{
    kDebug() << "KerenConfigDialog destroyed";
}

void KerenConfigDialog::stop()
{
    mVideoPipeline->setState(QtGstreamer::QGstElement::Null);
}

void KerenConfigDialog::setVideoFrameSize(const QSize &videoFrameSize)
{
    int index=videoSettingsUi.resolutionComboBox->findData(QVariant(videoFrameSize));
    if (index!=-1) {
        videoSettingsUi.resolutionComboBox->setCurrentIndex(index);
        mVideoFrameSize = videoFrameSize;
        updateFrameRateList();
    }else
        kDebug() << "cannot set";
}

void KerenConfigDialog::setVideoFrameRate(const QSize &videoFrameRate)
{
    int index=videoSettingsUi.frameRateComboBox->findData(QVariant(videoFrameRate));
    if (index!=-1) {
        mVideoFrameRate = videoFrameRate;
        videoSettingsUi.frameRateComboBox->setCurrentIndex(index);
    }else
        kDebug() << "cannot set frame rate";
}

QString KerenConfigDialog::videoDevicePath() const 
{ 
    return mVideoInputPath;
}

QSize KerenConfigDialog::videoFrameSize() const
{ 
    return mVideoFrameSize;
}

QSize KerenConfigDialog::videoFrameRate() const
{ 
    return mVideoFrameRate;
}
    
void KerenConfigDialog::slotPageChanged(KPageWidgetItem* currentPage,KPageWidgetItem* previousPage)
{
    if (currentPage) {    
        if( currentPage->name() == i18n("Video Device")) {
            restartPreview();
        }
        else {
            mVideoPipeline->setState(QtGstreamer::QGstElement::Paused);
        }
    }
}

void KerenConfigDialog::slotAudioInputDeviceActivated(int index)
{
    KGstDevices::Device device = mDeviceManager->availableDevices(KGstDevices::DeviceManager::AudioInput)[index];
    QString deviceDriver = device.driver();
    mAudioInputPath = device.driverHandle().toString();
    kDebug() << "driver = " << deviceDriver << ", path=" << mAudioInputPath; 
}

void KerenConfigDialog::slotAudioTestCapture()
{
    using namespace QtGstreamer;
    
    QGstElement::State state;
    mAudioPipeline->getState(&state, NULL, 0);
    if (state == QGstElement::Playing) {
        mAudioPipeline->setState(QtGstreamer::QGstElement::Ready);
        audioSettingsUi.testCaptureButton->setText("Test");
    }
    else {
        mAudioPipeline->setState(QtGstreamer::QGstElement::Playing);
        audioSettingsUi.testCaptureButton->setText("Stop");
    }
}

void KerenConfigDialog::slotVideoDeviceActivated(int index)
{
    KGstDevices::Device device = mDeviceManager->availableDevices(KGstDevices::DeviceManager::VideoInput)[index];
    QString deviceDriver = device.driver();
    mVideoInputPath = device.driverHandle().toString();

    mVideoCaptureBin->newDevice(deviceDriver,mVideoInputPath);
    mVideoCaptureBin->queryDeviceData();
    updateResolutionList();
    updateFrameRateList();
    restartPreview();
}

void KerenConfigDialog::slotVideoResolutionActivated(int index)
{
    mVideoFrameSize = videoSettingsUi.resolutionComboBox->itemData(index).toSize();
    updateFrameRateList();
    restartPreview();
}

void KerenConfigDialog::slotVideoFrameRateActivated(int index)
{
    mVideoFrameRate = videoSettingsUi.frameRateComboBox->itemData(index).toSize();
    restartPreview();
}

void KerenConfigDialog::slotBrightnessSliderChanged(int value)
{
    mVideoCaptureBin->setBrightness(value);
}

void KerenConfigDialog::slotContrastSliderChanged(int value)
{
    mVideoCaptureBin->setContrast(value);
}

void KerenConfigDialog::slotSaturationSliderChanged(int value)
{
    mVideoCaptureBin->setSaturation(value);
}

void KerenConfigDialog::slotHueSliderChanged(int value)
{
    mVideoCaptureBin->setHue(value);
}

void KerenConfigDialog::updateResolutionList()
{
    videoSettingsUi.resolutionComboBox->clear();
    //find supported resolutions
    QList<QSize> resolutions = mVideoCaptureBin->getSupportedResolution();
    for (int ii=0;ii<resolutions.size();++ii) {
        videoSettingsUi.resolutionComboBox->addItem(QString("%1x%2").arg(resolutions[ii].width()).arg(resolutions[ii].height()),resolutions[ii]);
    }
    if (resolutions.size()<1)
        return;
    mVideoFrameSize = resolutions[0];
}

void KerenConfigDialog::updateFrameRateList()
{
    videoSettingsUi.frameRateComboBox->clear();
    //find supported framerate
    QList<QSize> frameRates = mVideoCaptureBin->getSupportedFrameRate(mVideoFrameSize);
    for (int ii=0;ii<frameRates.size();++ii) {
        videoSettingsUi.frameRateComboBox->addItem(QString("%1 fps").arg((float)frameRates[ii].width()/frameRates[ii].height()),frameRates[ii]);
    }
    if (frameRates.size()<1)
        return;

    mVideoFrameRate = frameRates[0];
}

void KerenConfigDialog::restartPreview()
{
    mVideoPipeline->setState(QtGstreamer::QGstElement::Ready);

    mVideoCaptureBin->updateCapabilities(mVideoFrameSize,mVideoFrameRate);
    kDebug() << "Start capturing for " << mVideoInputPath << " size=" << mVideoFrameSize;
    mVideoPipeline->setState(QtGstreamer::QGstElement::Playing);

    //TODO:check capabilities before setting it
    videoSettingsUi.brightnessSlider->setValue(mVideoCaptureBin->brightness());
    videoSettingsUi.contrastSlider->setValue(mVideoCaptureBin->contrast());
    videoSettingsUi.saturationSlider->setValue(mVideoCaptureBin->saturation());
    videoSettingsUi.hueSlider->setValue(mVideoCaptureBin->hue());
}

#include "kerenconfigdialog.moc"