/*
 * This file is part of the KDE project
 * Copyright (C) 2009 Eva Brucherseifer <eva.brucherseifer@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "killswitchdevice.h"

#include <QtCore/QDebug>
#include <QtDBus/QDBusConnection>
#include <QtDBus/QDBusInterface>
#include <QtDBus/QDBusReply>
#include <QtDBus/QDBusArgument>
#include <QtDBus/QDBusMetaType>

#include <KDebug>
#include <solid/genericinterface.h>

class KillSwitchDevicePrivate
{
public:
    KillSwitchDevicePrivate(const QString &udi)
        : device("org.freedesktop.Hal",
                  udi,
                  "org.freedesktop.Hal.Device",
                  QDBusConnection::systemBus()),
          cacheSynced(false), parent(0) { }

    QDBusInterface device;
    QMap<QString,QVariant> cache;
    bool cacheSynced;
    KillSwitchDevice *parent;
};


Q_DECLARE_METATYPE(ChangeDescription)
Q_DECLARE_METATYPE(QList<ChangeDescription>)

const QDBusArgument &operator<<(QDBusArgument &arg, const ChangeDescription &change)
{
    arg.beginStructure();
    arg << change.key << change.added << change.removed;
    arg.endStructure();
    return arg;
}

const QDBusArgument &operator>>(const QDBusArgument &arg, ChangeDescription &change)
{
    arg.beginStructure();
    arg >> change.key >> change.added >> change.removed;
    arg.endStructure();
    return arg;
}



KillSwitchDevice::KillSwitchDevice(const QString &udi)
    : QObject(), d(new KillSwitchDevicePrivate(udi))
{
    qDBusRegisterMetaType<ChangeDescription>();
    qDBusRegisterMetaType< QList<ChangeDescription> >();

    d->device.connection().connect("org.freedesktop.Hal",
                                    udi, "org.freedesktop.Hal.Device",
                                    "PropertyModified",
                                    this, SLOT(slotPropertyModified(int, const QList<ChangeDescription> &)));
    d->device.connection().connect("org.freedesktop.Hal",
                                    udi, "org.freedesktop.Hal.Device",
                                    "Condition",
                                    this, SLOT(slotCondition(const QString &, const QString &)));
}

KillSwitchDevice::~KillSwitchDevice()
{
    delete d->parent;
    delete d;
}


QString KillSwitchDevice::udi() const
{
    return property("info.udi").toString();
}

QString KillSwitchDevice::parentUdi() const
{
    return property("info.parent").toString();
}

QString KillSwitchDevice::vendor() const
{
    return property("info.vendor").toString();
}

QString KillSwitchDevice::product() const
{
    return property("info.product").toString();
}

QString KillSwitchDevice::type() const
{
    return property("killswitch.type").toString();
}

int KillSwitchDevice::state() const
{
    return property("killswitch.state").toInt();
}

QString KillSwitchDevice::name() const
{
    return property("killswitch.name").toString();
}

QString KillSwitchDevice::access_method() const
{
    return property("killswitch.access_method").toString();
}


// file:///usr/share/doc/packages/hal/spec/hal-spec.html#device-properties-killswitch
// 
// KillSwitchDevice::State KillSwitchDevice::state()
// {
//     QString udi = this->udi();
//     QString interface = "org.freedesktop.Hal.Device.KillSwitch";
// 
// /*    QDBusConnection c = QDBusConnection::systemBus();
//     QDBusMessage msg = QDBusMessage::createMethodCall("org.freedesktop.Hal", udi,
//                                                       interface, "GetPower");
//     msg << QStringList();
// 
//     return c.callWithCallback(msg, this,
//                               SLOT(slotDBusReply(const QDBusMessage &)),
//                               SLOT(slotDBusError(const QDBusError &)));*/
// 
//    QDBusInterface parentIface(QLatin1String("org.freedesktop.Hal"), 
// 	udi, interface, 
// 	QDBusConnection::systemBus());
//         const QDBusMessage &reply = parentIface.call("GetPower");
//         if ((reply.type() != QDBusMessage::ReplyMessage && reply.errorName() == "org.freedesktop.Hal.NoSuchMethod") ||
//                 (reply.type() == QDBusMessage::ReplyMessage && reply.arguments().at(0) == "sound")) {
//             const QDBusMessage &reply2 = parentIface.call("GetProperty", QLatin1String("info.parent"));
//             if (reply2.type() == QDBusMessage::ReplyMessage) {
//                 parentUdi = reply2.arguments().at(0).toString();
//             }
//         }
// 
// }

bool KillSwitchDevice::setPower(bool poweron)
{
    QString udi = this->udi();
    QString interface = "org.freedesktop.Hal.Device.KillSwitch";

    kDebug() << this->name() << ": Setting device state " << poweron;

    QDBusConnection c = QDBusConnection::systemBus();
    QDBusMessage msg = QDBusMessage::createMethodCall("org.freedesktop.Hal", udi,
                                                      interface, "SetPower");
    msg << poweron;

    return c.callWithCallback(msg, this,
                              SLOT(slotDBusReply(const QDBusMessage &)),
                              SLOT(slotDBusError(const QDBusError &)));
}


void KillSwitchDevice::slotPropertyModified(int /*count */, const QList<ChangeDescription> &changes)
{
    QMap<QString,int> result;

    foreach (const ChangeDescription &change, changes)
    {
        QString key = change.key;
        bool added = change.added;
        bool removed = change.removed;

        Solid::GenericInterface::PropertyChange type = Solid::GenericInterface::PropertyModified;

        if (added)
        {
            type = Solid::GenericInterface::PropertyAdded;
        }
        else if (removed)
        {
            type = Solid::GenericInterface::PropertyRemoved;
        }

        result[key] = type;

        d->cache.remove(key);
    }

    d->cacheSynced = false;

    emit propertyChanged(result);
}

void KillSwitchDevice::slotCondition(const QString &condition, const QString &reason)
{
    emit conditionRaised(condition, reason);
}


void KillSwitchDevice::slotDBusReply(const QDBusMessage &/*reply*/)
{
    emit switchDone(); //Solid::NoError, QVariant(), m_device->udi());
}

void KillSwitchDevice::slotDBusError(const QDBusError &error)
{
    // TODO: Better error reporting here
    emit switchDone(); 
    qWarning() << "slotDBusError " << error.name() << ": " << error.message() << udi() << endl;
//Solid::UnauthorizedOperation,
//                   error.name()+": "+error.message(),
//                   m_device->udi());
}




QVariant KillSwitchDevice::property(const QString &key) const
{
    if (d->cache.contains(key))
    {
        return d->cache[key];
    }
    else if (d->cacheSynced)
    {
        return QVariant();
    }

    QDBusMessage reply = d->device.call("GetProperty", key);

    if (reply.type()!=QDBusMessage::ReplyMessage
      && reply.errorName()!="org.freedesktop.Hal.NoSuchProperty")
    {
        qWarning() << Q_FUNC_INFO << " error: " << reply.errorName()
                   << ", " << reply.arguments().at(0).toString() << endl;
        return QVariant();
    }

    if (reply.errorName()=="org.freedesktop.Hal.NoSuchProperty")
    {
        d->cache[key] = QVariant();
    }
    else
    {
        d->cache[key] = reply.arguments().at(0);
    }

    return d->cache[key];
}


bool KillSwitchDevice::queryDeviceInterface(const QStringList &cap_list) const
{
    foreach (const QString &cap, cap_list)
    {
        QDBusReply<bool> reply = d->device.call("QueryCapability", cap);

        if (!reply.isValid())
        {
            qWarning() << Q_FUNC_INFO << " error: " << reply.error().name() << endl;
            return false;
        }

        if (reply) return reply;
    }

    return false;
}

#include "killswitchdevice.moc"
