/*****************************************************************************
 *   Copyright (C) 2008 by Ben Cooksley <ben@eclipse.endoftheinternet.org>   *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .          *
 ****************************************************************************/

#include "kuickquiz.h"

#include <QPainter>
#include <QMap>
#include <QHeaderView>
#include <QRadioButton>
#include <QtCrypto>

#include <KConfigDialog>
#include <KMessageBox>

#include <plasma/theme.h>
#include <plasma/widgets/iconwidget.h>

KuickQuiz::KuickQuiz(QObject *parent, const QVariantList &args)
        : Plasma::PopupApplet(parent, args)
        , mainWidget(0)
{
    setBackgroundHints(DefaultBackground);
    setHasConfigurationInterface(true);
    setPopupIcon("kuickquiz");
    totalQuestions = 0;
    graphicsWidget();
}

/// General plasmoid

void KuickQuiz::init()
{   // Prepare and show splash
    updateModuleGeometry();
    splashActive();
}

void KuickQuiz::createConfigurationInterface(KConfigDialog *parent)
{
    QTableWidgetItem *temp;
    QStringList tableColumnNames;
    QString tempInput;

    /// Add Pages to master dialog
    configMain = new QWidget;
    passwordChange = new QWidget;
    uiMain.setupUi(configMain);
    uiSetPassword.setupUi(passwordChange);
    parent->addPage(configMain, i18n("Quiz options"), "configure", 0);
    parent->setButtons(KDialog::Ok | KDialog::Cancel);
    /// Prepare data table
    uiMain.TwQuestionList->clear(); // clear the table to clear stale information, invalid changes cancelled
    // Set up table column names
    tableColumnNames << i18n("Question") << i18n("Option 1") << i18n("Option 2") << i18n("Option 3") << i18n("Option 4") << i18n("Answer");
    uiMain.TwQuestionList->setHorizontalHeaderLabels(tableColumnNames);
    uiMain.TwQuestionList->verticalHeader()->hide();
    // Import Data
    for (int importQuestions = 0; importQuestions < totalQuestions; importQuestions = importQuestions + 1) {
        uiMain.TwQuestionList->setRowCount(uiMain.TwQuestionList->rowCount() + 1);
        temp = new QTableWidgetItem();
        for (int qanda = 0; qanda < 5; qanda = qanda + 1) {
            temp = new QTableWidgetItem();
            temp->setText(questionsAndAnswers[importQuestions][qanda]);
            uiMain.TwQuestionList->setItem(importQuestions, qanda, temp);
        }
        temp = new QTableWidgetItem();
        tempInput = QString::number(questionAnswers[importQuestions]);
        temp->setText(tempInput);
        uiMain.TwQuestionList->setItem(importQuestions, 5, temp);
    }
    /// Prepare quiz configuration dialog
    uiMain.LeQuizName->setText(quizName);
    // Connect things to functions so they work
    connect(uiMain.PbAddQuestion, SIGNAL(clicked()), SLOT(configAddQuestion()));
    connect(uiMain.PbDeleteQuestion, SIGNAL(clicked()), SLOT(configDelQuestion()));
    connect(uiMain.PbEditQuestion, SIGNAL(clicked()), SLOT(configEditQuestion()));
    connect(uiMain.PbClearScores, SIGNAL(clicked()), SLOT(configClearScores()));
    connect(uiMain.PbSaveQuestion, SIGNAL(clicked()), SLOT(configSaveQuestion()));
    connect(uiMain.PbCancel, SIGNAL(clicked()), SLOT(configCancelQuestionChanges()));
    connect(uiMain.PbPassContinue, SIGNAL(clicked()), SLOT(configAuthenticate()));
    connect(uiMain.PbSetPassword, SIGNAL(clicked()), SLOT(configPassword()));
    connect(uiSetPassword.PbAccept, SIGNAL(clicked()), SLOT(configSetPassword()));
    connect(uiSetPassword.PbCancel, SIGNAL(clicked()), passwordChange, SLOT(hide()));
    connect(parent, SIGNAL(okClicked()), SLOT(configAccept()));
    // Hide question editing widget
    if (password == QString()) {
        configSetAuthenticated(true);
    } else {
        configSetAuthenticated(false);
    }
    uiMain.FrEditQuestion->hide();
    /// Prepare plasmoid configuration dialog
}

void KuickQuiz::updateModuleGeometry()
{
    splash->setGeometry(mainWidget->contentsRect());
    question->setGeometry(mainWidget->contentsRect());
    finish->setGeometry(mainWidget->contentsRect());
}

QGraphicsWidget* KuickQuiz::graphicsWidget()
{
    if (!mainWidget) { // Prepare popup
        mainWidget = new QGraphicsWidget(this);
        mainWidget->setMinimumSize(500, 225);
        // Create the three display screens inside frames for easy management and layout
        splash = new QuizSplash(mainWidget);
        question = new QuizQuestion(mainWidget);
        finish = new QuizFinish(mainWidget);
        // Connect up everyone
        connect(this, SIGNAL(geometryChanged()), SLOT(updateModuleGeometry()));
        connect(&splash->startQuiz, SIGNAL(clicked()), SLOT(startQuiz()));
        connect(&question->submit, SIGNAL(clicked()), SLOT(nextQuestion()));
        connect(&question->abort, SIGNAL(clicked()), SLOT(abortQuiz()));
        connect(&finish->submitResult, SIGNAL(clicked()), SLOT(insertResults()));
        connect(&finish->noSubmit, SIGNAL(clicked()), SLOT(splashActive()));
    }
    return mainWidget;
}

/// Plasmoid configuration

void KuickQuiz::configSetAuthenticated(bool authenticated)
{
    uiMain.FrAuthenticate->setVisible(!authenticated);
    uiMain.FrEditQuestion->setVisible(authenticated);
    uiMain.PbAddQuestion->setVisible(authenticated);
    uiMain.PbDeleteQuestion->setVisible(authenticated);
    uiMain.PbEditQuestion->setVisible(authenticated);
    uiMain.TwQuestionList->setVisible(authenticated);
    uiMain.LeQuizName->setVisible(authenticated);
    uiMain.PbClearScores->setVisible(authenticated);
    uiMain.PbSetPassword->setVisible(authenticated);
    uiMain.TlQuizName->setVisible(authenticated);
}

void KuickQuiz::configAuthenticate()
{
    QCA::Initializer init;
    QCA::SecureArray Ut = uiMain.LePassword->text().toUtf8();
    QString hashedPass = QCA::Hash("sha1").hashToString(Ut);
    if (hashedPass == password) {
        configSetAuthenticated(true);
    } else {
        KMessageBox::detailedError(configMain, i18n("Incorrect password"), i18n("The password you have supplied was incorrect. Please try again."), i18n("Incorrect password"));
        uiMain.LePassword->clear();
    }
}

void KuickQuiz::configAddQuestion()
{
    QTableWidgetItem * newitem;

    uiMain.TwQuestionList->setRowCount(uiMain.TwQuestionList->rowCount() + 1);
    newitem = new QTableWidgetItem();
    uiMain.TwQuestionList->setCurrentCell(uiMain.TwQuestionList->rowCount() - 1, 0);
    addingQuestion = true;
    for (int addQuestion = 0; addQuestion < 6; addQuestion = addQuestion + 1) {
        newitem = new QTableWidgetItem();
        newitem->setText(QString());
        uiMain.TwQuestionList->setItem(uiMain.TwQuestionList->rowCount() - 1, addQuestion, newitem);
    }
    configEditQuestion();
}

void KuickQuiz::configDelQuestion()
{
    uiMain.TwQuestionList->removeRow(configActiveQuestion());
}

void KuickQuiz::configEditQuestion()
{
    int activeQuestion;

    uiMain.TwQuestionList->setEnabled(false); // Do not permit question to be changed when editing
    uiMain.PbAddQuestion->setEnabled(false);
    uiMain.PbDeleteQuestion->setEnabled(false);
    uiMain.PbEditQuestion->setEnabled(false);
    uiMain.FrEditQuestion->show();
    activeQuestion = configActiveQuestion();
    uiMain.LeQuestion->setText(uiMain.TwQuestionList->item(activeQuestion, 0)->text());
    uiMain.LeAnswer1->setText(uiMain.TwQuestionList->item(activeQuestion, 1)->text());
    uiMain.LeAnswer2->setText(uiMain.TwQuestionList->item(activeQuestion, 2)->text());
    uiMain.LeAnswer3->setText(uiMain.TwQuestionList->item(activeQuestion, 3)->text());
    uiMain.LeAnswer4->setText(uiMain.TwQuestionList->item(activeQuestion, 4)->text());
    uiMain.SbAnswer->setValue(uiMain.TwQuestionList->item(activeQuestion, 5)->text().toInt());
}

void KuickQuiz::configSaveQuestion()
{
    int activeQuestion = 0;
    QString answerValue;

    activeQuestion = configActiveQuestion();
    uiMain.FrEditQuestion->hide();
    uiMain.TwQuestionList->item(activeQuestion, 0)->setText(uiMain.LeQuestion->text());
    uiMain.TwQuestionList->item(activeQuestion, 1)->setText(uiMain.LeAnswer1->text());
    uiMain.TwQuestionList->item(activeQuestion, 2)->setText(uiMain.LeAnswer2->text());
    uiMain.TwQuestionList->item(activeQuestion, 3)->setText(uiMain.LeAnswer3->text());
    uiMain.TwQuestionList->item(activeQuestion, 4)->setText(uiMain.LeAnswer4->text());
    answerValue = QString::number(uiMain.SbAnswer->value());
    uiMain.TwQuestionList->item(activeQuestion, 5)->setText(answerValue);
    addingQuestion = false;
    configEnableQuestionManagement();
}

void KuickQuiz::configCancelQuestionChanges()
{
    uiMain.FrEditQuestion->hide();
    if (addingQuestion == true) {
        configDelQuestion();
    }
    configEnableQuestionManagement();
}

void KuickQuiz::configEnableQuestionManagement()
{
    uiMain.TwQuestionList->setEnabled(true);
    uiMain.PbAddQuestion->setEnabled(true);
    uiMain.PbDeleteQuestion->setEnabled(true);
    uiMain.PbEditQuestion->setEnabled(true);
}

int KuickQuiz::configActiveQuestion()
{
    return uiMain.TwQuestionList->currentRow();
}

void KuickQuiz::configAccept()
{
    quizName = uiMain.LeQuizName->text();
    totalQuestions = uiMain.TwQuestionList->rowCount();
    for (int saveQuestions = 0; saveQuestions < totalQuestions; saveQuestions = saveQuestions + 1) {
        for (int itemSaved = 0; itemSaved < 5; itemSaved = itemSaved + 1) {
            questionsAndAnswers[saveQuestions][itemSaved] = uiMain.TwQuestionList->item(saveQuestions, itemSaved)->text();
        }
        questionAnswers[saveQuestions] = uiMain.TwQuestionList->item(saveQuestions, 5)->text().toInt();
    }
    refreshSplash();
}

void KuickQuiz::configClearScores()
{
    results.clear();
}

void KuickQuiz::configExportQuiz()
{
}

void KuickQuiz::configImportQuiz()
{
}

void KuickQuiz::configPassword()
{
    if (password == QString()) {
        uiSetPassword.TlCurrentPass->hide();
        uiSetPassword.LeCurrentPass->hide();
    } else {
        uiSetPassword.TlCurrentPass->show();
        uiSetPassword.LeCurrentPass->show();
    }
    uiSetPassword.LeCurrentPass->clear();
    uiSetPassword.LeNewPass->clear();
    uiSetPassword.LeConfirmPass->clear();
    passwordChange->show();
}

void KuickQuiz::configSetPassword()
{
    bool oldPassValid = false;
    bool newPassMatch = false;
    QCA::Initializer init;
    QCA::SecureArray securePassword;
    QString hashedPass;

    // Hash the provided old password
    securePassword = uiSetPassword.LeCurrentPass->text().toUtf8();
    hashedPass = QCA::Hash("sha1").hashToString(securePassword);
    // Check to make sure the old password is valid
    if (hashedPass == password || password == QString()) {
        oldPassValid = true;
    }
    // Make sure the new passwords match
    if (uiSetPassword.LeNewPass->text() == uiSetPassword.LeConfirmPass->text()) {
        newPassMatch = true;
    }
    // Hash the new password
    securePassword = uiSetPassword.LeNewPass->text().toUtf8();
    hashedPass = QCA::Hash("sha1").hashToString(securePassword);
    // If the new password is blank, we set QString() not a hashed value of QString()
    if (uiSetPassword.LeNewPass->text() == QString()) {
        hashedPass = QString();
    }
    // If the old password and new password is valid, update the password
    if (newPassMatch && oldPassValid) {
        password = hashedPass;
        passwordChange->hide();
    }
    // Otherwise show an error! and clear the passwords so they can't try and crack it
    else {
        KMessageBox::detailedError(configMain, i18n("A supplied password did not match."), i18n("Either the new passwords do not match, or the old password was incorrect."), i18n("Incorrect password"));
        uiSetPassword.LeCurrentPass->clear();
        uiSetPassword.LeNewPass->clear();
        uiSetPassword.LeConfirmPass->clear();
    }
}

/// Splash screen

void KuickQuiz::refreshSplash()
{
    QStringList topScoreNames;
    QList<int> topScoreValues;
    QStringList usedNames;
    QString resultsList[5];
    int alreadyUsed;
    int userScore;
    int userPosition;

    // Set quiz header
    if (quizName != QString()) {
        splash->header.setText(i18n("Quiz for %1", quizName));
    } else {
        splash->header.setText(i18n("No quiz name has been specified"));
    }
    // Disable ability to do quiz if no questions exist
    if (totalQuestions < 1) {
        splash->startQuiz.setText(i18n("The quiz cannot be done because no questions exist."));
        splash->startQuiz.setEnabled(false);
    } else {
        splash->startQuiz.setText(i18n("Do this quiz"));
        splash->startQuiz.setEnabled(true);
    }
    // Fill the results table
    if (results.count() == 0) {
        splash->resultsHeader.hide();
        splash->results.setText(i18n("No scores have been entered yet.\nDo the quiz to receive some results."));
    } else {
        for (int questionDown = 0; topScoreNames.count() <= 5 && topScoreNames.count() < results.count() ; questionDown = questionDown + 1) {
            topScoreNames = topScoreNames + results.values(totalQuestions - questionDown);
        }
        for (int resultNumber = 0; resultNumber < 5 && resultNumber < topScoreNames.count() ; resultNumber = resultNumber + 1) {
            topScoreValues = results.keys(topScoreNames[resultNumber]);
            alreadyUsed = usedNames.count(topScoreNames[resultNumber]);
            qSort(topScoreValues.begin(), topScoreValues.end());
            userScore = topScoreValues.at(topScoreValues.count() - alreadyUsed - 1);
            userPosition = resultNumber + 1;
            resultsList[resultNumber] = i18np("Position %2 : 1 point scored by %3 \n", "Position %2 : %1 points scored by %3 \n", userScore, userPosition, topScoreNames[resultNumber]);
            usedNames.append(topScoreNames[resultNumber]);
        }
        splash->results.setText(resultsList[0] + resultsList[1] + resultsList[2] + resultsList[3] + resultsList[4]);
        splash->resultsHeader.show();

    }
}

void KuickQuiz::splashActive()
{
    question->hide();
    finish->hide();
    refreshSplash();
    splash->show();
}

/// Post quiz

void KuickQuiz::finishPrepare() // sets up post quiz recording
{
    question->hide();
    finish->score.setText(i18np("You scored 1 point from a total of %2", "You scored %1 points from a total of %2", userScore, totalQuestions));
    finish->name.setText(i18n("Your name here"));
    finish->show();
}

void KuickQuiz::insertResults()
{ // Get the name and user score
    QString name = finish->name.text();
    // Add them to the database
    results.insertMulti(userScore, name);
    // Switch back to splash
    splashActive();
}

/// Running quiz

void KuickQuiz::startQuiz()
{
    splash->hide();
    questionsDone = 0;
    userScore = 0;
    question->show();
    nextQuestion();
}

void KuickQuiz::abortQuiz()
{
    question->hide();
    splashActive();
}

void KuickQuiz::nextQuestion()
{
    if (questionsDone > 0) {
        if (question->chooseAnswer[questionAnswers[questionsDone-1] - 1].isChecked()) {
            userScore = userScore + 1;
        }
    }
    questionsDone = questionsDone + 1;
    if (totalQuestions < questionsDone) {
        finishPrepare();
        return;
    }
    question->question.setText(questionsAndAnswers[questionsDone-1][0]);
    question->buttonGroup.setExclusive(false);
    for (int answerWritten = 1; answerWritten <= 4; answerWritten = answerWritten + 1) {
        question->chooseAnswer[answerWritten-1].setChecked(false);
        question->chooseAnswer[answerWritten-1].setText(questionsAndAnswers[questionsDone-1][answerWritten]);
    }
    question->buttonGroup.setExclusive(true);
}

#include "kuickquiz.moc"
