/***********************************************************************************
* Mini Player: Advanced media player for Plasma.
* Copyright (C) 2008 - 2009 Michal Dutkiewicz aka Emdek <emdeck@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#include "MiniPlayerApplet.h"
#include "MiniPlayerPlayListModel.h"
#include "MiniPlayerPlayListImporter.h"
#include "PlayerDBusHandler.h"
#include "RootDBusHandler.h"
#include "TrackListDBusHandler.h"

#include <QTime>
#include <QFile>
#include <QFileInfo>
#include <QClipboard>
#include <QHeaderView>
#include <QActionGroup>
#include <QKeySequence>
#include <QDBusConnection>

#include <KIcon>
#include <KLocale>
#include <KMimeType>
#include <KMessageBox>
#include <KFileDialog>
#include <KInputDialog>
#include <KConfigDialog>
#include <KRandomSequence>

#include <Plasma/Theme>
#include <Plasma/Corona>
#include <Plasma/Containment>
#include <Plasma/ToolTipManager>

K_EXPORT_PLASMA_APPLET(miniplayer, MiniPlayerApplet)

MiniPlayerApplet::MiniPlayerApplet(QObject *parent, const QVariantList &args) : Plasma::Applet(parent, args)
{
    KGlobal::locale()->insertCatalog("miniplayer");

    setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));

    setHasConfigurationInterface(true);

    setAcceptDrops(true);

    setMinimumWidth(150);

    resize(250, 50);

    m_hideFullScreenControls = new QTimer(this);
    m_hideFullScreenControls->setSingleShot(true);

    m_showPlayList = new QTimer(this);
    m_showPlayList->setSingleShot(true);

    m_hideToolTip = new QTimer(this);
    m_hideToolTip->setSingleShot(true);

    m_notificationRestrictions = NULL;

    m_editorActive = false;

    m_mediaObject = new Phonon::MediaObject(this);
    m_mediaObject->setTickInterval(1000);

    m_mediaController = new Phonon::MediaController(m_mediaObject);

    m_videoWidget = new Phonon::VideoWidget;
    m_videoWidget->setScaleMode(Phonon::VideoWidget::FitInView);
    m_videoWidget->setWindowIcon(KIcon("applications-multimedia"));
    m_videoWidget->installEventFilter(this);
    m_videoWidget->setAcceptDrops(true);

    m_audioOutput = new Phonon::AudioOutput(this);

    Phonon::createPath(m_mediaObject, m_videoWidget);
    Phonon::createPath(m_mediaObject, m_audioOutput);

    m_proxyWidget = new QGraphicsProxyWidget(this);
    m_proxyWidget->setWidget(m_videoWidget);
    m_proxyWidget->setAcceptHoverEvents(true);

    m_mainLayout = new QGraphicsLinearLayout(Qt::Vertical, this);
    m_mainLayout->setSpacing(0);
    m_mainLayout->setContentsMargins(0, 0, 0, 0);
    m_mainLayout->insertItem(0, m_proxyWidget);

    setLayout(m_mainLayout);

    m_volumeDialog = new Plasma::Dialog;
    m_volumeDialog->setWindowFlags(Qt::Popup);

    m_volumeSlider = new Phonon::VolumeSlider(m_audioOutput, m_volumeDialog);
    m_volumeSlider->setOrientation(Qt::Vertical);

    QVBoxLayout *volumeLayout = new QVBoxLayout(m_volumeDialog);
    volumeLayout->setSpacing(0);
    volumeLayout->setContentsMargins(0, 0, 0, 0);
    volumeLayout->addWidget(m_volumeSlider);

    m_volumeDialog->resize(m_volumeSlider->sizeHint());

    m_playListDialog = new Plasma::Dialog(NULL, Qt::Tool);
    m_playListDialog->setFocusPolicy(Qt::NoFocus);
    m_playListDialog->setWindowTitle(i18n("Play list"));
    m_playListDialog->setWindowIcon(KIcon("applications-multimedia"));
    m_playListDialog->setResizeHandleCorners(Plasma::Dialog::All);

    m_fullScreen = new QWidget;
    m_fullScreen->installEventFilter(this);

    m_fullScreenUi.setupUi(m_fullScreen);

    m_playListUi.setupUi(m_playListDialog);

    m_playListDialog->setContentsMargins(0, 0, 0, 0);
    m_playListDialog->adjustSize();

    m_playListTabBar = new KTabBar(m_playListDialog);
    m_playListTabBar->setTabsClosable(true);
    m_playListTabBar->setMovable(true);
    m_playListTabBar->setDrawBase(false);

    m_playListUi.blankLabel->installEventFilter(this);
    m_playListUi.playListView->installEventFilter(this);
    m_playListUi.playListTabBarLayout->addWidget(m_playListTabBar);
    m_playListUi.blankLabel->setPixmap(KIcon("applications-multimedia").pixmap(KIconLoader::SizeEnormous));

    m_repeatModeMenu = new KMenu(m_playListUi.repeatButton);

    QActionGroup *repeatActionGroup = new QActionGroup(m_repeatModeMenu);

    QAction *noRepeatAction = m_repeatModeMenu->addAction(KIcon("dialog-cancel"), i18n("No Repeat"));
    noRepeatAction->setCheckable(true);
    noRepeatAction->setData(NoRepeat);

    repeatActionGroup->addAction(noRepeatAction);

    QAction *repeatTrackAction = m_repeatModeMenu->addAction(KIcon("audio-x-generic"), i18n("Repeat Track"));
    repeatTrackAction->setCheckable(true);
    repeatTrackAction->setData(RepeatTrack);

    repeatActionGroup->addAction(repeatTrackAction);

    QAction *repeatPlayListAction = m_repeatModeMenu->addAction(KIcon("view-media-playlist"), i18n("Repeat Play-list"));
    repeatPlayListAction->setCheckable(true);
    repeatPlayListAction->setData(RepeatPlayList);

    repeatActionGroup->addAction(repeatPlayListAction);

    m_playListUi.closeButton->setIcon(KIcon("window-close"));
    m_playListUi.addButton->setIcon(KIcon("list-add"));
    m_playListUi.removeButton->setIcon(KIcon("list-remove"));
    m_playListUi.editButton->setIcon(KIcon("document-edit"));
    m_playListUi.moveUpButton->setIcon(KIcon("arrow-up"));
    m_playListUi.moveDownButton->setIcon(KIcon("arrow-down"));
    m_playListUi.newButton->setIcon(KIcon("document-new"));
    m_playListUi.exportButton->setIcon(KIcon("document-export"));
    m_playListUi.clearButton->setIcon(KIcon("edit-clear"));
    m_playListUi.randomButton->setIcon(KIcon("media-playlist-shuffle"));
    m_playListUi.repeatButton->setIcon(KIcon("edit-redo"));
    m_playListUi.repeatButton->setMenu(m_repeatModeMenu);

    m_audioAction = new KAction(KIcon("player-volume"), i18n("Volume"), this);
    m_audioAction->setShortcut(QKeySequence(Qt::Key_V));

    m_openMenu = new KMenu;

    QAction* openFileAction = m_openMenu->addAction(KIcon("document-open"), i18n("Open File"));
    openFileAction->setShortcut(QKeySequence(Qt::Key_O));

    QAction* openUrlAction = m_openMenu->addAction(KIcon("uri-mms"), i18n("Open URL"));
    openUrlAction->setShortcut(QKeySequence(Qt::Key_U));

    m_openMenu->addSeparator();

    KAction* openAction = new KAction(i18n("Open"), this);
    openAction->setMenu(m_openMenu);

    m_playPauseAction = new KAction(KIcon("media-playback-start"), i18n("Play"), this);
    m_playPauseAction->setEnabled(false);
    m_playPauseAction->setShortcut(QKeySequence(Qt::Key_Space));

    m_stopAction = new KAction(KIcon("media-playback-stop"), i18n("Stop"), this);
    m_stopAction->setEnabled(false);
    m_stopAction->setShortcut(QKeySequence(Qt::Key_S));

    KMenu *navigationMenu = new KMenu;

    QAction* playPreviousAction = navigationMenu->addAction(KIcon("media-skip-backward"), i18n("Previous"));
    playPreviousAction->setShortcut(QKeySequence(Qt::Key_PageDown));

    QAction* playNextAction = navigationMenu->addAction(KIcon("media-skip-forward"), i18n("Next"));
    playNextAction->setShortcut(QKeySequence(Qt::Key_PageUp));

    navigationMenu->addSeparator();

    QAction* seekBackwardAction = navigationMenu->addAction(KIcon("media-seek-backward"), i18n("Seek Backward"));
    seekBackwardAction->setShortcut(QKeySequence(Qt::Key_Left));

    QAction* seekForwardAction = navigationMenu->addAction(KIcon("media-seek-forward"), i18n("Seek Forward"));
    seekForwardAction->setShortcut(QKeySequence(Qt::Key_Right));

    navigationMenu->addSeparator();

    QAction* jumpToPositionAction = navigationMenu->addAction(KIcon("go-jump"), i18n("Jump to Position"));
    jumpToPositionAction->setShortcut(QKeySequence(Qt::Key_G));

    m_navigationAction = new KAction(i18n("Navigation"), this);
    m_navigationAction->setMenu(navigationMenu);
    m_navigationAction->setEnabled(false);

    KMenu *audioMenu = new KMenu;

    m_audioChannelMenu = new KMenu();
    m_audioChannelMenu->setTitle(i18n("Audio Channel"));
    m_audioChannelMenu->setEnabled(false);

    audioMenu->addMenu(m_audioChannelMenu);
    audioMenu->addSeparator();

    QAction* increaseVolumeAction = audioMenu->addAction(KIcon("audio-volume-high"), i18n("Increase Volume"));
    increaseVolumeAction->setShortcut(QKeySequence(Qt::Key_Plus));

    QAction* decreaseVolumeAction = audioMenu->addAction(KIcon("audio-volume-low"), i18n("Decrease Volume"));
    decreaseVolumeAction->setShortcut(QKeySequence(Qt::Key_Minus));

    audioMenu->addSeparator();

    m_muteVolumeAction = new KAction(KIcon("player-volume"), i18n("Mute Volume"), audioMenu);
    m_muteVolumeAction->setShortcut(QKeySequence(Qt::Key_M));

    audioMenu->addAction(m_muteVolumeAction);

    KAction* audioAction = new KAction(i18n("Audio"), this);
    audioAction->setMenu(audioMenu);

    KMenu *videoMenu = new KMenu;

    m_aspectRatioMenu = new KMenu();
    m_aspectRatioMenu->setTitle(i18n("Aspect Ratio"));

    QActionGroup *aspectRatioActionGroup = new QActionGroup(m_aspectRatioMenu);

    QAction *aspectRatioAutomatic = m_aspectRatioMenu->addAction(i18n("Automatic"));
    aspectRatioAutomatic->setCheckable(true);
    aspectRatioAutomatic->setData(0);

    aspectRatioActionGroup->addAction(aspectRatioAutomatic);

    QAction *aspectRatio43 = m_aspectRatioMenu->addAction(i18n("4:3"));
    aspectRatio43->setCheckable(true);
    aspectRatio43->setData(1);

    aspectRatioActionGroup->addAction(aspectRatio43);

    QAction *aspectRatio169 = m_aspectRatioMenu->addAction(i18n("16:9"));
    aspectRatio169->setCheckable(true);
    aspectRatio169->setData(2);

    aspectRatioActionGroup->addAction(aspectRatio169);

    QAction *aspectRatioFitToWindow = m_aspectRatioMenu->addAction(i18n("Fit to Window"));
    aspectRatioFitToWindow->setCheckable(true);
    aspectRatioFitToWindow->setData(3);

    aspectRatioActionGroup->addAction(aspectRatioFitToWindow);

    videoMenu->addMenu(m_aspectRatioMenu);
    videoMenu->addSeparator();

    m_subtitleMenu = new KMenu();
    m_subtitleMenu->setTitle(i18n("Subtitles"));
    m_subtitleMenu->setEnabled(false);

    videoMenu->addMenu(m_subtitleMenu);

    m_chapterMenu = new KMenu();
    m_chapterMenu->setTitle(i18n("Chapter"));
    m_chapterMenu->setEnabled(false);

    videoMenu->addMenu(m_chapterMenu);

    m_angleMenu = new KMenu();
    m_angleMenu->setTitle(i18n("Angle"));
    m_angleMenu->setEnabled(false);

    videoMenu->addMenu(m_angleMenu);
    videoMenu->addSeparator();

    m_fullScreenAction = new KAction(KIcon("view-fullscreen"), i18n("Full Screen Mode"), this);
    m_fullScreenAction->setShortcut(QKeySequence(Qt::Key_F));

    videoMenu->addAction(m_fullScreenAction);

    m_videoAction = new KAction(i18n("Video"), this);
    m_videoAction->setMenu(videoMenu);

    m_controlsAction = new KAction(KIcon("show-menu"), i18n("Hide Controls"), this);
    m_controlsAction->setShortcut(QKeySequence(Qt::Key_C));

    m_playListAction = new KAction(KIcon("view-media-playlist"), i18n("Play list"), this);
    m_playListAction->setShortcut(QKeySequence(Qt::Key_P));

    QAction *separator1 = new QAction(this);
    separator1->setSeparator(true);

    QAction *separator2 = new QAction(this);
    separator2->setSeparator(true);

    QAction *separator3 = new QAction(this);
    separator3->setSeparator(true);

    QAction *separator4 = new QAction(this);
    separator4->setSeparator(true);

    QAction *separator5 = new QAction(this);
    separator5->setSeparator(true);

    QAction *separator6 = new QAction(this);
    separator6->setSeparator(true);

    QAction *separator7 = new QAction(this);
    separator7->setSeparator(true);

    m_actions.append(openAction);
    m_actions.append(separator1);
    m_actions.append(m_playPauseAction);
    m_actions.append(m_stopAction);
    m_actions.append(separator2);
    m_actions.append(m_controlsAction);
    m_actions.append(separator3);
    m_actions.append(m_navigationAction);
    m_actions.append(separator4);
    m_actions.append(audioAction);
    m_actions.append(separator5);
    m_actions.append(m_videoAction);
    m_actions.append(separator6);
    m_actions.append(m_playListAction);
    m_actions.append(separator7);

    m_controls = new QGraphicsWidget(this);
    m_controls->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));

    Plasma::ToolButton *openButton = new Plasma::ToolButton(m_controls);
    openButton->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));
    openButton->nativeWidget()->setDefaultAction(openFileAction);

    Plasma::ToolButton *playPauseButton = new Plasma::ToolButton(m_controls);
    playPauseButton->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));
    playPauseButton->nativeWidget()->setDefaultAction(m_playPauseAction);

    Plasma::ToolButton *stopButton = new Plasma::ToolButton(m_controls);
    stopButton->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));
    stopButton->nativeWidget()->setDefaultAction(m_stopAction);

    m_seekSlider = new Plasma::Slider(m_controls);
    m_seekSlider->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));
    m_seekSlider->setOrientation(Qt::Horizontal);
    m_seekSlider->setToolTip(i18n("Position"));
    m_seekSlider->setRange(0, 100);
    m_seekSlider->setEnabled(false);

    m_volumeButton = new Plasma::ToolButton(m_controls);
    m_volumeButton->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));
    m_volumeButton->nativeWidget()->setDefaultAction(m_audioAction);

    QGraphicsLinearLayout *controlsLayout = new QGraphicsLinearLayout(Qt::Horizontal);
    controlsLayout->setSpacing(0);
    controlsLayout->setContentsMargins(0, 0, 0, 0);
    controlsLayout->addItem(openButton);
    controlsLayout->addItem(playPauseButton);
    controlsLayout->addItem(stopButton);
    controlsLayout->addItem(m_seekSlider);
    controlsLayout->addItem(m_volumeButton);

    m_controls->setLayout(controlsLayout);

    m_mainLayout->insertItem(1, m_controls);

    m_playListUi.playPauseButton->setDefaultAction(m_playPauseAction);
    m_playListUi.stopButton->setDefaultAction(m_stopAction);
    m_playListUi.slidersLayout->addWidget(new Phonon::SeekSlider(m_mediaObject, m_playListDialog));
    m_playListUi.slidersLayout->addWidget(new Phonon::VolumeSlider(m_audioOutput, m_playListDialog));
    m_playListUi.slidersLayout->setStretch(0, 3);

    m_fullScreenUi.playPauseButton->setDefaultAction(m_playPauseAction);
    m_fullScreenUi.stopButton->setDefaultAction(m_stopAction);
    m_fullScreenUi.slidersLayout->addWidget(new Phonon::SeekSlider(m_mediaObject, m_playListDialog));
    m_fullScreenUi.slidersLayout->addWidget(new Phonon::VolumeSlider(m_audioOutput, m_playListDialog));
    m_fullScreenUi.slidersLayout->setStretch(0, 3);
    m_fullScreenUi.fullScreenButton->setDefaultAction(m_fullScreenAction);

    m_jumpToPositionDialog = new KDialog(m_playListDialog);
    m_jumpToPositionDialog->setCaption(i18n("Jump to Position"));
    m_jumpToPositionDialog->setButtons(KDialog::Ok | KDialog::Cancel);
    m_jumpToPositionDialog->setWindowModality(Qt::NonModal);

    QWidget *jumpToPositionWidget = new QWidget;

    m_jumpToPositionUi.setupUi(jumpToPositionWidget);

    m_jumpToPositionDialog->setMainWidget(jumpToPositionWidget);

    foreach (Solid::Device device, Solid::Device::listFromType(Solid::DeviceInterface::OpticalDisc, QString()))
    {
        addDevice(device.udi());
    }

    QDBusConnection dbus = QDBusConnection::sessionBus();
    dbus.registerService("org.mpris.PlasmaMiniPlayerApplet");
    dbus.registerObject("/PlasmaMiniPlayerApplet", this);

    if (args.count())
    {
        KUrl::List tracks;

        for (int i = 0; i < args.count(); ++i)
        {
            tracks.append(KUrl(args.value(i).toString()));
        }

        createPlayList(i18n("Default"), tracks);
    }

    connect(this, SIGNAL(activate()), this, SLOT(togglePlayListDialog()));
    connect(this, SIGNAL(destroyed()), m_playListDialog, SLOT(deleteLater()));
    connect(this, SIGNAL(destroyed()), m_volumeDialog, SLOT(deleteLater()));
    connect(this, SIGNAL(destroyed()), m_jumpToPositionDialog, SLOT(deleteLater()));
    connect(this, SIGNAL(destroyed()), m_videoWidget, SLOT(deleteLater()));
    connect(this, SIGNAL(destroyed()), m_fullScreen, SLOT(deleteLater()));
    connect(m_hideFullScreenControls, SIGNAL(timeout()), this, SLOT(hideFullScreenControls()));
    connect(m_showPlayList, SIGNAL(timeout()), this, SLOT(togglePlayListDialog()));
    connect(m_hideToolTip, SIGNAL(timeout()), this, SLOT(hideToolTip()));
    connect(m_mediaObject, SIGNAL(finished()), this, SLOT(trackFinished()));
    connect(m_mediaObject, SIGNAL(tick(qint64)), this, SLOT(ticked(qint64)));
    connect(m_mediaObject, SIGNAL(stateChanged(Phonon::State, Phonon::State)), this, SLOT(stateChanged(Phonon::State, Phonon::State)));
    connect(m_mediaObject, SIGNAL(hasVideoChanged(bool)), this, SLOT(hasVideoChanged(bool)));
    connect(m_mediaObject, SIGNAL(metaDataChanged()), this, SLOT(metaDataChanged()));
    connect(m_audioOutput, SIGNAL(mutedChanged(bool)), this, SLOT(mutedChanged(bool)));
    connect(m_audioOutput, SIGNAL(volumeChanged(qreal)), this, SLOT(volumeChanged(qreal)));
    connect(m_seekSlider, SIGNAL(sliderMoved(int)), this, SLOT(setPosition(int)));
    connect(m_jumpToPositionDialog, SIGNAL(okClicked()), this, SLOT(jumpToPosition()));
    connect(m_playListUi.splitter, SIGNAL(splitterMoved(int, int)), this, SLOT(savePlayListSettings(int, int)));
    connect(m_playListDialog, SIGNAL(dialogResized()), this, SLOT(savePlayListSettings()));
    connect(m_playListTabBar, SIGNAL(newTabRequest()), this, SLOT(newPlayList()));
    connect(m_playListTabBar, SIGNAL(tabDoubleClicked(int)), this, SLOT(renamePlayList(int)));
    connect(m_playListTabBar, SIGNAL(closeRequest(int)), this, SLOT(removePlayList(int)));
    connect(m_playListTabBar, SIGNAL(currentChanged(int)), this, SLOT(visiblePlayListChanged(int)));
    connect(m_playListTabBar, SIGNAL(tabMoved(int, int)), this, SLOT(movePlayList(int, int)));
    connect(m_playListUi.closeButton, SIGNAL(clicked()), m_playListDialog, SLOT(close()));
    connect(m_playListUi.addButton, SIGNAL(clicked()), this, SLOT(addToPlayList()));
    connect(m_playListUi.removeButton, SIGNAL(clicked()), this, SLOT(removeTrack()));
    connect(m_playListUi.editButton, SIGNAL(clicked()), this, SLOT(editTrackTitle()));
    connect(m_playListUi.moveUpButton, SIGNAL(clicked()), this, SLOT(moveUpTrack()));
    connect(m_playListUi.moveDownButton, SIGNAL(clicked()), this, SLOT(moveDownTrack()));
    connect(m_playListUi.newButton, SIGNAL(clicked()), this, SLOT(newPlayList()));
    connect(m_playListUi.exportButton, SIGNAL(clicked()), this, SLOT(exportPlayList()));
    connect(m_playListUi.clearButton, SIGNAL(clicked()), this, SLOT(clearPlayList()));
    connect(m_playListUi.randomButton, SIGNAL(toggled(bool)), this, SLOT(setRandomTrack(bool)));
    connect(m_playListUi.playListView, SIGNAL(pressed(QModelIndex)), this, SLOT(trackPressed(QModelIndex)));
    connect(m_playListUi.playListView, SIGNAL(activated(QModelIndex)), this, SLOT(playTrack(QModelIndex)));
    connect(m_playListUi.playListViewFilter, SIGNAL(textChanged(QString)), this, SLOT(filterPlayList(QString)));
    connect(m_fullScreenAction, SIGNAL(triggered()), this, SLOT(toggleFullScreen()));
    connect(m_controlsAction, SIGNAL(triggered()), this, SLOT(toggleControls()));
    connect(m_playListAction, SIGNAL(triggered()), this, SLOT(togglePlayListDialog()));
    connect(openFileAction, SIGNAL(triggered()), this, SLOT(openFiles()));
    connect(openUrlAction, SIGNAL(triggered()), this, SLOT(openUrl()));
    connect(m_playPauseAction, SIGNAL(triggered()), this, SLOT(playPause()));
    connect(m_stopAction, SIGNAL(triggered()), this, SLOT(stop()));
    connect(playPreviousAction, SIGNAL(triggered()), this, SLOT(playPrevious()));
    connect(playNextAction, SIGNAL(triggered()), this, SLOT(playNext()));
    connect(seekBackwardAction, SIGNAL(triggered()), this, SLOT(seekBackward()));
    connect(seekForwardAction, SIGNAL(triggered()), this, SLOT(seekForward()));
    connect(jumpToPositionAction, SIGNAL(triggered()), this, SLOT(toggleJumpToPosition()));
    connect(increaseVolumeAction, SIGNAL(triggered()), this, SLOT(increaseVolume()));
    connect(decreaseVolumeAction, SIGNAL(triggered()), this, SLOT(decreaseVolume()));
    connect(m_muteVolumeAction, SIGNAL(triggered()), this, SLOT(toggleMute()));
    connect(m_audioAction, SIGNAL(triggered()), this, SLOT(toggleVolumeDialog()));
    connect(m_openMenu, SIGNAL(triggered(QAction*)), this, SLOT(openDisc(QAction*)));
    connect(m_aspectRatioMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeAspectRatio(QAction*)));
    connect(m_subtitleMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeSubtitles(QAction*)));
    connect(m_chapterMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeChapter(QAction*)));
    connect(m_angleMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeAngle(QAction*)));
    connect(m_repeatModeMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeRepeatMode(QAction*)));
    connect(m_audioChannelMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeAudioChannel(QAction*)));
    connect(Solid::DeviceNotifier::instance(), SIGNAL(deviceAdded(QString)), this, SLOT(addDevice(QString)));
    connect(Solid::DeviceNotifier::instance(), SIGNAL(deviceRemoved(QString)), this, SLOT(removeDevice(QString)));
    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(updateTheme()));
}

void MiniPlayerApplet::init()
{
    KConfigGroup configuration = config();
    QStringList playLists = configuration.readEntry("playLists", QStringList(i18n("Default")));

    m_visiblePlayList = configuration.readEntry("currentPlayList", i18n("Default"));

    m_playListDialog->resize(configuration.readEntry("playListSize", m_playListDialog->size()));

    m_playListUi.splitter->restoreState(configuration.readEntry("playListSplitter", QByteArray()));

    KUrl::List tracks;
    QStringList titles;
    QString playList = m_visiblePlayList;
    int currentIndex = 0;
    bool playMedia = true;

    for (int i = 0; i < playLists.count(); ++i)
    {
        tracks = KUrl::List(configuration.readEntry("playListUrls-" + playLists.at(i), QStringList()));

        titles = configuration.readEntry("playListTitles-" + playLists.at(i), QStringList());

        for (int j = 0; j < titles.count(); ++j)
        {
            if (titles.at(j).isEmpty())
            {
                continue;
            }

            m_titles[tracks.at(j)] = titles.at(j);
        }

        createPlayList(playLists.at(i), tracks);

        if (playLists.at(i) == playList)
        {
            currentIndex = i;
        }
    }

    setActivePlayList(playList);

    m_playListTabBar->setCurrentIndex(currentIndex);

    if (m_playListTabBar->count() == 1)
    {
        m_playListTabBar->hide();
    }

    if (!configuration.readEntry("playOnStartup", false))
    {
        playMedia = false;
    }

    if (m_playLists[m_visiblePlayList]->getTracks().count())
    {
        KUrl currentTrack;

        currentTrack = configuration.readEntry("currentTrack", KUrl());

        savePlayList();

        updateControls(m_mediaObject->state());

        if (playMedia)
        {
            if (!currentTrack.isValid())
            {
                currentTrack = m_playLists[m_visiblePlayList]->getTracks().first();
            }

            play(currentTrack);
        }
    }

    constraintsEvent(Plasma::FormFactorConstraint);

    hasVideoChanged(false);

    configReset();

    updateTheme();

    new PlayerDBusHandler(this);
    new TrackListDBusHandler(this);
    new RootDBusHandler(this);
}

void MiniPlayerApplet::createConfigurationInterface(KConfigDialog *parent)
{
    KConfigGroup configuration = config();
    QWidget *generalWidget = new QWidget;
    QWidget *videoWidget = new QWidget;

    m_videoUi.setupUi(videoWidget);
    m_generalUi.setupUi(generalWidget);

    m_generalUi.playOnStartup->setChecked(configuration.readEntry("playOnStartup", false));
    m_generalUi.showVideoInDialog->setChecked(configuration.readEntry("showVideoInDialog", true));
    m_generalUi.showToolTipOnTrackChange->setValue(configuration.readEntry("showToolTipOnTrackChange", 5));

    m_videoUi.brightnessSlider->setValue(m_videoWidget->brightness() * 100);
    m_videoUi.contrastSlider->setValue(m_videoWidget->contrast() * 100);
    m_videoUi.hueSlider->setValue(m_videoWidget->hue() * 100);
    m_videoUi.saturationSlider->setValue(m_videoWidget->saturation() * 100);

    parent->addPage(generalWidget, i18n("General"), "go-home");
    parent->addPage(videoWidget, i18n("Video settings"), "video-display");

    connect(m_videoUi.brightnessSlider, SIGNAL(valueChanged(int)), this, SLOT(setBrightness(int)));
    connect(m_videoUi.contrastSlider, SIGNAL(valueChanged(int)), this, SLOT(setContrast(int)));
    connect(m_videoUi.hueSlider, SIGNAL(valueChanged(int)), this, SLOT(setHue(int)));
    connect(m_videoUi.saturationSlider, SIGNAL(valueChanged(int)), this, SLOT(setSaturation(int)));
    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(finished()), this, SLOT(configReset()));
}

void MiniPlayerApplet::configAccepted()
{
    KConfigGroup configuration = config();

    configuration.writeEntry("playOnStartup", m_generalUi.playOnStartup->isChecked());
    configuration.writeEntry("showToolTipOnTrackChange", m_generalUi.showToolTipOnTrackChange->value());
    configuration.writeEntry("brightness", m_videoUi.brightnessSlider->value());
    configuration.writeEntry("contrast", m_videoUi.contrastSlider->value());
    configuration.writeEntry("hue", m_videoUi.hueSlider->value());
    configuration.writeEntry("saturation",m_videoUi.saturationSlider->value());

    showVideoInDialog(m_generalUi.showVideoInDialog->isChecked());

    emit configNeedsSaving();

    configReset();
}

void MiniPlayerApplet::configReset()
{
    KConfigGroup configuration = config();

    setBrightness(configuration.readEntry("brightness", 0));

    setContrast(configuration.readEntry("contrast", 0));

    setHue(configuration.readEntry("hue", 0));

    setSaturation(configuration.readEntry("saturation", 0));

    setAspectRatio(configuration.readEntry("saturation", 0));

    m_audioOutput->setMuted(configuration.readEntry("mute", false));

    mutedChanged(configuration.readEntry("mute", false));

    m_audioOutput->setVolume(configuration.readEntry("volume", 0.8));

    volumeChanged(configuration.readEntry("volume", 0.8));

    setRepeatMode(configuration.readEntry("repeatMode", 2));

    setRandomTrack(configuration.readEntry("randomTrack", false));

    showVideoInDialog(configuration.readEntry("showVideoInDialog", true));
}

void MiniPlayerApplet::constraintsEvent(Plasma::Constraints constraints)
{
    if (constraints & Plasma::FormFactorConstraint && (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical))
    {
        showVideoInDialog(true);
    }

    if ((constraints & Plasma::LocationConstraint || constraints & Plasma::SizeConstraint) && m_playListDialog->isVisible())
    {
        m_playListDialog->move(popupPosition(m_playListDialog->size()));
    }
}

void MiniPlayerApplet::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
{
    if (!event->mimeData()->hasUrls())
    {
        event->ignore();

        return;
    }

    if (!m_playListDialog->isVisible())
    {
        m_showPlayList->start(500);
    }
}

void MiniPlayerApplet::dragLeaveEvent(QGraphicsSceneDragDropEvent *event)
{
    Q_UNUSED(event)

    m_showPlayList->stop();
}

void MiniPlayerApplet::dropEvent(QGraphicsSceneDragDropEvent *event)
{
    m_showPlayList->stop();

    addToPlayList(KUrl::List::fromMimeData(event->mimeData()), true);
}

void MiniPlayerApplet::keyPressEvent(QKeyEvent *event)
{
    switch (event->key())
    {
        case Qt::Key_PageDown:
            playPrevious();
        break;
        case Qt::Key_PageUp:
            playNext();
        break;
        case Qt::Key_Left:
            seekBackward();
        break;
        case Qt::Key_Right:
            seekForward();
        break;
        case Qt::Key_Plus:
            increaseVolume();
        break;
        case Qt::Key_Minus:
            decreaseVolume();
        break;
        case Qt::Key_Space:
            playPause();
        break;
        case Qt::Key_Escape:
            if (m_fullScreen->isFullScreen())
            {
                toggleFullScreen();
            }
            else if (m_playListDialog->isVisible())
            {
                m_playListDialog->close();
            }
        break;
        case Qt::Key_C:
            toggleControls();
        break;
        case Qt::Key_F:
            toggleFullScreen();
        break;
        case Qt::Key_G:
            toggleJumpToPosition();
        break;
        case Qt::Key_M:
            toggleMute();
        break;
        case Qt::Key_O:
            openFiles();
        break;
        case Qt::Key_P:
            togglePlayListDialog();
        break;
        case Qt::Key_S:
            stop();
        break;
        case Qt::Key_U:
            openUrl();
        break;
        case Qt::Key_V:
            toggleVolumeDialog();
        break;
        default:
            event->ignore();
        break;
    }
}

void MiniPlayerApplet::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    if (event->button() == Qt::LeftButton)
    {
        if (m_lastClick.secsTo(QDateTime::currentDateTime()) < 2)
        {
            togglePlayListDialog();

            m_lastClick= QDateTime();
        }
        else
        {
            m_lastClick = QDateTime::currentDateTime();
        }
    }

    event->ignore();
}

void MiniPlayerApplet::stateChanged(Phonon::State newState, Phonon::State oldState)
{
    if (newState == Phonon::ErrorState && oldState != Phonon::ErrorState)
    {
        KMessageBox::error(NULL, m_mediaObject->currentSource().url().toString().replace("%20", " ") + "\n\n" + m_mediaObject->errorString());
    }

    if (m_mediaObject->hasVideo() && newState == Phonon::PlayingState)
    {
        m_stopSleepCookie = Solid::PowerManagement::beginSuppressingSleep("Plasma MiniPlayerApplet: playing video");

        if (!m_notificationRestrictions)
        {
            m_notificationRestrictions = new KNotificationRestrictions(KNotificationRestrictions::NonCriticalServices, this);
        }
    }
    else if (m_notificationRestrictions)
    {
        Solid::PowerManagement::stopSuppressingSleep(m_stopSleepCookie);

        delete m_notificationRestrictions;

        m_notificationRestrictions = NULL;
    }

    updateControls(newState);
}

void MiniPlayerApplet::hasVideoChanged(bool hasVideo)
{
    KConfigGroup configuration = config();

    m_videoAction->setEnabled(hasVideo);

    m_controlsAction->setEnabled(hasVideo && m_proxyWidget->widget());

    toggleProxyWidget(hasVideo);

    if (!hasVideo && !m_controls->isVisible())
    {
        toggleControls();
    }

    if (!hasVideo && m_fullScreen->isFullScreen())
    {
        toggleFullScreen();
    }

    if (configuration.readEntry("showVideoInDialog", true))
    {
        m_playListUi.blankLabel->setVisible(!hasVideo);

        m_videoWidget->setVisible(hasVideo);
    }
}

void MiniPlayerApplet::volumeChanged(qreal volume)
{
    KConfigGroup configuration = config();

    configuration.writeEntry("volume", volume);

    emit configNeedsSaving();

    if (!m_audioOutput->isMuted())
    {
        m_audioAction->setText(i18n("Volume: %1%", (volume * 100)));
    }
}

void MiniPlayerApplet::mutedChanged(bool muted)
{
    KConfigGroup configuration = config();
    configuration.writeEntry("muted", muted);

    emit configNeedsSaving();

    m_muteVolumeAction->setIcon(KIcon(muted?"audio-volume-muted":"audio-volume-medium"));
    m_muteVolumeAction->setText(muted?i18n("Unmute Volume"):i18n("Mute Volume"));

    m_audioAction->setIcon(KIcon(muted?"player-volume-muted":"player-volume"));
    m_audioAction->setText(muted?i18n("Muted"):i18n("Volume: %1%", (m_audioOutput->volume() * 100)));
}

void MiniPlayerApplet::metaDataChanged()
{
    QStringList titles = m_mediaObject->metaData(Phonon::TitleMetaData);
    KUrl url;

    if (m_mediaObject->currentSource().type() == Phonon::MediaSource::Disc)
    {
        url = KUrl(QString("disc:/%1/%2").arg(m_mediaObject->currentSource().deviceName()).arg(m_mediaController->currentTitle()));
    }
    else 
    {
        url = m_mediaObject->currentSource().url();
    }

    if ((!titles.isEmpty() || m_mediaObject->currentSource().type() == Phonon::MediaSource::Disc) && titles.first().length() > 0)
    {
        m_title = titles.first();

        if (!m_titles.contains(url) || m_titles[url].isEmpty() || m_mediaObject->currentSource().type() == Phonon::MediaSource::Disc)
        {
            m_titles[url] = m_title;

            savePlayList();
        }
    }
    else if (m_titles.contains(url) && !m_titles[url].isEmpty())
    {
        m_title = m_titles[url];
    }
    else
    {
        QFileInfo *location = new QFileInfo(m_mediaObject->currentSource().url().toString());

        m_title = location->completeBaseName().replace("%20", " ");

        delete location;
    }

    updateToolTip();

    m_playListUi.titleLabel->setText(m_title);
    m_fullScreenUi.titleLabel->setText(m_title);

    QTimer::singleShot(500, this, SLOT(showToolTip()));
}

void MiniPlayerApplet::ticked(qint64 position)
{
    if (m_mediaObject->totalTime() > 0)
    {
        QString tip = i18n("Position: %1 / %2", timeToString(position), timeToString(m_mediaObject->totalTime()));

        m_playListUi.slidersLayout->itemAt(0)->widget()->setToolTip(tip);

        m_fullScreenUi.slidersLayout->itemAt(0)->widget()->setToolTip(tip);

        m_seekSlider->setToolTip(tip);
        m_seekSlider->setValue(position);
    }
    else
    {
        m_playListUi.slidersLayout->itemAt(0)->widget()->setToolTip(i18n("Position"));

        m_fullScreenUi.slidersLayout->itemAt(0)->widget()->setToolTip(i18n("Position"));

        m_seekSlider->setToolTip(i18n("Position"));
        m_seekSlider->setValue(0);
    }

    if (Plasma::ToolTipManager::self()->isVisible(this))
    {
        updateToolTip();
    }
}

void MiniPlayerApplet::trackFinished()
{
    KConfigGroup configuration = config();
    RepeatMode repeatMode = static_cast<RepeatMode>(configuration.readEntry("repeatMode", 2));

    m_playListUi.titleLabel->setText(QString());
    m_fullScreenUi.titleLabel->setText(QString());

    if (!m_playLists[m_activePlayList]->getTracks().isEmpty())
    {
        KUrl url = currentUrl();

        if (configuration.readEntry("randomTrack", false) && m_playLists[m_activePlayList]->getTracks().count() > 1)
        {
            qsrand(QDateTime::currentDateTime().toTime_t());

            KRandomSequence *sequence = new KRandomSequence(qrand() % 1000);

            KUrl randomUrl = url;

            while (randomUrl == url)
            {
                randomUrl = m_playLists[m_activePlayList]->getTracks().at(sequence->getLong(m_playLists[m_activePlayList]->getTracks().count() - 1));
            }

            play(randomUrl);

            delete sequence;
        }
        else if (repeatMode == RepeatTrack && m_playLists[m_activePlayList]->getTracks().indexOf(url) > -1)
        {
            play(url);
        }
        else if (repeatMode == RepeatPlayList)
        {
            if (url == m_playLists[m_activePlayList]->getTracks().last())
            {
                play(m_playLists[m_activePlayList]->getTracks().first());
            }
            else if (m_playLists[m_activePlayList]->getTracks().indexOf(url) > -1)
            {
                play(m_playLists[m_activePlayList]->getTracks().at(m_playLists[m_activePlayList]->getTracks().indexOf(url) + 1));
            }
            else
            {
                play(m_playLists[m_activePlayList]->getTracks().at(0));
            }
        }
        else if (m_playLists[m_activePlayList]->getTracks().indexOf(url) > -1 && url != m_playLists[m_activePlayList]->getTracks().last())
        {
            play(m_playLists[m_activePlayList]->getTracks().at(m_playLists[m_activePlayList]->getTracks().indexOf(url) + 1));
        }
    }

    hasVideoChanged(m_mediaObject->hasVideo());
}

void MiniPlayerApplet::trackPressed(QModelIndex index)
{
    Q_UNUSED(index)

    QModelIndexList selectedIndexes = m_playListUi.playListView->selectionModel()->selectedIndexes();
    bool hasTracks = m_playLists[m_visiblePlayList]->getTracks().count();
    bool isDisc = m_discTitles.contains(m_visiblePlayList);

    m_playListUi.addButton->setEnabled(!isDisc);
    m_playListUi.removeButton->setEnabled(!selectedIndexes.isEmpty());
    m_playListUi.editButton->setEnabled(!selectedIndexes.isEmpty() && !isDisc);
    m_playListUi.moveUpButton->setEnabled(!selectedIndexes.isEmpty() && selectedIndexes.first().row() != 0);
    m_playListUi.moveDownButton->setEnabled(!selectedIndexes.isEmpty() && selectedIndexes.last().row() != (m_playLists[m_visiblePlayList]->getTracks().count() - 1));
    m_playListUi.clearButton->setEnabled(hasTracks && !isDisc);
    m_playListUi.randomButton->setEnabled(hasTracks);
    m_playListUi.repeatButton->setEnabled(hasTracks);
    m_playListUi.exportButton->setEnabled(hasTracks && !isDisc);
    m_playListUi.playListViewFilter->setEnabled(hasTracks);

    m_editorActive = false;
}

void MiniPlayerApplet::trackChanged(QModelIndex index)
{
    Q_UNUSED(index)

    m_editorActive = false;
}

void MiniPlayerApplet::moveUpTrack()
{
    QModelIndex from = m_playListUi.playListView->currentIndex();
    QModelIndex to = m_playLists[m_visiblePlayList]->index((m_playListUi.playListView->currentIndex().row() - 1), 0);
    QString url = from.data(Qt::ToolTipRole).toString();

    m_playLists[m_visiblePlayList]->setData(from, to.data(Qt::ToolTipRole), Qt::ToolTipRole);
    m_playLists[m_visiblePlayList]->setData(to, url, Qt::ToolTipRole);

    m_playListUi.playListView->setCurrentIndex(to);

    trackPressed(to);
}

void MiniPlayerApplet::moveDownTrack()
{
    QModelIndex from = m_playListUi.playListView->currentIndex();
    QModelIndex to = m_playLists[m_visiblePlayList]->index((m_playListUi.playListView->currentIndex().row() + 1), 0);
    QString url = from.data(Qt::ToolTipRole).toString();

    m_playLists[m_visiblePlayList]->setData(from, to.data(Qt::ToolTipRole), Qt::ToolTipRole);
    m_playLists[m_visiblePlayList]->setData(to, url, Qt::ToolTipRole);

    m_playListUi.playListView->setCurrentIndex(to);

    trackPressed(to);
}

void MiniPlayerApplet::playTrack(QModelIndex index)
{
    if (!index.isValid())
    {
        index = m_playListUi.playListView->currentIndex();
    }

    KUrl url = (index.isValid()?KUrl(index.data(Qt::ToolTipRole).toString()):(m_playLists[m_activePlayList]->getTracks().count()?m_playLists[m_activePlayList]->getTracks().at(0):KUrl()));

    setActivePlayList(m_visiblePlayList);

    play(url);
}

void MiniPlayerApplet::editTrackTitle()
{
    m_editorActive = true;

    m_playListUi.playListView->edit(m_playListUi.playListView->currentIndex());
}

void MiniPlayerApplet::copyTrackUrl()
{
    QApplication::clipboard()->setText(m_playListUi.playListView->currentIndex().data(Qt::ToolTipRole).toString());
}

void MiniPlayerApplet::removeTrack()
{
    int position = m_playListUi.playListView->currentIndex().row();

    removeFromPlayList(KUrl::List(KUrl(m_playListUi.playListView->currentIndex().data(Qt::ToolTipRole).toString())));

    m_playListUi.playListView->setCurrentIndex(m_playLists[m_visiblePlayList]->index(position, 0));
}

void MiniPlayerApplet::openUrl()
{
    addToPlayList(KUrl::List(KInputDialog::getText(i18n("Open URL"), i18n("Enter a URL:"))), true);
}

void MiniPlayerApplet::openFiles(bool playFirst)
{
    KConfigGroup configuration = config();

    KFileDialog *dialog = new KFileDialog(KUrl(configuration.readEntry("directory", "~")), "", NULL);
    dialog->setMimeFilter(Phonon::BackendCapabilities::availableMimeTypes() << "audio/x-scpls" << "audio/x-mpegurl" << "application/xspf+xml" << "audio/x-ms-asx");
    dialog->setWindowModality(Qt::NonModal);
    dialog->setMode(KFile::Files | KFile::Directory);
    dialog->setOperationMode(KFileDialog::Opening);
    dialog->exec();

    KUrl::List urls = dialog->selectedUrls();

    delete dialog;

    if (urls.isEmpty())
    {
        return;
    }

    QFileInfo *location = new QFileInfo(urls.at(0).toLocalFile());

    configuration.writeEntry("directory", location->absolutePath());

    delete location;

    emit configNeedsSaving();

    addToPlayList(urls, playFirst);
}

void MiniPlayerApplet::openDisc(QAction *action)
{
    QString udi = action->data().toString();

    if (!m_discActions.contains(udi))
    {
        return;
    }

    QString title = m_discTitles[udi];

    m_mediaObject->setCurrentSource(Phonon::MediaSource(m_discTypes[udi], udi));

    if (!m_mediaController->availableTitles())
    {
        return;
    }


    KUrl::List tracks;

    for (int i = 1; i <= m_mediaController->availableTitles(); ++i)
    {
        KUrl url = KUrl(QString("disc:/%1/%2").arg(udi).arg(i));

        tracks.append(url);

        m_titles[url] = i18n("Track %1", i);
    }

    m_discPlayLists[title] = udi;

    m_visiblePlayList = title;

    createPlayList(title, tracks);

    setActivePlayList(title);

    m_mediaObject->play();

    play(tracks.at(0));

    m_playListDialog->show();
}

void MiniPlayerApplet::play(KUrl url)
{
    if (url.protocol() == "disc")
    {
        QString device = deviceForUrl(url);

        if (!device.isEmpty())
        {
            int track = url.url().right(url.url().length() - url.url().lastIndexOf("/") - 1).toInt();

            if (track < 1)
            {
                return;
            }

            m_mediaObject->setCurrentSource(Phonon::MediaSource(m_discTypes[device], device));

            if (track > m_mediaController->availableTitles())
            {
                track = 1;
            }

            m_mediaController->setCurrentTitle(track);
        }
        else
        {
            return;
        }
    }
    else if (!url.isValid())
    {
        return;
    }
    else
    {
        KConfigGroup configuration = config();
        configuration.writeEntry("currentTrack", url);

        emit configNeedsSaving();

        m_mediaObject->setCurrentSource(Phonon::MediaSource(url));
    }

    m_mediaObject->play();

    hasVideoChanged(m_mediaObject->hasVideo());

    m_videoWidget->setWindowTitle(m_title);

    m_playListUi.titleLabel->setText(m_title);
    m_fullScreenUi.titleLabel->setText(m_title);

    m_playLists[m_visiblePlayList]->setCurrentTrack(m_playLists[m_visiblePlayList]->getTracks().indexOf(url));

    m_audioChannelMenu->clear();
    m_audioChannelMenu->setEnabled(false);

    QActionGroup *audioChannelActionGroup = new QActionGroup(m_audioChannelMenu);

    if (m_mediaController->availableAudioChannels().count() > 1)
    {
        for (int i = 0; i < m_mediaController->availableAudioChannels().count(); ++i)
        {
            QAction *action = m_audioChannelMenu->addAction(m_mediaController->availableAudioChannels().at(i).name());
            action->setData(i);
            action->setCheckable(true);

            audioChannelActionGroup->addAction(action);
        }

        m_audioChannelMenu->setEnabled(true);
    }

    m_subtitleMenu->clear();
    m_subtitleMenu->setEnabled(false);

    QActionGroup *subtitleActionGroup = new QActionGroup(m_subtitleMenu);

    if (m_mediaController->availableSubtitles().count())
    {
        for (int i = 0; i < m_mediaController->availableSubtitles().count(); ++i)
        {
            QAction *action = m_subtitleMenu->addAction(m_mediaController->availableSubtitles().at(i).name());
            action->setData(i);
            action->setCheckable(true);

            subtitleActionGroup->addAction(action);
        }

        m_subtitleMenu->setEnabled(true);
    }

    m_chapterMenu->clear();
    m_chapterMenu->setEnabled(false);

    QActionGroup *chapterActionGroup = new QActionGroup(m_chapterMenu);

    if (m_mediaController->availableChapters() > 1)
    {
        for (int i = 0; i < m_mediaController->availableChapters(); ++i)
        {
            QAction *action = m_chapterMenu->addAction(i18n("Chapter %1", i));
            action->setData(i);
            action->setCheckable(true);

            chapterActionGroup->addAction(action);
        }

        m_chapterMenu->setEnabled(true);
    }

    m_angleMenu->clear();
    m_angleMenu->setEnabled(false);

    QActionGroup *angleActionGroup = new QActionGroup(m_angleMenu);

    if (m_mediaController->availableAngles() > 1)
    {
        for (int i = 0; i < m_mediaController->availableAngles(); ++i)
        {
            QAction *action = m_angleMenu->addAction(i18n("Angle %1", i));
            action->setData(i);
            action->setCheckable(true);

            angleActionGroup->addAction(action);
        }

        m_angleMenu->setEnabled(true);
    }

    QTimer::singleShot(500, this, SLOT(showToolTip()));
}

void MiniPlayerApplet::playPause()
{
    if (m_mediaObject->state() == Phonon::PlayingState)
    {
        pause();
    }
    else
    {
        play();
    }
}

void MiniPlayerApplet::playPrevious()
{
    if(!m_playLists[m_activePlayList]->getTracks().isEmpty())
    {
        if (m_playLists[m_activePlayList]->getTracks().at(0) == m_mediaObject->currentSource().url().toString() || m_playLists[m_activePlayList]->getTracks().count() == 1)
        {
            play(m_playLists[m_activePlayList]->getTracks().last());
        }
        else
        {
            play(m_playLists[m_activePlayList]->getTracks().at(m_playLists[m_activePlayList]->getTracks().indexOf(m_mediaObject->currentSource().url()) - 1));
        }
    }
}

void MiniPlayerApplet::playNext()
{
    if (!m_playLists[m_activePlayList]->getTracks().isEmpty())
    {
        if (m_playLists[m_activePlayList]->getTracks().last() == m_mediaObject->currentSource().url() || m_playLists[m_activePlayList]->getTracks().count() == 1)
        {
            play(m_playLists[m_activePlayList]->getTracks().first());
        }
        else
        {
            play(m_playLists[m_activePlayList]->getTracks().at(m_playLists[m_activePlayList]->getTracks().indexOf(m_mediaObject->currentSource().url()) + 1));
        }
    }
}

void MiniPlayerApplet::play()
{
    if (m_mediaObject->currentSource().url().isValid())
    {
        m_mediaObject->play();
    }
    else
    {
        playTrack();
    }

    hasVideoChanged(m_mediaObject->hasVideo());

    m_playListUi.titleLabel->setText(m_title);
    m_fullScreenUi.titleLabel->setText(m_title);
}

void MiniPlayerApplet::pause()
{
    m_mediaObject->pause();
}

void MiniPlayerApplet::stop()
{
    m_mediaObject->stop();

    m_playListUi.titleLabel->setText(QString());
    m_fullScreenUi.titleLabel->setText(QString());

    Plasma::ToolTipManager::self()->clearContent(this);

    m_seekSlider->setValue(0);

    hasVideoChanged(false);
}

void MiniPlayerApplet::seekBackward()
{
    setPosition(m_mediaObject->currentTime() - (m_mediaObject->totalTime() / 30));
}

void MiniPlayerApplet::seekForward()
{
    setPosition(m_mediaObject->currentTime() + (m_mediaObject->totalTime() / 30));
}

void MiniPlayerApplet::increaseVolume()
{
    m_audioOutput->setVolume(qMin(qreal(1.0), qreal(m_audioOutput->volume() + 0.1)));
}

void MiniPlayerApplet::decreaseVolume()
{
    m_audioOutput->setVolume(qMax(qreal(0.0), qreal(m_audioOutput->volume() - 0.1)));
}

void MiniPlayerApplet::movePlayList(int from, int to)
{
    Q_UNUSED(from)
    Q_UNUSED(to)

    savePlayListNames();
}

void MiniPlayerApplet::renamePlayList(int position)
{
    QString oldName = KGlobal::locale()->removeAcceleratorMarker(m_playListTabBar->tabText(position));

    if (m_discTitles.contains(oldName))
    {
        return;
    }

    QString newName = KInputDialog::getText(i18n("Rename play-list"), i18n("Enter name:"), oldName);

    if (newName.isEmpty() || m_playLists.contains(newName))
    {
        return;
    }

    KConfigGroup configuration = config();
    configuration.writeEntry("currentPlayList", newName);
    configuration.writeEntry("playListUrls-" + newName, configuration.readEntry("playListUrls-" + oldName, QStringList()));
    configuration.writeEntry("playListTitles-" + newName, configuration.readEntry("playListTitles-" + oldName, QStringList()));
    configuration.deleteEntry("playListUrls-" + oldName);
    configuration.deleteEntry("playListTitles-" + oldName);

    m_playLists[newName] = m_playLists[oldName];
    m_playLists.remove(oldName);

    if (m_activePlayList == oldName)
    {
        setActivePlayList(newName);
    }

    m_playListTabBar->setTabText(position, newName);

    savePlayListNames();

    emit configNeedsSaving();
}

void MiniPlayerApplet::removePlayList(int position)
{
    if (m_playListTabBar->count() == 1)
    {
        clearPlayList();

        return;
    }

    QString oldName = KGlobal::locale()->removeAcceleratorMarker(m_playListTabBar->tabText(position));

    m_playListTabBar->removeTab(position);

    QString newName = KGlobal::locale()->removeAcceleratorMarker(m_playListTabBar->tabText(m_playListTabBar->currentIndex()));

    m_playLists.remove(oldName);

    KConfigGroup configuration = config();
    configuration.writeEntry("currentPlayList", newName);
    configuration.deleteEntry("playListUrls-" + oldName);
    configuration.deleteEntry("playListTitles-" + oldName);

    if (m_activePlayList == oldName)
    {
        setActivePlayList(newName);
    }

    if (m_playListTabBar->count() == 1)
    {
        m_playListTabBar->hide();
    }

    m_discPlayLists.remove(oldName);

    savePlayListNames();

    emit configNeedsSaving();
}

void MiniPlayerApplet::visiblePlayListChanged(int position)
{
    QString name = KGlobal::locale()->removeAcceleratorMarker(m_playListTabBar->tabText(position));

    if (!m_playLists.contains(name))
    {
        return;
    }

    m_visiblePlayList = name;

    filterPlayList(m_playListUi.playListViewFilter->text());

    if (m_mediaObject->state() != Phonon::PlayingState && m_mediaObject->state() != Phonon::PausedState)
    {
        setActivePlayList(KGlobal::locale()->removeAcceleratorMarker(m_playListTabBar->tabText(position)));
    }

    m_playListUi.playListView->setModel(m_playLists[m_visiblePlayList]);

    savePlayListNames();

    updateControls(m_mediaObject->state());
}

void MiniPlayerApplet::jumpToPosition()
{
    if ((m_mediaObject->state() == Phonon::PlayingState || m_mediaObject->state() == Phonon::PausedState) && m_mediaObject->totalTime() > 0)
    {
        m_mediaObject->seek(-m_jumpToPositionUi.position->time().msecsTo(QTime()));
    }
}

void MiniPlayerApplet::toggleJumpToPosition()
{
    if ((m_mediaObject->state() == Phonon::PlayingState) || (m_mediaObject->state() == Phonon::PausedState))
    {
        QTime totalTime(0, 0);
        QTime currentTime(0, 0);

        m_jumpToPositionUi.position->setMaximumTime(totalTime.addMSecs(m_mediaObject->totalTime()));
        m_jumpToPositionUi.position->setTime(currentTime.addMSecs(m_mediaObject->currentTime()));

        m_jumpToPositionDialog->show();
    }
    else
    {
        m_jumpToPositionDialog->close();
    }
}

void MiniPlayerApplet::toggleVolumeDialog()
{
    if (m_volumeDialog->isVisible())
    {
        m_volumeDialog->close();
    }
    else
    {
        m_volumeDialog->move(containment()->corona()->popupPosition(m_volumeButton, m_volumeDialog->size()));
        m_volumeDialog->show();
    }
}

void MiniPlayerApplet::togglePlayListDialog()
{
    if (m_playListDialog->isVisible())
    {
        m_playListDialog->close();
    }
    else
    {
        m_playListDialog->move(popupPosition(m_playListDialog->size()));

        trackPressed(QModelIndex());

        m_playListDialog->show();
    }
}

void MiniPlayerApplet::toggleFullScreen()
{
    if (m_fullScreen->isFullScreen() || !m_mediaObject->hasVideo())
    {
        KConfigGroup configuration = config();

        m_fullScreenUi.videoLayout->removeWidget(m_videoWidget);
        m_fullScreen->showNormal();
        m_fullScreen->hide();

        m_hideFullScreenControls->stop();

        m_videoWidget->setCursor(QCursor(Qt::ArrowCursor));

        showVideoInDialog(configuration.readEntry("showVideoInDialog", true));

        m_fullScreenAction->setIcon(KIcon("view-fullscreen"));
        m_fullScreenAction->setText(i18n("Full Screen Mode"));
    }
    else
    {
        toggleProxyWidget(false);

        m_fullScreenUi.videoLayout->addWidget(m_videoWidget);

        m_fullScreen->showFullScreen();
        m_fullScreen->setWindowTitle(m_title);
        m_fullScreenUi.titleLabel->hide();
        m_fullScreenUi.controlsWidget->hide();

        m_fullScreenAction->setIcon(KIcon("view-restore"));
        m_fullScreenAction->setText(i18n("Exit Full Screen Mode"));

        m_hideFullScreenControls->start(2000);
    }
}

void MiniPlayerApplet::toggleMute()
{
    m_audioOutput->setMuted(!m_audioOutput->isMuted());
}

void MiniPlayerApplet::toggleProxyWidget(bool show)
{
    if (!m_proxyWidget->widget())
    {
        return;
    }

    m_proxyWidget->setVisible(show);

    m_mainLayout->removeItem(m_proxyWidget);

    if (show)
    {
        m_mainLayout->insertItem(0, m_proxyWidget);

        m_proxyWidget->show();
        m_proxyWidget->setPreferredSize(boundingRect().width(), (boundingRect().width() * 0.75));

        resize(boundingRect().width(), ((boundingRect().width() * 0.75) + m_controls->size().height()));
    }
}

void MiniPlayerApplet::toggleControls()
{
    if (!m_proxyWidget->isVisible())
    {
        return;
    }

    m_mainLayout->removeItem(m_controls);

    if (m_controls->isVisible())
    {
        m_controls->hide();

        m_controlsAction->setText(i18n("Show Controls"));
    }
    else
    {
        m_mainLayout->insertItem(1, m_controls);

        m_controls->show();

        m_controlsAction->setText(i18n("Hide Controls"));
    }
}

void MiniPlayerApplet::updateControls(Phonon::State state)
{
    m_playPauseAction->setIcon(KIcon((state == Phonon::PlayingState)?"media-playback-pause":"media-playback-start"));
    m_playPauseAction->setText((state == Phonon::PlayingState)?i18n("Pause"):i18n("Play"));
    m_playPauseAction->setEnabled(state == Phonon::PlayingState || state == Phonon::PausedState || m_playLists[m_visiblePlayList]->getTracks().count());

    m_stopAction->setEnabled(state == Phonon::PlayingState || state == Phonon::PausedState);

    m_navigationAction->setEnabled(state == Phonon::PlayingState || state == Phonon::PausedState);

    m_seekSlider->setEnabled(state == Phonon::PlayingState || state == Phonon::PausedState);
    m_seekSlider->setRange(0, m_mediaObject->totalTime());

    trackPressed(QModelIndex());
}

void MiniPlayerApplet::filterPlayList(QString text)
{
    KUrl::List urls = m_playLists[m_visiblePlayList]->getTracks();

    for (int i = 0; i < urls.count(); ++i)
    {
        m_playListUi.playListView->setRowHidden(i, (!text.isEmpty() && !m_playLists[m_visiblePlayList]->index(i, 0).data(Qt::DisplayRole).toString().contains(text, Qt::CaseInsensitive)));
    }
}

void MiniPlayerApplet::savePlayListNames()
{
    KConfigGroup configuration = config();
    QStringList names;
    QString name;

    for (int i = 0; i < m_playListTabBar->count(); ++i)
    {
        name = KGlobal::locale()->removeAcceleratorMarker(m_playListTabBar->tabText(i));

        if (!m_discPlayLists.contains(name))
        {
            names.append(name);
        }
    }

    configuration.writeEntry("playLists", names);

    emit configNeedsSaving();
}

void MiniPlayerApplet::savePlayList()
{
    KConfigGroup configuration = config();
    KUrl::List urls = m_playLists[m_visiblePlayList]->getTracks();
    QStringList titles;

    for (int i = 0; i < urls.count(); ++i)
    {
        titles.append(m_titles[urls.at(i)]);
    }

    configuration.writeEntry("playListUrls-" + m_visiblePlayList, urls.toStringList());
    configuration.writeEntry("playListTitles-" + m_visiblePlayList, titles);

    emit configNeedsSaving();
}

void MiniPlayerApplet::createPlayList(QString playList, KUrl::List tracks)
{
    if (m_playLists.contains(playList) || playList.isEmpty())
    {
        return;
    }

    m_playLists[playList] = new MiniPlayerPlayListModel(this);
    m_playLists[playList]->addTracks(tracks);

    m_playListTabBar->show();

    int position = m_playListTabBar->currentIndex();

    if (position == (m_playListTabBar->count() - 1))
    {
        m_playListTabBar->addTab(iconForPlayList(playList), playList);
    }
    else
    {
        m_playListTabBar->insertTab((position + 1), iconForPlayList(playList), playList);
    }

    m_playListTabBar->setCurrentIndex(position + 1);

    savePlayListNames();

    connect(m_playLists[playList], SIGNAL(needsSaving()), this, SLOT(savePlayList()));
    connect(m_playLists[playList], SIGNAL(itemChanged(QModelIndex)), this, SLOT(trackChanged(QModelIndex)));
}

void MiniPlayerApplet::addToPlayList()
{
    openFiles(false);
}

void MiniPlayerApplet::addToPlayList(KUrl::List items, bool play, int index)
{
    new MiniPlayerPlayListImporter(m_visiblePlayList, items, play, index, this);
}

void MiniPlayerApplet::exportPlayList()
{
    KConfigGroup configuration = config();

    KFileDialog *dialog = new KFileDialog(KUrl(configuration.readEntry("directory", "~")), "", NULL);
    dialog->setMimeFilter(QStringList() << "audio/x-scpls" << "audio/x-mpegurl");
    dialog->setWindowModality(Qt::NonModal);
    dialog->setMode(KFile::File);
    dialog->setOperationMode(KFileDialog::Saving);
    dialog->setConfirmOverwrite(true);
    dialog->setSelection(m_playListTabBar->tabText(m_playListTabBar->currentIndex()).remove('&') + ".pls");
    dialog->exec();

    if (dialog->selectedUrl().isEmpty())
    {
        return;
    }

    QFile data(dialog->selectedUrl().toLocalFile());

    if (data.open(QFile::WriteOnly))
    {
        QTextStream out(&data);
        int i = 0;

        PlayListType type = (dialog->selectedUrl().toLocalFile().endsWith(".pls")?PLS:M3U);

        if (type == PLS)
        {
            out << "[playlist]\n";
            out << "NumberOfEntries=" << m_playLists[m_visiblePlayList]->getTracks().count() << "\n\n";
        }
        else
        {
            out << "#EXTM3U\n\n";
        }

        foreach (const KUrl &url, m_playLists[m_visiblePlayList]->getTracks())
        {
            QString title = ((m_titles.contains(url) && !m_titles[url].isEmpty())?m_titles[url]:"");

            if (type == PLS)
            {
                out << "File" << QString::number(++i) << "=";
            }
            else
            {
                out << "#EXTINF:," << title << "\n";
            }

            out << url.pathOrUrl() << '\n';

            if (type == PLS)
            {
                out << "Title" << QString::number(i) << "=" << title << '\n';
                out << "Length" << QString::number(i) << "=\n\n";
            }
        }

        if (type == PLS)
        {
            out << "Version=2";
        }

        data.close();

        KMessageBox::information(NULL, i18n("File saved successfully."));
    }
    else
    {
        KMessageBox::error(NULL, i18n("Cannot open file for writing."));
    }

    delete dialog;
}

void MiniPlayerApplet::newPlayList()
{
    createPlayList(KInputDialog::getText(i18n("New play-list"), i18n("Enter name:")));
}

void MiniPlayerApplet::clearPlayList()
{
    m_playLists[m_visiblePlayList]->clearPlayList();

    updateControls(m_mediaObject->state());
}

void MiniPlayerApplet::setActivePlayList(QString playList)
{
    QString name;

    m_activePlayList = playList;

    for (int i = 0; i < m_playListTabBar->count(); ++i)
    {
        name = KGlobal::locale()->removeAcceleratorMarker(m_playListTabBar->tabText(i));

        m_playListTabBar->setTabIcon(i, ((playList == name)?KIcon("media-playback-start"):iconForPlayList(name)));
    }

    if (!m_discTitles.contains(playList))
    {
        KConfigGroup configuration = config();
        configuration.writeEntry("currentPlayList", playList);

        emit configNeedsSaving();
    }
}

void MiniPlayerApplet::showVideoInDialog(bool show)
{
    KConfigGroup configuration = config();

    configuration.writeEntry("showVideoInDialog", show);

    if (m_fullScreen->isFullScreen())
    {
        return;
    }

    if (show)
    {
        if (m_playListUi.videoWidget->layout()->indexOf(m_videoWidget) == -1)
        {
            toggleProxyWidget(false);

            m_proxyWidget->setWidget(NULL);

            m_playListUi.videoWidget->layout()->addWidget(m_videoWidget);

            m_videoWidget->show();
            m_playListUi.blankLabel->hide();
        }
    }
    else
    {
        if (m_playListUi.videoWidget->layout()->indexOf(m_videoWidget) > -1 && !m_fullScreen->isFullScreen())
        {
            m_playListUi.blankLabel->show();

            m_playListUi.videoWidget->layout()->removeWidget(m_videoWidget);

            m_videoWidget->setParent(NULL);

            m_proxyWidget->setWidget(m_videoWidget);

            toggleProxyWidget(true);
        }
    }

    hasVideoChanged(m_mediaObject->hasVideo());

    emit configNeedsSaving();
}

void MiniPlayerApplet::setPosition(int position)
{
    disconnect(m_seekSlider, SIGNAL(sliderMoved(int)), this, SLOT(setPosition(int)));

    m_mediaObject->seek(position);

    m_seekSlider->setValue(position);

    connect(m_seekSlider, SIGNAL(sliderMoved(int)), this, SLOT(setPosition(int)));
}

void MiniPlayerApplet::setBrightness(int value)
{
    m_videoWidget->setBrightness(value?((float) value / 100):0);
}

void MiniPlayerApplet::setContrast(int value)
{
    m_videoWidget->setContrast(value?((float) value / 100):0);
}

void MiniPlayerApplet::setHue(int value)
{
    m_videoWidget->setHue(value?((float) value / 100):0);
}

void MiniPlayerApplet::setSaturation(int value)
{
    m_videoWidget->setSaturation(value?((float) value / 100):0);
}

void MiniPlayerApplet::setRandomTrack(bool random)
{
    KConfigGroup configuration = config();

    configuration.writeEntry("randomTrack", random);

    m_playListUi.randomButton->setChecked(random);

    m_playListUi.repeatButton->setEnabled(!random);

    emit configNeedsSaving();
}

void MiniPlayerApplet::setAspectRatio(int ratio)
{
    KConfigGroup configuration = config();

    if (ratio == 1)
    {
        m_videoWidget->setAspectRatio(Phonon::VideoWidget::AspectRatio4_3);
    }
    else if (ratio == 2)
    {
        m_videoWidget->setAspectRatio(Phonon::VideoWidget::AspectRatio16_9);
    }
    else if (ratio == 3)
    {
        m_videoWidget->setAspectRatio(Phonon::VideoWidget::AspectRatioWidget);
    }
    else
    {
        m_videoWidget->setAspectRatio(Phonon::VideoWidget::AspectRatioAuto);

        ratio = 0;
    }

    configuration.writeEntry("ratio", ratio);

    m_aspectRatioMenu->actions().at(ratio)->setChecked(true);

    emit configNeedsSaving();
}

void MiniPlayerApplet::setRepeatMode(int repeat)
{
    KConfigGroup configuration = config();
    RepeatMode repeatMode = static_cast<RepeatMode>(repeat);

    m_playListUi.randomButton->setEnabled(repeatMode == NoRepeat);

    if (isTrackRandom())
    {
        setRandomTrack(false);
    }

    configuration.writeEntry("repeatMode", repeat);

    m_repeatModeMenu->actions().at(repeat)->setChecked(true);

    emit configNeedsSaving();
}

void MiniPlayerApplet::changeAspectRatio(QAction *action)
{
    setAspectRatio(action->data().toInt());
}

void MiniPlayerApplet::changeAudioChannel(QAction *action)
{
    m_mediaController->setCurrentAudioChannel(m_mediaController->availableAudioChannels().at(action->data().toInt()));
}

void MiniPlayerApplet::changeSubtitles(QAction *action)
{
    m_mediaController->setCurrentSubtitle(m_mediaController->availableSubtitles().at(action->data().toInt()));
}

void MiniPlayerApplet::changeChapter(QAction *action)
{
    m_mediaController->setCurrentChapter(action->data().toInt());
}

void MiniPlayerApplet::changeAngle(QAction *action)
{
    m_mediaController->setCurrentAngle(action->data().toInt());
}

void MiniPlayerApplet::changeRepeatMode(QAction *action)
{
    setRepeatMode(action->data().toInt());
}

void MiniPlayerApplet::hideFullScreenControls()
{
    m_videoWidget->setCursor(QCursor(Qt::BlankCursor));

    m_fullScreenUi.titleLabel->hide();
    m_fullScreenUi.controlsWidget->hide();
}

void MiniPlayerApplet::showToolTip()
{
    KConfigGroup configuration = config();

    if (configuration.readEntry("showToolTipOnTrackChange", 5) > 0)
    {
        m_hideToolTip->stop();

        Plasma::ToolTipManager::self()->show(this);

        m_hideToolTip->start(configuration.readEntry("showToolTipOnTrackChange", 5) * 1000);
    }
}

void MiniPlayerApplet::hideToolTip()
{
    Plasma::ToolTipManager::self()->hide(this);
}

void MiniPlayerApplet::updateToolTip()
{
    Plasma::ToolTipContent data;

    if (m_mediaObject->state() == Phonon::PlayingState || m_mediaObject->state() == Phonon::PausedState)
    {
        data.setMainText(m_title);
        data.setSubText((m_mediaObject->totalTime() > 0)?i18n("Position: %1 / %2", timeToString(m_mediaObject->currentTime()), timeToString(m_mediaObject->totalTime())):"");
        data.setImage(iconForUrl(currentUrl()).pixmap(IconSize(KIconLoader::Desktop)));
        data.setAutohide(false);
    }

    Plasma::ToolTipManager::self()->setContent(this, data);
}

void MiniPlayerApplet::toolTipAboutToShow()
{
    updateToolTip();
}

void MiniPlayerApplet::toolTipHidden()
{
    Plasma::ToolTipManager::self()->clearContent(this);
}

void MiniPlayerApplet::importPlayList(QString playList, KUrl::List tracks, QHash<KUrl, QString> titles, bool playFirst, int position)
{
    if (m_playLists.contains(playList))
    {
        m_playLists[playList]->addTracks(tracks, position);

        if (playFirst && tracks.count())
        {
            setActivePlayList(playList);

            play(tracks.at(0));
        }

        if (playList == m_visiblePlayList)
        {
            m_playListUi.playListView->setCurrentIndex(m_playLists[m_visiblePlayList]->index((position - 1), 0));

            trackPressed(QModelIndex());
        }
    }

    m_titles.unite(titles);
}

void MiniPlayerApplet::savePlayListSettings(int position, int index)
{
    Q_UNUSED(position)
    Q_UNUSED(index)

    KConfigGroup configuration = config();

    configuration.writeEntry("playListSize", m_playListDialog->size());
    configuration.writeEntry("playListSplitter", m_playListUi.splitter->saveState());

    emit configNeedsSaving();
}

void MiniPlayerApplet::addDevice(const QString &udi)
{
    Solid::Device device = Solid::Device(udi);

    if (device.isDeviceInterface(Solid::DeviceInterface::OpticalDisc))
    {
        Solid::OpticalDisc *opticalDisc = static_cast<Solid::OpticalDisc*>(device.asDeviceInterface(Solid::DeviceInterface::OpticalDisc));
        QString label = opticalDisc->label().replace('_', ' ');

        if (label.isEmpty())
        {
            label = i18n("Disc %1", (m_discTypes.count() + 1));
        }

        if (opticalDisc->availableContent() & Solid::OpticalDisc::VideoDvd)
        {
            m_discActions[udi] = m_openMenu->addAction(KIcon("media-optical"), i18n("Open DVD: %1", label));

            m_discTypes[udi] = Phonon::Dvd;
        }
        else if (opticalDisc->availableContent() & Solid::OpticalDisc::VideoCd || opticalDisc->availableContent() & Solid::OpticalDisc::SuperVideoCd)
        {
            m_discActions[udi] = m_openMenu->addAction(KIcon("media-optical"), i18n("Open VCD: %1", label));

            m_discTypes[udi] = Phonon::Vcd;
        }
        else if (opticalDisc->availableContent() & Solid::OpticalDisc::Audio)
        {
            m_discActions[udi] = m_openMenu->addAction(KIcon("media-optical-audio"), i18n("Open Audio CD: %1", label));

            m_discTypes[udi] = Phonon::Cd;
        }

        if (m_discActions.contains(udi))
        {
            m_discActions[udi]->setData(udi);

            m_discTitles[udi] = label;
        }
    }
}

void MiniPlayerApplet::removeDevice(const QString &udi)
{
    if (m_discActions.contains(udi))
    {
        if (m_discPlayLists.contains(m_discTitles[udi]))
        {
            QString name;

            for (int i = 0; i < m_playListTabBar->count(); ++i)
            {
                name = KGlobal::locale()->removeAcceleratorMarker(m_playListTabBar->tabText(i));

                if (name == m_discTitles[udi])
                {
                    if (m_activePlayList == m_discTitles[udi])
                    {
                        stop();
                    }

                    removePlayList(i);
                }
            }
        }

        m_discActions[udi]->deleteLater();

        m_discActions.remove(udi);

        m_discTitles.remove(udi);

        m_discTypes.remove(udi);
    }
}

void MiniPlayerApplet::updateTheme()
{
    QPalette palette = m_playListDialog->palette();
    palette.setColor(QPalette::WindowText, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
    palette.setColor(QPalette::ButtonText, Plasma::Theme::defaultTheme()->color(Plasma::Theme::ButtonTextColor));
    palette.setColor(QPalette::Background, Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));
    palette.setColor(QPalette::Button, palette.color(QPalette::Background).lighter(250));

    m_playListDialog->setPalette(palette);
}

void MiniPlayerApplet::removeFromPlayList(KUrl::List items)
{
    m_playLists[m_visiblePlayList]->removeTracks(items);
}

void MiniPlayerApplet::changeUrlTitle(KUrl url, QString title)
{
    m_titles[url] = title;
}

QVariantMap MiniPlayerApplet::metaData(Phonon::MediaObject *mediaObject)
{
    QVariantMap metaData;
    QMultiMap<QString, QString> stringMap = mediaObject->metaData();
    QMultiMap<QString, QString>::const_iterator i = stringMap.constBegin();

    while (i != stringMap.constEnd())
    {
        bool number = false;
        int value = i.value().toInt(&number);

        if (number && (i.key().toLower() != "tracknumber"))
        {
            metaData[i.key().toLower()] = value;
        }
        else
        {
            metaData[i.key().toLower()] = QVariant(i.value());
        }

        ++i;
    }

    metaData["time"] = (mediaObject->totalTime() / 1000);
    metaData["location"] = mediaObject->currentSource().url().toString();

    return metaData;
}

QString MiniPlayerApplet::timeToString(qint64 time)
{
    QString string;
    int seconds = (time / 1000);
    int minutes = (seconds / 60);
    int hours = (minutes / 60);

    string.append(QString::number(hours));
    string.append(':');

    minutes = (minutes - (hours * 60));

    if (minutes < 10)
    {
        string.append('0');
    }

    string.append(QString::number(minutes));
    string.append(':');

    seconds = (seconds - (hours * 3600) - (minutes * 60));

    if (seconds < 10)
    {
        string.append('0');
    }

    string.append(QString::number(seconds));

    return string;
}

QList<QAction*> MiniPlayerApplet::contextualActions()
{
    return m_actions;
}

KUrl::List MiniPlayerApplet::currentPlayList()
{
    return m_playLists[m_activePlayList]->getTracks();
}

Phonon::MediaObject* MiniPlayerApplet::mediaObject()
{
    return m_mediaObject;
}

Phonon::AudioOutput* MiniPlayerApplet::audioOutput()
{
    return m_audioOutput;
}

KUrl MiniPlayerApplet::currentUrl()
{
    switch (m_mediaObject->currentSource().type())
    {
        case Phonon::MediaSource::Invalid:
            return KUrl();
        case Phonon::MediaSource::Disc:
            return KUrl(QString("disc:/%1/%2").arg(m_mediaObject->currentSource().deviceName()).arg(m_mediaController->currentTitle()));
        default:
            return KUrl(m_mediaObject->currentSource().url());
    }
}

QString MiniPlayerApplet::titleForUrl(KUrl url)
{
    QString title;

    if (!m_titles[url].isEmpty())
    {
        title = m_titles[url];
    }
    else
    {
        QFileInfo *location = new QFileInfo(url.pathOrUrl());

        title = location->completeBaseName().replace("%20", " ");

        delete location;
    }

    return title;
}

QString MiniPlayerApplet::deviceForUrl(KUrl url)
{
    if (url.protocol() == "disc")
    {
        QString device = url.url().mid(6, (url.url().lastIndexOf("/") - 6));

        if (m_discTypes.contains(device))
        {
            return device;
        }
        else
        {
            return QString();
        }
    }
    else
    {
        return QString();
    }
}

KIcon MiniPlayerApplet::iconForUrl(KUrl url)
{
    if (url.protocol() == "disc")
    {
        QString device = deviceForUrl(url);

        if (device.isEmpty())
        {
            return KIcon("application-x-zerosize");
        }
        else
        {
            return KIcon((m_discTypes[device] == Phonon::Cd)?"application-x-cda":"video-x-generic");
        }
    }
    else if (!url.isValid())
    {
        return KIcon("application-x-zerosize");
    }
    else
    {
        return KIcon(KMimeType::iconNameForUrl(url));
    }
}

KIcon MiniPlayerApplet::iconForPlayList(QString playList)
{
    KIcon icon;

    if (m_discPlayLists.contains(playList))
    {
        switch (m_discTypes[m_discPlayLists[playList]])
        {
            case Phonon::Cd:
                icon = KIcon("media-optical-audio");
            break;
            case Phonon::Dvd:
            case Phonon::Vcd:
                icon = KIcon("media-optical-video");
            break;
            default:
                icon = KIcon("view-media-playlist");
            break;
        }
    }
    else
    {
        icon = KIcon("view-media-playlist");
    }

    return icon;
}

bool MiniPlayerApplet::isPlayListRepeated()
{
    KConfigGroup configuration = config();
    RepeatMode repeatMode = static_cast<RepeatMode>(configuration.readEntry("repeatMode", 2));

    return (repeatMode == RepeatPlayList);
}

bool MiniPlayerApplet::isTrackRepeated()
{
    KConfigGroup configuration = config();
    RepeatMode repeatMode = static_cast<RepeatMode>(configuration.readEntry("repeatMode", 2));

    return (repeatMode == RepeatTrack);
}

bool MiniPlayerApplet::isTrackRandom()
{
    KConfigGroup configuration = config();

    return configuration.readEntry("randomTrack", false);
}

bool MiniPlayerApplet::eventFilter(QObject *object, QEvent *event)
{
    if (object == m_videoWidget || object == m_fullScreen || object == m_playListUi.blankLabel)
    {
        if (event->type() == QEvent::ContextMenu)
        {
            QList<QAction*> actions;
            KMenu *menu = new KMenu;
            KAction *settingsAction = new KAction(KIcon("configure"), i18n("Settings"), menu);

            connect(settingsAction, SIGNAL(triggered()), this, SLOT(showConfigurationInterface()));

            actions = m_actions;
            actions.append(settingsAction);

            menu->addActions(actions);
            menu->exec(QCursor::pos());

            delete menu;

            return true;
        }
        else if (event->type() == QEvent::MouseButtonPress && static_cast<QMouseEvent*>(event)->button() == Qt::LeftButton)
        {
            if (m_lastClick.secsTo(QDateTime::currentDateTime()) < 2)
            {
                toggleFullScreen();

                m_lastClick= QDateTime();
            }
            else
            {
                m_lastClick = QDateTime::currentDateTime();
            }

            return true;
        }
        else if (event->type() == QEvent::MouseButtonDblClick && static_cast<QMouseEvent*>(event)->button() == Qt::LeftButton)
        {
            toggleFullScreen();

            return true;
        }
        else if (event->type() == QEvent::MouseMove && m_fullScreen->isFullScreen())
        {
            m_hideFullScreenControls->stop();
            m_hideFullScreenControls->start(2000);

            m_videoWidget->setCursor(QCursor(Qt::ArrowCursor));

            m_fullScreenUi.titleLabel->show();
            m_fullScreenUi.controlsWidget->show();

            return true;
        }
        else if (event->type() == QEvent::KeyPress)
        {
            keyPressEvent(static_cast<QKeyEvent*>(event));

            return true;
        }
        else if (event->type() == QEvent::DragEnter)
        {
            static_cast<QDragEnterEvent*>(event)->acceptProposedAction();

            return true;
        }
        else if (event->type() == QEvent::Drop)
        {
            addToPlayList(KUrl::List::fromMimeData(static_cast<QDropEvent*>(event)->mimeData()), true);

            return true;
        }
    }
    else if (object == m_playListUi.playListView)
    {
        if (event->type() == QEvent::KeyPress && !m_editorActive)
        {
            QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);

            if (keyEvent->key() == Qt::Key_Return && m_playListUi.playListView->selectionModel()->selectedIndexes().count())
            {
                playTrack(m_playListUi.playListView->selectionModel()->selectedIndexes().first());
            }
            else
            {
                keyPressEvent(keyEvent);
            }

            return true;
        }
        else if (event->type() == QEvent::ContextMenu)
        {
            QPoint point = static_cast<QContextMenuEvent*>(event)->pos();
            point.setY(point.y() - m_playListUi.playListView->horizontalHeader()->height());

            QModelIndex index = m_playListUi.playListView->indexAt(point);

            if (index.data(Qt::DisplayRole).toString().isEmpty())
            {
                return false;
            }

            KMenu *menu = new KMenu;
            KUrl url = KUrl(index.data(Qt::ToolTipRole).toString());
            QString device = deviceForUrl(url);
            QAction *editAction;
            QAction *copyAction;

            if (device.isEmpty())
            {
                editAction = menu->addAction(KIcon("document-edit"), i18n("Edit title"));
                copyAction = menu->addAction(KIcon("edit-copy"), i18n("Copy URL"));

                menu->addSeparator();

                connect(editAction, SIGNAL(triggered()), this, SLOT(editTrackTitle()));
                connect(copyAction, SIGNAL(triggered()), this, SLOT(copyTrackUrl()));
            }

            if (url == currentUrl())
            {
                menu->addAction(m_playPauseAction);
                menu->addAction(m_stopAction);
            }
            else
            {
                QAction *playAction = menu->addAction(KIcon("media-playback-start"), i18n("Play"));

                connect(playAction, SIGNAL(triggered()), this, SLOT(playTrack()));
            }

            menu->addSeparator();

            QAction *removeAction = menu->addAction(KIcon("list-remove"), i18n("Remove"));

            connect(removeAction, SIGNAL(triggered()), this, SLOT(removeTrack()));

            menu->exec(QCursor::pos());

            delete menu;

            return true;
        }
    }

    return QObject::eventFilter(object, event);
}

#include "MiniPlayerApplet.moc"
