/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "gamewidget.h"
#include "game.h"
#include "buttoncontainer.h"
#ifdef NEPOMUK
#include "ratingwidget.h"
#endif
#include "gamelabel.h"

// Qt
#include <QDrag>
#include <QGraphicsGridLayout>
#include <QApplication>
#include <QGraphicsSceneWheelEvent>

// KDE
#include <KIcon>
#include <KDebug>
#include <Plasma/IconWidget>




GameWidget::GameWidget(QGraphicsWidget *parent, Game *game)
        : Plasma::Frame(parent),
        m_game(game)
{
    
    setFrameShadow(Plasma::Frame::Plain);
    m_game->setParent(this);  
    setAcceptHoverEvents(true);
    
    
    QGraphicsGridLayout *layout = new QGraphicsGridLayout;
    layout->setContentsMargins(0, 0, 0, 0);
    
    m_icon = new Plasma::IconWidget(this);
    m_icon->setMaximumWidth(KIconLoader::SizeMedium);
    m_icon->setMinimumWidth(KIconLoader::SizeMedium);
    connect(m_icon, SIGNAL(clicked()), m_game, SLOT(start()));
    
    layout->addItem(m_icon, 0, 0, 2, 1);
    
    GameLabel *title = new GameLabel(this, m_game, GameLabel::Title);
    GameLabel *description = new GameLabel(this, m_game, GameLabel::Description);
    layout->addItem(title, 0, 1, 1, 1);
    layout->addItem(description, 1, 1, 1, 2);
                
    m_container = new ButtonContainer(this);
    
    connect(m_container, SIGNAL(up()), this, SIGNAL(up()));
    connect(m_container, SIGNAL(down()), this, SIGNAL(down()));
    connect(m_container, SIGNAL(edit()), this, SIGNAL(edit()));
    connect(m_container, SIGNAL(remove()), this, SIGNAL(remove()));
    
    layout->addItem(m_container, 0, 4, 2, 1);
    layout->setAlignment(m_container, Qt::AlignCenter);
    m_container->setVisible(false);
    
#ifdef NEPOMUK
    RatingWidget *rating = new RatingWidget(this, m_game);
    layout->addItem(rating, 0, 2, 1, 1);
#endif

    setLayout(layout);
    
    
    connect(m_game, SIGNAL(changed()), this, SLOT(gameChanged()));
    gameChanged();
    
}



GameWidget::~GameWidget()
{

    delete m_game;
    
}


Game *GameWidget::game() const
{

    return m_game;

}


void GameWidget::hideButtons()
{

    m_container->hide();

}


void GameWidget::showButtons()
{

    m_container->show();

}


void GameWidget::setSortingEnabled(const bool &enabled)
{

    m_container->setSortingEnabled(enabled);

}


void GameWidget::gameChanged()
{

    m_icon->setIcon(KIcon(m_game->icon()));
    setMaximumHeight(layout()->effectiveSizeHint(Qt::MinimumSize).height());
    
}


void GameWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
{

    event->accept();
    
}


void GameWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{

    event->accept();
    m_game->start();

}


void GameWidget::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{

    if (QLineF(event->screenPos(), event->buttonDownScreenPos(Qt::LeftButton))
        .length() < QApplication::startDragDistance()) {
        return;
    }
    QMimeData *data = new QMimeData;
    
    QString posString = QString::number(pos().x());
    posString += ' ';
    posString += QString::number(pos().y());
    data->setData("Pos", posString.toLatin1());

    QDrag *drag = new QDrag(event->widget());
    drag->setMimeData(data);            
    drag->setPixmap(m_icon->icon().pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium));
    data->setUrls(QList<QUrl>() << QUrl(m_game->desktopFile()));
    data->setText(m_game->title());
    
    drag->exec();
    
}


void GameWidget::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
{

    Q_UNUSED(event);

}


void GameWidget::keyPressEvent(QKeyEvent *event)
{

    Q_UNUSED(event)

}


void GameWidget::wheelEvent(QGraphicsSceneWheelEvent *event)
{

    event->ignore();
    
}


void GameWidget::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{


    m_container->show();
    emit hover(event->pos());

}


void GameWidget::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{

    Q_UNUSED(event);
    m_container->show();
    
}


void GameWidget::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
    
    Q_UNUSED(event);
    m_container->hide();
    emit hover(QPointF(-1, -1));
  
}


#include "gamewidget.moc"

