/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "pgame.h"
#include "gamelist.h"
#include "game.h"

// Qt
#include <QGraphicsLinearLayout>
#include <QDir>
#include <QtAlgorithms>

// KDE
#include <KIcon>
#include <KConfigGroup>
#include <KDesktopFile>
#include <Plasma/ScrollWidget>
#include <KConfigDialog>



PGame::PGame(QObject *parent, QVariantList args)
    : Plasma::PopupApplet(parent, args)
{

    setHasConfigurationInterface(true);
    resize(400, 400);
    setAcceptDrops(true);
    setPopupIcon(KIcon("applications-games"));
    setAspectRatioMode(Plasma::IgnoreAspectRatio);

}


PGame::~PGame()
{

    delete m_widget;
    
}


void PGame::destroy()
{

    foreach (Game *game, m_gameList->games()) {
        game->remove();
    }
    Plasma::PopupApplet::destroy();
    
}


void PGame::init()
{
        
    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical);

    m_widget = new QGraphicsWidget(this);
    m_widget->setLayout(layout);
 
    m_gameList = new GameList(m_widget);
    connect(m_gameList, SIGNAL(gameDrop(QPointF,KUrl)), this, SLOT(gameDrop(QPointF,KUrl)));
    
    m_list = new Plasma::ScrollWidget(m_widget);
    m_list->setWidget(m_gameList);
    layout->addItem(m_list);
    
    loadGames();
    
    if (m_gameList->isEmpty()) {
        QStringList defaultGames;
        defaultGames << "KBreakout" << "KBounce" << "KBlackbox" << "Kapman" << "KSirk"
        << "KSudoku" << "LSkat" << "Kolf" << "Bomber" << "KGoldrunner" << "KSpaceDuel";           
        
        foreach (const QString &defaultGame, defaultGames) {                                                       
            KService::Ptr service = KService::serviceByStorageId(defaultGame);
            if (service && service->isValid()) {                                                                   
                QString path = service->entryPath();                        
                if (!path.isEmpty() && QDir::isAbsolutePath(path)) {
                    QVariantMap data;
                    data["DesktopFile"] = path;
                    data["Copy"] = true;
                    data["Id"] = id();
                    
                    Game *game = new Game(this, data, m_gameList);
                    connect(game, SIGNAL(changed()), this, SLOT(saveGames()));

                    m_gameList->appendGame(game);
                }                                                                       
            }                                                                               
        }
        saveGames();
    }
    
}


QList<QAction*> PGame::contextualActions()
{

    QList<QAction*> actions = Plasma::PopupApplet::contextualActions();

    QAction *newGameAction = new QAction(this);
    newGameAction->setText(i18n("New Game"));
    newGameAction->setIcon(KIcon("list-add"));
    
    connect(newGameAction, SIGNAL(triggered(bool)), this, SLOT(newGame()));
    
    
    actions.append(newGameAction);
    
    
    return actions;
    
}


void PGame::newGame()
{

    QVariantMap data;
    data["NewGame"] = true;
    data["Id"] = id();
    data["Title"] = "New Game";
    data["Icon"] = "applications-games";
    
    Game *newGame = new Game(this, data, m_gameList);
    connect(newGame, SIGNAL(changed()), this, SLOT(saveGames()));
    
    newGame->edit();
    m_gameList->appendGame(newGame);
    
}


void PGame::gameDrop(const QPointF &pos, const KUrl &file)
{
    
    KDesktopFile desktop(file.path());
    KConfig cfg(file.path());
    KConfigGroup desktopCfg(&cfg, "Desktop Entry");
    
    const bool isGame = desktopCfg.readEntry("Categories").split(';').contains("Game");
    
    if (!isGame) {
        return;
    }

    QVariantMap data;
    data["DesktopFile"] = file.path();
    data["Copy"] = true;
    data["Id"] = QString::number(id());
    
    Game *game = new Game(this, data, m_gameList);
    connect(game, SIGNAL(changed()), this, SLOT(saveGames()));

    QGraphicsWidget *destinationWidget = m_gameList->widgetAt(pos);
    
    int destination = 0;
    if (destinationWidget) {
        destination = m_gameList->indexOf(destinationWidget);
    }
    
    m_gameList->insertGame(destination, game);
    
}


void PGame::saveGames()
{

    KConfigGroup cfg = config();
    KConfigGroup gameCfg(&cfg, "Games");
    
    int count = 0;
    foreach (Game *game, m_gameList->games()) {
        gameCfg.writeEntry(QString("Game %1").arg(count), game->desktopFile());
        count++;
    }
    gameCfg.writeEntry("GameCount", count);

}


void PGame::loadGames()
{

    KConfigGroup cfg = config();
    KConfigGroup gameCfg(&cfg, "Games");
    
    m_gameList->setSortOrder(GameList::SortOrder(cfg.readEntry("Sort", int(GameList::NoSort))));
    
    const int count = gameCfg.readEntry("GameCount", 0);
    for (int i = 0; i < count; i++) {
        const QString file = gameCfg.readEntry(QString("Game %1").arg(i), QString());

        QVariantMap data;
        data["DesktopFile"] = file;
        data["Id"] = QString::number(id());
        Game *game = new Game(this, data, m_gameList);
        connect(game, SIGNAL(changed()), this, SLOT(saveGames()));
        m_gameList->appendGame(game);
    }

}

    
void PGame::createConfigurationInterface(KConfigDialog *parent)
{

    QWidget *settings = new QWidget;
    setupUi(settings);
#ifndef NEPOMUK
    sortBox->removeItem(3);
#endif
    sortBox->setCurrentIndex(int(m_gameList->sort()));
    
    parent->addPage(settings, i18n("Settings"), "configure");
    connect(parent, SIGNAL(accepted()), this, SLOT(configAccepted()));
    
    parent->resize(300, 300);
    parent->show();
    
}


void PGame::configAccepted()
{

    m_gameList->setSortOrder(GameList::SortOrder(sortBox->currentIndex()));
    KConfigGroup cfg = config();
    cfg.writeEntry("Sort", sortBox->currentIndex());
    
}

    
QGraphicsWidget* PGame::graphicsWidget() 
{

    return m_widget;

}



#include "pgame.moc"


K_EXPORT_PLASMA_APPLET(pgame, PGame)




