/*
    This file is part of the KDE project.

    Copyright 2005,2009 Friedrich W. H. Kossebau <kossebau@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301  USA
*/

#ifndef KLOCALSESSIONMANAGER_H
#define KLOCALSESSIONMANAGER_H

// KDE
#include <KConfig>
#include <KApplication>
// Qt
#include <QtCore/QObject>

class QWidget;


struct KSession
{
  /** name of the display */
  QString DisplayName;
  QString Origin;
  /** login name of the user */
  QString LoginName;
  /** Name of the session */
  QString Session;
  /** id of the vt */
  int VTId;
  /** true if this is the current session */
  bool Current: 1;
  /** true if a plain terminal */
  bool IsTty: 1;
};

typedef QList<KSession> KSessionList;


class KLocalSessionManager : public QObject
{
  Q_OBJECT

  /**
   * The possible values for the @p sdtype parameter of requestShutDown().
   */
  enum ShutdownType {
    /**
     * Select previous action or the default if it's the first time.
     */
    ShutdownTypeDefault = -1,
    /**
     * Only log out
     */
    ShutdownTypeNone = 0,
    /**
     * Log out and reboot the machine.
     */
    ShutdownTypeReboot = 1,
    /**
     * Log out and halt the machine.
     */
    ShutdownTypeHalt = 2,
    /**
     * Temporary brain damage. Don't use. Same as ShutdownTypeNone
     */
    // KDE5: kill this
    ShutdownTypeLogout = 3
  };

  /**
   * The possible values for the @p sdmode parameter of requestShutDown().
   */
  // KDE5: this seems fairly useless
  enum ShutdownMode {
    /**
     * Select previous mode or the default if it's the first time.
     */
    ShutdownModeDefault = -1,
    /**
     * Schedule a shutdown (halt or reboot) for the time all active sessions
     * have exited.
     */
    ShutdownModeSchedule = 0,
    /**
     * Shut down, if no sessions are active. Otherwise do nothing.
     */
    ShutdownModeTryNow = 1,
    /**
     * Force shutdown. Kill any possibly active sessions.
     */
    ShutdownModeForceNow = 2,
    /**
     * Pop up a dialog asking the user what to do if sessions are still active.
     */
    ShutdownModeInteractive = 3
  };

  public:
    /**
     * @param S screen widget to place the dialogs on
     * @param SN screen number
     */
    KLocalSessionManager( QWidget* screen, int screeNumber );

  public:
    bool canCreateSession() const;
    bool canSwitchSession() const;
    bool canLogoutSession() const;
    bool canLockSession() const;
    bool canSaveSession() const;
    bool canShutdown() const;

    int numberOfReservedSessions() const;
    bool getBootOptions( QStringList& options, int& defaultOption, int& currentOption ) const;
    bool doesLockOnSwitch() const;
    bool getLocalSessions( KSessionList& list );
    KSessionList localSessions();

  public slots:
    void createSession() const;
    void switchToTerminal( int terminalId ) const;
    void saveSession() const;
    void lockSession() const;
    void logoutSession() const;
    void shutdown( ShutdownType shutdownType,
                   ShutdownMode shutdownMode,
                   const QString& bootOption = QString() );

    void setLockOnSwitch( bool lock );

  protected:
    /** pointer to the screen widget to place the dialogs on */
    QWidget* mScreen;
    int mScreenNumber;
    bool mLockOnSwitch;
};

#endif
