/*
 *   Copyright (C) 2009 Aaron Seigo <aseigo@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the
 *      Free Software Foundation, Inc.
 *      51 Franklin Street, Fifth Floor
 *      Boston, MA  02110-1301  USA.
 *
 */

#include "timeline.h"

#include <QPainter>
#include <QLinearGradient>

#include <KDebug>

static const int ANIM_FRAME_INTERVAL = 200;

TimeLineApplet::TimeLineApplet(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
      m_zoomLevel(HourZoom),
      m_stepSize(10),
      m_moveOffset(0)
{
    //setHasConfigurationInterface(false);
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    m_moveTimer.setSingleShot(true);
    m_moveTimer.setInterval(ANIM_FRAME_INTERVAL);
    connect(&m_moveTimer, SIGNAL(timeout()), this, SLOT(moveTimeLine()));
}

void TimeLineApplet::init()
{
    dataEngine("time")->connectSource("Local", this, 60 * 1000, Plasma::AlignToMinute);
}

void TimeLineApplet::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    Q_UNUSED(option)
    // leave some space at the bottom for the labels (TODO)
    const QRect linesArea = contentsRect.adjusted(0, 0, 0, -(contentsRect.height() / 5));
    const int step = m_stepSize; // TODO: make adjustable
    const int minsPerStep = m_zoomLevel;
    const int longStepMultiple = 3; // TODO: needs to shift based on the zoom level


    // keep it most of the way to the left, so we see a small bit of the past but mostly the future
    const int momentStep = (linesArea.width() / step) / 4.0;
    const uint timeRounded = m_now.toTime_t() - (m_now.toTime_t() % (minsPerStep * 60));
    const int whichStep = (timeRounded / (minsPerStep * 60)) % longStepMultiple;
    //kDebug() << timeRounded << whichStep << m_moveOffset;

    // draw our current position in time
    QLinearGradient momentBrush(linesArea.topLeft(), QPointF(linesArea.left(), linesArea.bottom()));
    momentBrush.setColorAt(0.3, QColor(Qt::transparent));
    momentBrush.setColorAt(1.0, QColor(255, 0, 0, 120));
    const int moment = linesArea.left() + (momentStep * step) + 1;
    painter->fillRect(moment, linesArea.top(), step - 1, linesArea.height(), momentBrush);

    //TODO: Plasma::Theme colors
    QLinearGradient shortLineBrush(linesArea.topLeft(), QPointF(linesArea.left(), linesArea.bottom()));
    shortLineBrush.setColorAt(0.7, QColor(Qt::transparent));
    shortLineBrush.setColorAt(1.0, Qt::black);

    QLinearGradient longLineBrush(linesArea.topLeft(), QPointF(linesArea.left(), linesArea.bottom()));
    longLineBrush.setColorAt(0.4, QColor(Qt::transparent));
    longLineBrush.setColorAt(1.0, Qt::green);

    int count = whichStep - 1;
    for (int i = linesArea.left(); i < linesArea.right(); i += step) {
        if (i < linesArea.left()) {
            continue;
        }

        if (count % longStepMultiple == 0) {
            painter->fillRect(i, linesArea.top(), 1, linesArea.height(), longLineBrush);
        } else {
            painter->fillRect(i, linesArea.top(), 1, linesArea.height(), shortLineBrush);
        }
        ++count;
    }

    //painter->drawText(linesArea, KGlobal::locale()->formatDateTime(m_now, KLocale::ShortDate, true));
    painter->drawText(linesArea.adjusted(moment - linesArea.left(), 0, 0, 0), KGlobal::locale()->formatDateTime(m_now));
}

void TimeLineApplet::createConfigurationInterface(KConfigDialog *parent)
{
    //QWidget *widget = new QWidget();
    //parent->addPage(widget, parent->windowTitle(), icon());
    //connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    //connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
}

void TimeLineApplet::configAccepted()
{
}

void TimeLineApplet::dataUpdated(const QString& source, const Plasma::DataEngine::Data &data)
{
    Q_UNUSED(source)
    m_now.setTime(data.value("Time").toTime());
    m_now.setDate(data.value("Date").toDate());

    if (m_now.time().minute() % m_zoomLevel == 0) {
        m_moveOffset = m_stepSize;
        moveTimeLine();
    } else {
        update();
    }
}

void TimeLineApplet::moveTimeLine()
{
    if (m_moveOffset) {
        --m_moveOffset;
        m_moveTimer.start();
        update();
    }
}

TimeLineApplet::~TimeLineApplet()
{
}

#include "timeline.moc"

K_EXPORT_PLASMA_APPLET(timelineapplet, TimeLineApplet)
