/*
    Copyright (C) 2010 Kevin Krammer <kevin.krammer@gmx.at>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "akonadiagentsengine.h"

#include "akonadiagentsservice.h"

#include <Akonadi/AgentInstance>
#include <Akonadi/AgentManager>

#include <QIcon>

using namespace Akonadi;

AkonadiAgentsEngine::AkonadiAgentsEngine( QObject* parent, const QVariantList& args )
    : DataEngine( parent, args )
{
  setMinimumPollingInterval( -1 );
}

void AkonadiAgentsEngine::init()
{
  AgentManager *manager = AgentManager::self();

  connect( manager, SIGNAL( instanceAdded( Akonadi::AgentInstance ) ), SLOT( instanceAdded( Akonadi::AgentInstance ) ) );
  connect( manager, SIGNAL( instanceRemoved( Akonadi::AgentInstance ) ), SLOT( instanceRemoved( Akonadi::AgentInstance ) ) );

  connect( manager, SIGNAL( instanceStatusChanged( Akonadi::AgentInstance ) ), SLOT( instanceStatusChanged( Akonadi::AgentInstance ) ) );
  connect( manager, SIGNAL( instanceProgressChanged( Akonadi::AgentInstance ) ), SLOT( instanceProgressChanged( Akonadi::AgentInstance ) ) );
  connect( manager, SIGNAL( instanceNameChanged( Akonadi::AgentInstance ) ), SLOT( instanceNameChanged( Akonadi::AgentInstance ) ) );
  connect( manager, SIGNAL( instanceOnline( Akonadi::AgentInstance, bool ) ), SLOT( instanceOnlineChanged( Akonadi::AgentInstance ) ) );

  const QStringList agents = sources();
  Q_FOREACH( const QString &agent, agents ) {
    updateSourceEvent( agent );
  }
}

QStringList AkonadiAgentsEngine::sources() const
{
  QStringList result;

  const AgentInstance::List agents = AgentManager::self()->instances();
  Q_FOREACH( const AgentInstance &agent, agents ) {
    result << agent.identifier();
  }

  return result;
}

Plasma::Service *AkonadiAgentsEngine::serviceForSource( const QString &source )
{
  const AgentInstance agent = AgentManager::self()->instance( source );
  if ( !agent.isValid() ) {
    kError() << "Requesting Service for invalid agent" << source;
    return 0;
  }

  AkonadiAgentsService *service = new AkonadiAgentsService( this );
  service->setDestination( source );

  return service;
}

bool AkonadiAgentsEngine::sourceRequestEvent( const QString &name )
{
  return updateSourceEvent( name );
}

bool AkonadiAgentsEngine::updateSourceEvent( const QString &name )
{
  const AgentInstance agent = AgentManager::self()->instance( name );
  setData( name, QLatin1String( "name" ), agent.name() );
  setData( name, QLatin1String( "status" ), agent.status() );
  setData( name, QLatin1String( "statusMessage" ), agent.statusMessage() );
  setData( name, QLatin1String( "online" ), agent.isOnline() );
  setData( name, QLatin1String( "progress" ), agent.progress() );
  setData( name, QLatin1String( "typeName" ), agent.type().name() );
  setData( name, QLatin1String( "typeCapabilities" ), agent.type().capabilities() );
  setData( name, QLatin1String( "typeMimeTypes" ), agent.type().mimeTypes() );
  setData( name, QLatin1String( "typeIcon" ), agent.type().icon() );

  return true;
}

void AkonadiAgentsEngine::instanceAdded( const AgentInstance &agent )
{
  Q_UNUSED( agent );
/*  kDebug() << "agent=" << agent.identifier();*/

  // seems this is not needed
  // actually engineexplorer will add it twice if we emit this
  //emit sourceAdded( agent.identifier() );
}

void AkonadiAgentsEngine::instanceRemoved( const AgentInstance &agent )
{
/*  kDebug() << "agent=" << agent.identifier();*/
  emit sourceRemoved( agent.identifier() );
}

void AkonadiAgentsEngine::instanceStatusChanged( const AgentInstance &agent )
{
/*  kDebug() << "agent=" << agent.identifier() << "status=" << agent.statusMessage();*/
  setData( agent.identifier(), QLatin1String( "status" ), agent.status() );
  setData( agent.identifier(), QLatin1String( "statusMessage" ), agent.statusMessage() );
}

void AkonadiAgentsEngine::instanceProgressChanged( const AgentInstance &agent )
{
/*  kDebug() << "agent=" << agent.identifier() << "progress=" << agent.progress();*/
  setData( agent.identifier(), QLatin1String( "progress" ), agent.progress() );
}

void AkonadiAgentsEngine::instanceNameChanged( const AgentInstance &agent )
{
/*  kDebug() << "agent=" << agent.identifier() << "name=" << agent.name();*/
  setData( agent.identifier(), QLatin1String( "name" ), agent.name() );
}

void AkonadiAgentsEngine::instanceOnlineChanged( const AgentInstance &agent )
{
/*  kDebug() << "agent=" << agent.identifier() << "online=" << agent.isOnline();*/
  setData( agent.identifier(), QLatin1String( "online" ), agent.isOnline() );
}

#include "akonadiagentsengine.moc"

K_EXPORT_PLASMA_DATAENGINE( akonadiagents, AkonadiAgentsEngine )

// kate: space-indent on; indent-width 2; replace-tabs on;
