/*
 *   Copyright (C) 2009 Andrew Coles  <andrew.coles@yahoo.co.uk>
 *   Based on code (C) 2009 Petri Damstén <damu@iki.fi>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "location_wifi.h"
#include "coordinate_backend.h"

#include <QtXml/QXmlStreamReader>

#include <solid/control/networkmanager.h>
#include <solid/control/wirelessaccesspoint.h>
#include <solid/control/wirelessnetworkinterface.h>

#include <kstandarddirs.h>
#include <KUrl>

class WiFi::Private : public QObject {
     
public:
  
    Private()
    {};
    
    ~Private()
    {
        foreach (CoordinateBackend* src, m_backends) {
	    delete src; 
	}
    };

    QString m_data;
    QXmlStreamReader m_xmlReader;
    QList<CoordinateBackend*> m_backends;
    
    QList<Solid::Control::WirelessNetworkInterface *> m_interfaces;
    WiFiScan * m_wifiScan;
    
    void addInterfaceInternal(Solid::Control::NetworkInterface* iface)
    {
	Q_ASSERT(iface);
	m_interfaces.append(static_cast<Solid::Control::WirelessNetworkInterface *>(iface));
    }
    
    Coordinate locateMAC(const QString & s) {
        Coordinate toReturn;
	foreach (CoordinateBackend* src, m_backends) {
	    Coordinate newCoord = src->resolveMAC(s);
	    if (!newCoord.status == COORD_UNDEFINED) {
	        if (   toReturn.status == COORD_UNDEFINED
		    || toReturn.error < toReturn.error    ) {
		    toReturn = newCoord; 
		}
            }
	}
	return toReturn;
    };

};


WiFiScan::WiFiScan(WiFi * p) : parent(p),m_abort(false) {}
 
WiFiScan::~WiFiScan()
{
    m_abort = true;
    m_condition.wakeOne();
    wait();
}

void WiFiScan::update()
{
    if (!isRunning()) {
        start();
    } else {
        m_condition.wakeOne();
    }
}

void WiFiScan::run()
{
    while (!m_abort) {	
        Plasma::DataEngine::Data d;
	double lati = 0.0;
	double longi = 0.0;
	double alti = 0.0;
	double error = 0.0;
	int divisor = 0;

        Coordinate minError;
	
	foreach (Solid::Control::WirelessNetworkInterface * iface, parent->d->m_interfaces) {
	    AccessPointList accessPoints = iface->accessPoints();
	    foreach (const QString& uni, accessPoints) {
	        Solid::Control::AccessPoint * const realAP = iface->findAccessPoint(uni);
		if (realAP) {
		     const QString MAC(realAP->hardwareAddress());
		     Coordinate coord = parent->d->locateMAC(MAC);
		     if (coord.status != COORD_UNDEFINED) {
		         if (!divisor || error < minError.error) {
			     minError = coord; 
			 }
		         lati += coord.latitude;
			 longi += coord.longitude;
			 alti += coord.altitude;
			 error += coord.error;
			 ++divisor;
		     } else {
                         kDebug() << "Unknown Access Point " << MAC << " (" << realAP->ssid() << ")";
                     }
		}
	    }
	}
	if (divisor) {
	    lati /= divisor;
	    longi /= divisor;
	    alti /= divisor;
	    error /= divisor;
	    error += 500;
	    d["latitude"] = QString::number(lati);
	    d["longitude"] = QString::number(longi);
	    d["altitude"] = QString::number(alti);
	    d["accuracy"] = QString::number(error);
	    d["country code"] = minError.countryCode;
	    d["country"] = minError.countryName;
	    d["city"] = minError.city;
	}
	emit dataReady(d);
        m_condition.wait(&m_mutex);
    }
}


WiFi::WiFi(QObject* parent, const QVariantList& args)
    : GeolocationProvider(parent, args), d(new Private())
{
    setUpdateTriggers(SourceEvent);

    d->m_wifiScan = 0;
    
    foreach (Solid::Control::NetworkInterface * iface, Solid::Control::NetworkManager::networkInterfaces()) {
        if (iface->type() == Solid::Control::NetworkInterface::Ieee80211) {
            d->addInterfaceInternal(iface);
            //kDebug() << "Network Interface:" << iface->interfaceName() << iface->driver() << iface->designSpeed();
        }
    }
    
    QString configPath = KStandardDirs::locateLocal("data","wifigeolocation/accesspoints", true);
    
    if (!configPath.isEmpty()) {    
        KUrl configUrl(configPath);
	d->m_backends.append(new FileBackend(configUrl));
    }
    d->m_wifiScan = new WiFiScan(this);
    connect(d->m_wifiScan, SIGNAL(dataReady(const Plasma::DataEngine::Data&)),
	             this, SLOT(setData(const Plasma::DataEngine::Data&)));

    setIsAvailable(true);

}

WiFi::~WiFi()
{
    delete d->m_wifiScan;
    delete d;
}

void WiFi::update()
{
    if (d->m_wifiScan) {
        d->m_wifiScan->update();
    }
}

void WiFi::addAccessPointData(const QString & mac, const Coordinate & data) {
    foreach (CoordinateBackend* src, d->m_backends) {
	src->addAccessPointData(mac, data);
    }
}

bool WiFi::saveAccessPointData() {
    foreach (CoordinateBackend* src, d->m_backends) {
	if (!src->saveAccessPointData()) return false;
    }
    return true;
}

K_EXPORT_PLASMA_GEOLOCATIONPROVIDER(wifi, WiFi)

#include "location_wifi.moc"
