/***************************************************************************
 *   Copyright (C) 2008 by Gerhard Gappmeier <gappy1502@gmx.net>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// Local includes
#include "stocksengine.h"

// Qt includes
#include <QString>
#include <QVariantList>
#include <QFile>
#include <QTextStream>
// KDE includes
#include <KIO/Job>
#include <KIO/StoredTransferJob>

//#define TEST_DATA
#ifdef TEST_DATA
/** Some test data from de.old.finance.yahoo.com, so we don't need to spam the yahoo service.
 * As you can see, the format is not always the same!
 */
const char* testdata[5] = {
    /* ID;Value;N/A;Time;Date;Change;N/A;N/A;N/A;N/A */
    "GB0030932676EUR;22,35;N/A;14:00;11/04/2008;+0,12;N/A;N/A;N/A;N/A",
    /* ID;Value;N/A;Time;Date;Change;BeginningRate;DayRangeHigh;DayRangeLow */
    "^GDAXI;6547,86;N/A;10:44;14/04/2008;-55,71;6561,90;6564,87;6520,85;0",
    /* ID;Value;N/A;Time;Date;Change;N/A;N/A;N/A;N/A */
    "^NYA;8936,11;N/A;23:05;11/04/2008;0,00;0,00;0,00;0,00;0",
    /* ID;Value;Time;Date;?;?;?;?;? */
    "SI;106,70;22:02;11/04/2008;0,00;0,00;0,00;0,00;0",
    /* ID;Value;Time;Date;Change;BeginningRate;DayRangeHigh;DayRangeLow;Volume */
    "SIE.DE;67,68;10:50;14/04/2008;-0,57;67,51;67,93;66,85;1584774"
};
#endif

StocksEngine::StocksEngine(QObject *parent, const QVariantList &args)
    : Plasma::DataEngine(parent, args)
{
    kDebug() << "Created StocksEngine instance";
    
    // don't update faster than once a minute
    setMinimumPollingInterval(60000);
    // dafault update rate is 10 minutes
    setPollingInterval(600000);
    
    m_sPath = "http://de.finance.yahoo.com/d/quotes.csv?f=sl1d1t1c1ohgv&e=.csv&s=%s";
}

StocksEngine::~StocksEngine()
{
    kDebug() << "Deleted StocksEngine instance";
}

void StocksEngine::setPath(QString sPath)
{
    m_sPath = sPath;
}

void StocksEngine::setRegEx(QString sRegEx)
{
    m_sRegEx = sRegEx;
}

bool StocksEngine::sourceRequestEvent(const QString &name)
{
    kDebug() << "name='" << name << "'";

    // create empty source
    setData(name, DataEngine::Data());
    
    updateSourceEvent(name);
    
    return true;
}

/** Updates the stock information for the given source.
 * @param source the name of the source that should be updated.
 * @return true if the data was changed, or false if there was no change or if the change will occur later.
 */
bool StocksEngine::updateSourceEvent(const QString &source)
{
    QString sRequest = m_sPath;
    QString tmpFile;
    
    sRequest.replace("%s", source);
    
#ifdef TEST_DATA
    if (source == "GB0030932676EUR")
    {
        parseCSVLine(source, testdata[0]);
    }
    else if (source == "^GDAXI")
    {
        parseCSVLine(source, testdata[1]);
    }
    else if (source == "^NYA")
    {
        parseCSVLine(source, testdata[2]);
    }
    else if (source == "SI")
    {
        parseCSVLine(source, testdata[3]);
    }
    else if (source == "SIE.DE")
    {
        parseCSVLine(source, testdata[4]);
    }
#else
    kDebug() << "GET " << sRequest;
  
    KIO::StoredTransferJob *job = KIO::storedGet(sRequest, KIO::NoReload, KIO::HideProgressInfo);
    job->setProperty( "source", source );
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( jobDone( KJob* ) ) );

#endif
    
    // change will occur later
    return false;
}

void StocksEngine::jobDone(KJob *job)
{
    KIO::StoredTransferJob *pStoredJob = qobject_cast<KIO::StoredTransferJob*>( job );

    if (pStoredJob)
    {
        QString sSource = pStoredJob->property("source").toString();
        QString sLine   = pStoredJob->data();

        parseCSVLine(sSource, sLine);
    }
}

void StocksEngine::parseCSVLine(const QString &sSource, const QString &sLine)
{
    QStringList list = sLine.split(";");
    for (int i=0; i<list.size(); i++)
    {
        kDebug() << i << ": " << list[i];
    }
            
    if (list.size() > 4)
    {
        double dblValue = list[1].toDouble();
        double dblOldValue;
        double dblChange, dblPercentChange;
        QString sTime;
        QString sDate;
                
        if (list[2] == "N/A")
        {
            sTime = list[3];
            sDate = list[4];
            dblChange = list[5].toDouble();
                    
        }
        else
        {
            sTime = list[2];
            sDate = list[3];
            dblChange = list[4].toDouble();
        }
                
        dblOldValue = dblValue - dblChange;
        if (dblOldValue == 0.0) dblOldValue = 1.0; // avoid division by zero
        dblPercentChange = 100.0 / dblOldValue * dblChange;
                
        qDebug("OldValue: %.2f, Value: %.2f, Change: %.2f, PercentChange %.2f%%", dblOldValue, dblValue, dblChange, dblPercentChange);
                
                // store data in dataEngine
        kDebug() << "Calling DataEngine::setData for source " << sSource;
        setData(sSource, "Value",         dblValue);
        setData(sSource, "OldValue",      dblOldValue);
        setData(sSource, "Change",        dblChange);
        setData(sSource, "PercentChange", dblPercentChange);
    }
}

K_EXPORT_PLASMA_DATAENGINE(stocks, StocksEngine)


