/***************************************************************************
 *   Copyright (C) 2007 by Teemu Rytilahti <tpr@d5k.net>                   *
 *   Copyright (C) 2009 by Rivo Laks <rivolaks@hot.ee>                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#include "emhi.h"

#include <KUrl>
#include <KDebug>

#include <kio/job.h>
#include <plasma/weather/weatherutils.h>

#include <QTimer>


EMHIIon::EMHIIon(QObject* parent, const QVariantList& args) : IonInterface(parent, args)
{
    kDebug();

    mPendingListRequest = false;
    mTransferJob = 0;
}

EMHIIon::~EMHIIon()
{
    delete mTransferJob;
}

void EMHIIon::init()
{
    kDebug();
    mUrl = "http://www.emhi.ee/ilma_andmed/xml/observations.php";
    mIsFetching = false;

    // Populate the map for EMHI condition name -> IonInterface enum mapping
    mConditionName2Icon["clear"] = IonInterface::ClearDay;
    mConditionName2Icon["few clouds"] = IonInterface::FewCloudsDay;
    mConditionName2Icon["variable clouds"] = IonInterface::PartlyCloudyDay;
    mConditionName2Icon["cloudy with clear spells"] = IonInterface::PartlyCloudyDay;
    mConditionName2Icon["overcast"] = IonInterface::Overcast;

    mConditionName2Icon["light snow shower"] = IonInterface::RainSnow;
    mConditionName2Icon["moderate snow shower"] = IonInterface::RainSnow;
    mConditionName2Icon["heavy snow shower"] = IonInterface::RainSnow;

    mConditionName2Icon["light shower"] = IonInterface::LightRain;
    mConditionName2Icon["moderate shower"] = IonInterface::Showers;
    mConditionName2Icon["heavy shower"] = IonInterface::Showers;

    mConditionName2Icon["light rain"] = IonInterface::LightRain;
    mConditionName2Icon["moderate rain"] = IonInterface::Rain;
    mConditionName2Icon["heavy rain"] = IonInterface::Rain;

    mConditionName2Icon["glaze"] = IonInterface::Showers;
    mConditionName2Icon["light sleet"] = IonInterface::Showers;
    mConditionName2Icon["moderate sleet"] = IonInterface::Rain;

    mConditionName2Icon["light snowfall"] = IonInterface::LightSnow;
    mConditionName2Icon["moderate snowfall"] = IonInterface::Snow;
    mConditionName2Icon["heavy snowfall"] = IonInterface::Snow;
    mConditionName2Icon["blowing snow"] = IonInterface::Flurries;
    mConditionName2Icon["drifting snow"] = IonInterface::Flurries;

    mConditionName2Icon["hail"] = IonInterface::NotAvailable;
    mConditionName2Icon["mist"] = IonInterface::Mist;
    mConditionName2Icon["fog"] = IonInterface::NotAvailable;
    mConditionName2Icon["thunder"] = IonInterface::Thunderstorm;
    mConditionName2Icon["thunderstorm"] = IonInterface::Thunderstorm;

    setInitialized(true);
}

bool EMHIIon::updateIonSource(const QString& source)
{
    kDebug() << source;
    QStringList src = source.split("|");
    Q_ASSERT(src[0] == "emhi");

    // Requests are always added to the queue here, and then processed later. This allows having
    //  common and simple code here, no matter if the data is available or if it has to be downloaded first.
    if (src[1] == "list") {
        mPendingListRequest = true;
    } else if (src[1] == "validate") {
        mPendingValidationRequests.append(src[2]);
    } else if (src[1] == "weather") {
        mPendingWeatherRequests.append(src[2]);
    } else {
        kDebug() << "Unknown request:" << source;
        return false;
    }

    processPendingRequests();

    return true;
}

QStringList EMHIIon::getLocationsList()
{
    QStringList locations;

    QDomNodeList nodes = mXMLDocument.elementsByTagName("name");
    for (int i = 0; i < nodes.count(); i++) {
        QDomElement e = nodes.item(i).toElement();
        locations.append(e.text());
    }

    return locations;
}

void EMHIIon::validateLocation(const QString& location)
{
    kDebug() << location;

    QStringList matches;
    QString locationLC = location.toLower();
    foreach (const QString& l, getLocationsList()) {
        if (locationLC.contains(l.toLower()) || l.toLower().contains(locationLC)) {
            matches.append(l);
        }
    }

    QString source = "emhi|validate|" + location;
    if (matches.isEmpty()) {
        setData(source, "validate", "emhi|invalid|single|" + location);
    } else if (matches.count() == 1) {
        setData(source, "validate", "emhi|valid|single|place|" + matches.first());
    } else {
        setData(source, "validate", "emhi|valid|multiple|place|" + matches.join("|place|"));
    }
}

void EMHIIon::listLocations()
{
    setData("emhi|list", "validate", "emhi|valid|multiple|place|" + getLocationsList().join("|place|"));
}


void EMHIIon::updateWeatherData(const QString& location)
{
    kDebug() << location;

    QString source = "emhi|weather|" + location;

    QDomNodeList nodes = mXMLDocument.elementsByTagName("station");
    for (int i = 0; i < nodes.count(); i++) {
        QDomElement e = nodes.item(i).toElement();
        if (e.firstChildElement("name").text() != location) {
            continue;
        }

        DataEngine::Data data;
        data["Place"] = location;
        data["Station"] = location;
        data["Credit"] = "EMHI";
        data["Credit Url"] = "http://www.emhi.ee";
        data["Total Weather Days"] = 0;

        data["Current Conditions"] = e.firstChildElement("phenomenon").text();
        data["Condition Icon"] = getWeatherIcon(mConditionName2Icon, e.firstChildElement("phenomenon").text());

        data["Temperature"] = e.firstChildElement("airtemperature").text();
        data["Temperature Unit"] = WeatherUtils::Celsius;

        data["Pressure"] = e.firstChildElement("airpressure").text();
        data["Pressure Unit"] = WeatherUtils::Millibars;

        data["Humidity"] = e.firstChildElement("relativehumidity").text();
        data["Visibility"] = e.firstChildElement("visibility").text();
        data["Visibility Unit"] = WeatherUtils::Kilometers;

        data["Wind Speed"] = e.firstChildElement("windspeed").text();
        data["Wind Speed Unit"] = WeatherUtils::MetersPerSecond;
        data["Wind Gust"] = e.firstChildElement("windspeedmax").text();
        data["Wind Gust Unit"] = WeatherUtils::MetersPerSecond;
        data["Wind Direction"] = WeatherUtils::degreesToCardinal(e.firstChildElement("winddirection").text().toFloat());

        data["Precipitations"] = e.firstChildElement("precipitations").text();

        setData(source, data);
    }
}

bool EMHIIon::checkDataAvailability(int maxAge)
{
    if (mFetchTime.isNull()) {
        kDebug() << "No data available, starting fetch";
        fetch();
        return false;
    } else if (maxAge >= 0 && mFetchTime.secsTo(QDateTime::currentDateTime()) > maxAge) {
        kDebug() << "Data is obsolete: age:" << mFetchTime.secsTo(QDateTime::currentDateTime()) << ", starting fetch";
        fetch();
        return false;
    } else {
        return true;
    }
}

void EMHIIon::reset()
{
#if KDE_IS_VERSION(4, 2, 50)
    emit resetCompleted(this, true);
#endif
}

void EMHIIon::fetch()
{
    kDebug();
    if(!mIsFetching) {
        mIsFetching = true;
        kDebug() << "Starting fetch";
        mSource.clear();
        mTransferJob = KIO::get(mUrl, KIO::Reload, KIO::HideProgressInfo);
        connect(mTransferJob, SIGNAL(data(KIO::Job *, const QByteArray &)), this, SLOT(dataFetched(KIO::Job *, const QByteArray &)));
        connect(mTransferJob, SIGNAL(result(KJob *)), this, SLOT(fetchingDone(KJob *)));
    }
}

void EMHIIon::dataFetched(KIO::Job *job, const QByteArray &data)
{
    Q_UNUSED(job);
    mSource += data;
}

void EMHIIon::fetchingDone(KJob *job)
{
    if (job && job->error()) {
        kDebug() << "error in fetching: " << job->error() << job->errorString(); 
    } else {
        kDebug() << "fetching done!";
    }

    mTransferJob = 0;
    mFetchTime = QDateTime::currentDateTime();
    mIsFetching = false;

    mXMLDocument.setContent(mSource);

    processPendingRequests();
}

void EMHIIon::processPendingRequests()
{
    // Make sure that the data is available
    if (mIsFetching) {
        return;
    }
    if (!checkDataAvailability()) {
        return;
    }

    // Process list request
    if (mPendingListRequest) {
        listLocations();
        mPendingListRequest = false;
    }

    // Process pending validation requests
    foreach (const QString& location, mPendingValidationRequests) {
        validateLocation(location);
    }
    mPendingValidationRequests.clear();

    // Process pending weather data requests
    const int maxDataAge = 30*60;
    if (!mPendingWeatherRequests.isEmpty() && !checkDataAvailability(maxDataAge)) {
        return;
    }
    foreach (const QString& location, mPendingWeatherRequests) {
        updateWeatherData(location);
    }
    mPendingWeatherRequests.clear();
}

#include "emhi.moc"

