/***************************************************************************
 *   Copyright (C) 2007 by Shawn Starr <shawn.starr@rogers.com>            *
 *   Copyright (C) 2008 by Petri Damstén <damu@iki.fi>                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#include "ion_savonia_fi.h"
#include <KIO/Job>
#include <KUnitConversion/Converter>
#include <KJob>
#include <KDebug>
#include <QDomDocument>

static const char *URL = "http://weather.savonia.fi/datas/savoniaweather.xml";

SavoniaFi::SavoniaFi(QObject *parent, const QVariantList &args)
        : IonInterface(parent, args), m_job(0)
{
    Q_UNUSED(args);
    connect(&m_clearTimer, SIGNAL(timeout()), this, SLOT(clearResult()));
}

SavoniaFi::~SavoniaFi()
{
}

void SavoniaFi::init()
{
    setInitialized(true);
}

bool SavoniaFi::updateIonSource(const QString& source)
{
    // We expect the applet to send the source in the following tokenization:
    // ionname|validate|place_name - Triggers validation of place
    // ionname|weather|place_name - Triggers receiving weather of place
    QStringList sourceAction = source.split('|');

    if (sourceAction[1] == QString("validate")) {
        setData(source, "validate", QString("savonia_fi|valid|single|place|%1").arg("Kuopio"));
        return true;
    } else if (sourceAction[1] == QString("list")) {
       setData(source, "validate", QString("savonia_fi|valid|single|place|%1").arg("Kuopio"));
       return true;
    } else if (sourceAction[1] == QString("weather")) {
        if (m_job) {
            return true;
        }
        m_source = source;
        if (!m_data.isEmpty()) {       // This is cleared by timer so we don't update more than once
            setData(m_source, m_data); // a minute.
        } else {
            m_result = "";
            m_data.clear();
            m_clearTimer.stop();
            m_job = KIO::get(KUrl(URL), KIO::Reload, KIO::HideProgressInfo);
            if (m_job) {
                m_job->addMetaData("cookies", "none"); // Disable displaying cookies
                connect(m_job, SIGNAL(data(KIO::Job*, const QByteArray&)),
                        this, SLOT(data(KIO::Job*, const QByteArray&)));

                connect(m_job, SIGNAL(result(KJob*)), this, SLOT(result(KJob*)));
                m_job->start();
            }
        }
        return true;
    } else {
        return false;
    }
    return false;
}

void SavoniaFi::data(KIO::Job *job, const QByteArray &data)
{
    Q_UNUSED(job);
    //kDebug() << data;
    m_result += QString::fromUtf8(data);
}

void SavoniaFi::result(KJob *job)
{
    //kDebug() << m_source << m_result;
    if (job && job->error()) {
        kDebug() << job->error() << job->errorString();
        return;
    }
    static QStringList cardinalDirections = QStringList() <<
        "N" << "NNE" << "NE" << "ENE" << "E" << "ESE" << "SE" << "SSE" <<
        "S" << "SSW" << "SW" << "WSW" << "W" << "WNW" << "NW" << "NNW";

    QDomDocument doc;

    doc.setContent(m_result);
    QDomNodeList list = doc.elementsByTagName("sensorData");
    m_data["Place"]  = "Kuopio, Finland";
    m_data["Credit"] = "http://weather.savonia.fi";
    m_data["Credit Url"] = "http://weather.savonia.fi";
    m_data["Observation Period"] = doc.elementsByTagName("MeasurementTimeUTC")
            .item(0).toElement().text();
    for (int i = 0; i < list.count(); ++i) {
        int id = list.item(i).firstChildElement("ID").text().toInt();
        int subId = list.item(i).firstChildElement("SubID").text().toInt();
        QString value = list.item(i).firstChildElement("Value").text();
        QString unit = list.item(i).firstChildElement("Unit").text();
        //kDebug() << id << subId << value << unit;
        switch (id) {
            case 16:
                if (subId == 0) {
                    m_data["Wind Speed"] =
                            QString::number(value.toDouble() / 1000.0 * 3600.0, 'f', 1);
                    m_data["Wind Speed Unit"] = KUnitConversion::KilometerPerHour;
                }
                break;
            case 19:
                if (subId == 0) {
                    m_data["Wind Direction"] =
                            cardinalDirections[(int)((value.toDouble() + 11.25) / 22.5) % 16];
                }
                break;
            case 21:
                if (subId == 0) {
                    m_data["Pressure Unit"] = KUnitConversion::Hectopascal;
                    m_data["Pressure"] = QString::number(value.toDouble(), 'f', 1);
                } else if (subId == 1) {
                    m_data["Pressure Tendency"] = value;
                }
                break;
            case 22:
                if (subId == 0) {
                    m_data["Temperature"] = QString::number(value.toDouble(), 'f', 1);
                    m_data["Temperature Unit"] = KUnitConversion::Celsius;
                }
                break;
            case 24:
                if (subId == 0) {
                    m_data["Humidity"] = QString::number(value.toDouble(), 'f', 0);
                }
                break;
        }
    }
    m_data["Latitude"] = "62.888833";
    m_data["Longitude"] = "27.62925";
    setData(m_source, m_data);

    if (!m_clearTimer.isActive()) {
        m_clearTimer.start(60000);
    }
    m_job = 0;
}

void SavoniaFi::clearResult()
{
    m_data.clear();
    m_clearTimer.stop();
}

void SavoniaFi::setTimezoneFormat(const QString& tz)
{
    Q_UNUSED(tz);
}

bool SavoniaFi::timezone()
{
    return true;
}

void SavoniaFi::reset()
{
	updateAllSources(); 
}

#include "ion_savonia_fi.moc"
