/*
 *   Copyright 2008 Aike J Sommer <dev@aikesommer.name>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


#include <KDebug>

#include "outputservice.h"

#include <kephal/outputs.h>
#include <kephal/kephal.h>


OutputService::OutputService(QObject * parent, Kephal::Output * output)
    : Plasma::Service(parent)
{
    setName("screens-output");
    m_output = output;
}

Plasma::ServiceJob * OutputService::createJob(const QString & operation, QMap<QString,QVariant> & parameters)
{
    kDebug() << "Job" << operation << "with arguments" << parameters << "requested";
    return new OutputServiceJob(m_output, operation, parameters, this);
}



OutputServiceJob::OutputServiceJob(Kephal::Output * output, const QString & operation,
        QMap<QString,QVariant> & parameters, QObject* parent)
    : ServiceJob(output->id(), operation, parameters, parent)
{
    m_output = output;
}

void OutputServiceJob::start() {
    const QString operation = operationName();
    
    if (operation == "size") {
        if (parameters().contains("width") && parameters().contains("height")) {
            int width = parameters()["width"].toInt();
            int height = parameters()["height"].toInt();
            
            QSize size(width, height);
            bool found = false;
            foreach (const QSize& s, m_output->availableSizes()) {
                if (s == size) {
                    m_output->resize(size);
                    found = true;
                    break;
                }
            }
            if (! found) {
                setErrorText(i18n("The 'size' command requires a valid combination of width and height"));
                setError(-2);
            }
        } else {
            setErrorText(i18n("The 'size' command requires a 'width' and a 'height' argument"));
            setError(-2);
        }
    } else if (operation == "position") {
        if (parameters().contains("x") && parameters().contains("y")) {
            int x = parameters()["x"].toInt();
            int y = parameters()["y"].toInt();
            
            QPoint position(x, y);
            bool found = false;
            foreach (const QPoint& p, m_output->availablePositions()) {
                if (p == position) {
                    m_output->move(p);
                    found = true;
                    break;
                }
            }
            if (! found) {
                setErrorText(i18n("The 'position' command requires a valid combination of x and y"));
                setError(-2);
            }
        } else {
            setErrorText(i18n("The 'position' command requires a 'x' and a 'y' argument"));
            setError(-2);
        }
    } else if (operation == "rotation") {
        if (parameters().contains("rotate")) {
            int rotate = parameters()["rotate"].toInt();
            
            switch (rotate) {
                case 0:
                    m_output->rotate(Kephal::RotateNormal);
                    break;
                case 90:
                    m_output->rotate(Kephal::RotateRight);
                    break;
                case 180:
                    m_output->rotate(Kephal::RotateInverted);
                    break;
                case 270:
                    m_output->rotate(Kephal::RotateLeft);
                    break;
                default:
                    setErrorText(i18n("The 'rotation' command requires a valid value for 'rotate' (0, 90, 180 or 270)"));
                    setError(-2);
            }
        } else {
            setErrorText(i18n("The 'rotation' command requires a 'rotate' argument"));
            setError(-2);
        }
    }
    
    emitResult();
}
