/*
Copyright 2009 Beat Wolf <asraniel@fryx.ch>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License or (at your option) version 3 or any later version
accepted by the membership of KDE e.V. (or its successor approved
by the membership of KDE e.V.), which shall act as a proxy
defined in Section 14 of version 3 of the license.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "particles.h"

#include <QFileInfo>
#include <QImage>
#include <QPainter>
#include <QPixmap>

#include <KDebug>
#include <KGlobal>
#include <KStandardDirs>
#include <QTimer>
#include <QVariant>

//TODO: find bug of not showing correctly in mini preview
//#define BENCHMARKPAINT
//#define BENCHMARK
#define TRAIL_STEPS 40

ParticlesWallpaper::ParticlesWallpaper(QObject * parent, const QVariantList & args ) : Plasma::Wallpaper(parent, args)
{
    updateTime = 0;
    m_size = boundingRect().size().toSize();
    QTimer::singleShot(updateTime, this, SLOT(updateView()));
}

ParticlesWallpaper::~ParticlesWallpaper()
{
}

void ParticlesWallpaper::init(const KConfigGroup & config)
{
    m_pColor = config.readEntry("ParticleColor", QColor(Qt::blue));
    m_bgColor = config.readEntry("BackgroundColor", QColor(Qt::black));
    updateTime = config.readEntry("UpdateDelay", 90);
    engine.setParticles(config.readEntry("Particles", 5000));
    engine.setType((Types)config.readEntry("Type", (int)ColidingGalaxies));
    m_alpha = config.readEntry("Trail", 15);
    m_bgColor.setAlpha(m_alpha);
    resetPixmap();
    
    emit update(boundingRect());
}

void ParticlesWallpaper::save(KConfigGroup & config)
{
    config.writeEntry("ParticleColor", m_pColor);
    config.writeEntry("BackgroundColor", m_bgColor);
    config.writeEntry("UpdateDelay", updateTime);
    config.writeEntry("Particles", engine.particles());
    config.writeEntry("Type", (int)engine.getType());
    config.writeEntry("Trail", m_alpha);
}

QWidget * ParticlesWallpaper::createConfigurationInterface(QWidget * parent)
{
    QWidget * configWidget = new QWidget(parent);
    m_ui.setupUi(configWidget);
    m_ui.m_fgColor->setColor(m_pColor);
    m_ui.m_bgColor->setColor(m_bgColor);
    m_ui.updateTime->setValue(updateTime);
    m_ui.particles->setValue(engine.particles());
    m_ui.m_trail->setValue(-((m_alpha - 255)/ TRAIL_STEPS));
    
    m_ui.m_layout->addItem(i18n("Rotating Ring"), QVariant(CentralStarRotatingCircle));
    m_ui.m_layout->addItem(i18n("Random"), QVariant(Random1));
    m_ui.m_layout->addItem(i18n("Random2"), QVariant(Random2));
    m_ui.m_layout->addItem(i18n("Central Point"), QVariant(CentralStar));
    m_ui.m_layout->addItem(i18n("Colliding Galaxies"), QVariant(ColidingGalaxies));
    m_ui.m_layout->addItem(i18n("Test"), QVariant(RotatingCircles));
    m_ui.m_layout->setCurrentIndex(m_ui.m_layout->findData(QVariant(engine.getType())));
    
    connect(m_ui.m_layout, SIGNAL(currentIndexChanged ( int )), SLOT(widgetChanged()));
    connect(m_ui.m_fgColor, SIGNAL(changed(const QColor&)), SLOT(widgetChanged()));
    connect(m_ui.m_bgColor, SIGNAL(changed(const QColor&)), SLOT(widgetChanged()));
    connect(m_ui.updateTime, SIGNAL(valueChanged( int )), SLOT(widgetChanged()));
    connect(m_ui.particles, SIGNAL(valueChanged( int )), SLOT(widgetChanged()));
    connect(m_ui.m_trail, SIGNAL(valueChanged( int )), SLOT(widgetChanged()));

    connect(this, SIGNAL(settingsChanged(bool)), parent, SLOT(settingsChanged(bool)));
    
    return configWidget;
}

void ParticlesWallpaper::widgetChanged()
{
    m_pColor = m_ui.m_fgColor->color();
    m_bgColor = m_ui.m_bgColor->color();
    m_alpha = 255 - m_ui.m_trail->value() * TRAIL_STEPS;
    m_bgColor.setAlpha(m_alpha);
    updateTime = m_ui.updateTime->value();
    engine.setParticles(m_ui.particles->value());
    engine.setType((Types)m_ui.m_layout->itemData(m_ui.m_layout->currentIndex()).toInt());
    
    resetPixmap();
    updatePixmap();
    
    emit update(boundingRect());
    emit settingsChanged(true);
}

/**
 * Paint the default color on the background
 */
void ParticlesWallpaper::resetPixmap()
{
      //create new background
      m_image = QImage(m_size.width() , m_size.height(), QImage::Format_RGB16);
      
      //m_pixmap = QPixmap(m_size.width() , m_size.height());
      if(!m_image.isNull()){
        //QPainter painterLocal(&m_pixmap);
        QPainter painterLocal(&m_image);
        m_bgColor.setAlpha(255);
        painterLocal.setBrush(m_bgColor);
        painterLocal.drawRect(boundingRect());
        painterLocal.end();
        
        m_bgColor.setAlpha(m_alpha);
      }
}

/**
 * Recalculate the particles positions and paint them on the pixmap cache
 */
void ParticlesWallpaper::updatePixmap()
{
    //*************benchmark
    #ifdef BENCHMARK
    static long bench = 0;
    static long benchAmount = 0;
    static long bench2 = 0;
    
    static QTime lastUpdate = QTime::currentTime();
    qDebug() << "last Paint was"<<lastUpdate.msecsTo(QTime::currentTime())<<"ms ago";
    qDebug() << "Drawing" << engine.particles() << "particles";
    lastUpdate = QTime::currentTime();
    
    QTime a = QTime::currentTime();
    #endif
    //*************benchmark
    
    engine.moveParticles();
    
    //*************benchmark
    #ifdef BENCHMARK
    bench2+= a.msecsTo(QTime::currentTime());
    a = QTime::currentTime();
    #endif
    //*************benchmark
    
    const QRectF exposedRect = boundingRect();

    QPainter painter(&m_image);
    //QPainter painter(&m_pixmap);
    painter.setRenderHints(0);
    
    painter.setBrush(m_bgColor);
    painter.drawRect(exposedRect);
    
    //Paint particels
    painter.setBrush(m_pColor);
    painter.setPen(m_pColor);
    
    int batchAmount = qMax(5000, qMin(500, engine.particles() / 100));
    QPolygon poly(batchAmount);
    int drawn = 0;
    
    int drawInBatch = engine.particles() - batchAmount;
    while(drawn < drawInBatch){
        for(int loop = 0; loop < batchAmount; ++loop){
            struct particle *p = engine.getParticle(drawn+loop);
            poly.setPoint(loop, p->p[0], p->p[1]);
        }
        painter.drawPoints(poly);
        drawn += batchAmount;
    }
    
    int rest = engine.particles();
    for(int i = drawn; i < rest; ++i){
        struct particle *p = engine.getParticle(i);
        painter.drawPoint(p->p[0], p->p[1]);
    }
    
    //draw gravity points
    /*painter.setPen(QColor(255,255,255));
    for(int i = 0; i < engine.gravityPoints(); ++i){
        struct gravityPoint *p = engine.getGravityPoint(i);
        painter.drawPoint(p->p[0], p->p[1]);
    }*/
    
    //*************benchmark
    #ifdef BENCHMARK
    bench += a.msecsTo(QTime::currentTime());
    qDebug() << "time to paint" << bench2 / (benchAmount) << bench / (++benchAmount);
    #endif
    //*************benchmark
   
}

void ParticlesWallpaper::paint(QPainter * painter, const QRectF & exposedRect)
{
    #ifdef BENCHMARKPAINT
    QTime lastUpdate = QTime::currentTime();
    #endif
    
    
    //update image if needed
    if(m_size != boundingRect().size().toSize()){
        m_size = boundingRect().size().toSize();
        engine.setSize(m_size.width(), m_size.height());
        resetPixmap();
        
        updatePixmap();
    }
    
    painter->setRenderHints(0);
    
    int x = exposedRect.topLeft().x();
    int y = exposedRect.topLeft().y();
    
    painter->drawImage(x, y, m_image, x, y, exposedRect.width(), exposedRect.height(), Qt::ThresholdDither);
    //painter->drawPixmap(x, y, m_pixmap, x, y, exposedRect.width(), exposedRect.height());
    
    #ifdef BENCHMARKPAINT
    qDebug() << "paint needed "<<lastUpdate.msecsTo(QTime::currentTime())<<"ms";
    #endif
}

void ParticlesWallpaper::updateView()
{
    if(engine.inited()){
        updatePixmap();
    }
    
    emit update(boundingRect());
    QTimer::singleShot(updateTime, this, SLOT(updateView()));
}

#include "particles.moc"
