/***************************************************************************
 *   Copyright (C) 2009 by Wang Hoi <zealot.hoi@gmail.com>                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "korattab.h"
#include "koratgroup.h"
#include "koratscrollwidget.h"

#include <KDebug>

#include <QGraphicsLinearLayout>

namespace Korat
{
class TabPrivate
{
public:
    TabPrivate(Tab *container)
        : q(container)
        , widget(new QGraphicsWidget())
        , layout(new QGraphicsLinearLayout())
        , scrollWidget(new ScrollWidget())
        , scrollLayout(new QGraphicsLinearLayout())
        , relaxedWidth(0)
    {
    }

    void sizeUpdate()
    {
        adjustWidth();
    }

    void updateGroupBorders()
    {
        if (groupList.isEmpty()) {
            return;
        }
        Plasma::FrameSvg::EnabledBorders baseBorders = Plasma::FrameSvg::TopBorder
            | Plasma::FrameSvg::BottomBorder;
        if (groupList.size() == 1) {
            groupList.first()->setEnabledBorders(Plasma::FrameSvg::AllBorders);
            return;
        }
        groupList.first()->setEnabledBorders(baseBorders | Plasma::FrameSvg::LeftBorder);
        for (int i = 1; i < groupList.size() - 1; ++i) {
            groupList.at(i)->setEnabledBorders(baseBorders);
        }
        groupList.last()->setEnabledBorders(baseBorders | Plasma::FrameSvg::RightBorder);
    }

    qreal getWidth(const WeightList &weightList)
    {
        qreal width = 0;
        int i = 0;
        foreach (Item *item, groupList) {
            WeightList tempWeightList;
            tempWeightList = weightList.mid(i,item->availableWidths().size());

            width += item->getWidth(tempWeightList);
            i += tempWeightList.size();
        }
        return width;
    }

    bool greedyRelax()
    {
        if (groupWeightList.empty()) {
            return false;
        }
        if (relaxedWeightList.isEmpty()) {
            indexList.clear();
            foreach (const WeightList &weightList, groupWeightList) {
                relaxedWeightList << weightList.first();
                indexList << 1;
            }
            relaxedWidth = getWidth(relaxedWeightList);
            return true;
        }
        int maximumWeight = -1;
        int indexToRelax = -1;
        for (int i = 0; i < relaxedWeightList.size(); i++) {
            if ((indexList[i] < groupWeightList[i].size()) && (relaxedWeightList[i] >= maximumWeight)) {
                maximumWeight = relaxedWeightList[i];
                indexToRelax = i;
            }
        }
        if (maximumWeight == -1) {
            return false;
        } else {
            relaxedWeightList[indexToRelax] = groupWeightList[indexToRelax].at(indexList[indexToRelax]);
            indexList[indexToRelax]++;
            relaxedWidth = getWidth(relaxedWeightList);
//            kDebug() << relaxedWeightList << relaxedWidth;
            return true;
        }
    }

    // do all the adaptive layout work.....
    void adjustWidth()
    {
        // sometimes q->size() is invalid
        if (qFuzzyCompare(q->size().width(),0)) {
            return;
        }
        relaxedWeightList.clear();
        indexList.clear();
        relaxedWidth = widget->size().width();
        groupWeightList.clear();
        foreach (Group *grp, groupList) {
            groupWeightList << grp->availableWidths();
        }
//        kDebug() << groupWeightList;
        while (greedyRelax() && (relaxedWidth > q->size().width())) {
            ;
        }
        if (relaxedWidth > q->size().width()) {
        }
//        if ((relaxedWidth <= q->size().width()) || (relaxedWidth < widget->size().width())) {
            int i = 0;
            int len = 0;
            foreach (Item *item, groupList) {
                WeightList tempWeightList;
                len = item->availableWidths().size();
                tempWeightList = relaxedWeightList.mid(i,len);

//                kDebug() << "ADJUST WIDTH:" << tempWeightList;
                item->adjustWidth(tempWeightList);
                i += len;
            }
//        }
    }

    Tab *q;

    QGraphicsWidget *widget;
    QGraphicsLinearLayout *layout;
    ScrollWidget *scrollWidget;
    QGraphicsLinearLayout *scrollLayout;

    QString title;
    QIcon icon;

    QList<Group *> groupList;

    QList<WeightList> groupWeightList;
    qreal relaxedWidth;
    WeightList relaxedWeightList;
    QList<int> indexList;
};

Tab::Tab(const QString &title,QGraphicsItem *parent)
    : QGraphicsWidget(parent)
    , d(new TabPrivate(this))
{
    setFlag(QGraphicsItem::ItemClipsChildrenToShape, true);
    setTitle(title);

    setContentsMargins(0,0,0,0);

    d->layout->setSpacing(0);
    d->layout->setContentsMargins(0,0,0,0);
    d->widget->setLayout(d->layout);
    d->scrollWidget->setWidget(d->widget);

    d->scrollLayout->setSpacing(0);
    d->scrollLayout->setContentsMargins(0,0,0,0);
    d->scrollLayout->addItem(d->scrollWidget);
    setLayout(d->scrollLayout);
}

Tab::Tab(const QIcon &icon,QGraphicsItem *parent)
    : QGraphicsWidget(parent)
    , d(new TabPrivate(this))
{
    setFlag(QGraphicsItem::ItemClipsChildrenToShape, true);
    setIcon(icon);

    d->layout->setSpacing(0);
    d->layout->setContentsMargins(0,0,0,0);
    d->widget->setLayout(d->layout);
    d->scrollWidget->setWidget(d->widget);

    d->scrollLayout->setSpacing(0);
    d->scrollLayout->setContentsMargins(0,0,0,0);
    d->scrollLayout->addItem(d->scrollWidget);
    setLayout(d->scrollLayout);
}

Tab::~Tab()
{
    delete d;
}

void Tab::addGroup(Group *grp)
{
    d->layout->addItem(grp);
    d->groupList << grp;
    connect(grp,SIGNAL(sizeChanged()),this,SLOT(sizeUpdate()));
    d->updateGroupBorders();
}

void Tab::removeGroup(Group *grp)
{
    d->layout->removeItem(grp);
    d->groupList.removeAll(grp);
    disconnect(grp,SIGNAL(sizeChanged()),this,SLOT(sizeUpdate()));
    d->updateGroupBorders();
}

QString Tab::title() const
{
    return d->title;
}

void Tab::setTitle(const QString &title)
{
    d->title = title;
}

QIcon Tab::icon() const
{
    return d->icon;
}

void Tab::setIcon(const QIcon &icon)
{
    d->icon = icon;
}

void Tab::resizeEvent(QGraphicsSceneResizeEvent *event)
{
    QGraphicsWidget::resizeEvent(event);
    d->adjustWidth();
}

} // namespace Korat

#include "korattab.moc"
// vim: sw=4 sts=4 et tw=100
