
#include "propertyeditor.h"

#include <QGridLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSpinBox>
#include <QCheckBox>


PropertyEditor::PropertyEditor(QWidget* parent) :
    QWidget(parent), m_node(NULL)
{
    connect(&m_signalMapper, SIGNAL(mapped(const QString&)), this, SLOT(valueChanged(const QString&)));
}

void PropertyEditor::setNode(Quasar::Node* node)
{
    m_node = node;
    qDeleteAll(children());
    delete layout();

    if (m_node == NULL)
        return;

    QGridLayout* layout = new QGridLayout();

    int row = 0;
    foreach (QString propName, node->inputKeys())
    {
        QWidget* w = createEditor(propName);
        if (!w)
            continue;
        QLabel* label = new QLabel(propName + ":");
        label->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
        layout->addWidget(label, row, 0);
        layout->addWidget(w, row, 1);
        row++;
    }
    layout->setColumnStretch(0, 2);
    layout->setColumnStretch(1, 4);
    layout->setColumnStretch(2, 1);
    layout->setRowStretch(row, 1);

    setLayout(layout);
}

void PropertyEditor::valueChanged(const QString& propName)
{
    if (m_node == NULL)
        return;
    QWidget* w = qobject_cast<QWidget*>(m_signalMapper.mapping(propName));
    int type = m_node->propertyType(propName);
    const char* widgetProp = 0;
    switch (type) {
        case QVariant::String: widgetProp = "text"; break;
        case QVariant::Int: widgetProp = "value"; break;
        case QVariant::Bool: widgetProp = "checked"; break;
    }
    
    if (!w || !widgetProp || type == QVariant::Invalid)
        return;

    QVariant value = w->property(widgetProp);
    if (!m_node->setPropertyValue(propName, value))
        qWarning("PropertyEditor::valueChanged(): could not update node value.");
    else
        emit propertyChanged();
    w->setProperty(widgetProp, m_node->propertyValue(propName));
}

QWidget* PropertyEditor::createEditor(const QString& propName)
{
    if (m_node == NULL)
        return NULL;
    int type = m_node->propertyType(propName);
    if (type == QVariant::Invalid)
        return NULL;
    QVariant propValue = m_node->propertyValue(propName);

    QWidget* w = NULL;
    switch (type)
    {
        case QVariant::String:
            w = new QLineEdit(propValue.toString());
            connect(w, SIGNAL(editingFinished()), &m_signalMapper, SLOT(map()));
            break;

        case QVariant::Int: {
            QSpinBox* sb = new QSpinBox();
            const Quasar::Property prop = m_node->property(propName);
            if (prop.min.isValid() && prop.max.isValid())
                sb->setRange(prop.min.toInt(), prop.max.toInt());
            sb->setValue(propValue.toInt());
            connect(sb, SIGNAL(valueChanged(int)), &m_signalMapper, SLOT(map()));
            w = sb;
            break;
        }

        case QVariant::Bool: {
            QCheckBox* cb = new QCheckBox();
            cb->setChecked(propValue.toBool());
            connect(cb, SIGNAL(stateChanged(int)), &m_signalMapper, SLOT(map()));
            w = cb;
            break;
        }
    }

    if (w) 
        m_signalMapper.setMapping(w, propName);
    return w;
}

#include "moc_propertyeditor.cpp"
