#include "window.h"

#include "propertyeditor.h"
#include "widget.h"

#include "composition.h"
#include "node.h"

#include <QFileDialog>
#include <QFileInfo>
#include <QTreeWidget>
#include <QDockWidget>
#include <QPushButton>
#include <QVBoxLayout>
#include <QDebug>
#include <QTimer>



Window::Window()
    : QMainWindow(),
      m_compo(0)
{
    ui.setupUi(this);

    m_location = QLatin1String(":/images/sample.png");
    m_compo = createComposition("/color/saturation");

    setupDockWidgets();
    setupFilters();

    m_widget = new Widget();
    m_widget->setComposition(m_compo);

    connect(m_editor, SIGNAL(edited()),
            m_widget, SLOT(update()));
    connect(m_treeWidget, SIGNAL(itemActivated(QTreeWidgetItem *, int)),
            this, SLOT(itemActivated(QTreeWidgetItem *, int)));
    setCentralWidget(m_widget);
}

void Window::setupDockWidgets()
{
    QDockWidget *dockWidget = new QDockWidget(tr("Filters"), this);
    dockWidget->setAllowedAreas(Qt::LeftDockWidgetArea |
                                Qt::RightDockWidgetArea);
    QWidget *widget = new QWidget;
    QVBoxLayout *layout = new QVBoxLayout;
    widget->setLayout(layout);
    m_treeWidget = new QTreeWidget;

    m_fileButton = new QPushButton("Select Image");
    connect(m_fileButton, SIGNAL(pressed()),
            this, SLOT(selectFile()));

    layout->addWidget(m_fileButton);
    layout->addWidget(m_treeWidget);

    dockWidget->setWidget(widget);
    addDockWidget(Qt::LeftDockWidgetArea, dockWidget);

    dockWidget = new QDockWidget(tr("Properties"), this);
    dockWidget->setAllowedAreas(Qt::LeftDockWidgetArea |
                                Qt::RightDockWidgetArea);
    m_editor = new PropertyEditor();
    m_editor->setNode(m_filterNode);
    dockWidget->setWidget(m_editor);
    addDockWidget(Qt::RightDockWidgetArea, dockWidget);
}

void Window::setupFilters()
{
    Q_ASSERT(m_widget);

    //qDebug()<<"Groups = "<<m_compo->groups();
    foreach(QString group, m_compo->groups()) {
        if (group == QLatin1String("input") ||
            group == QLatin1String("output")) {
            qDebug()<<"Skipping group "<<group;
            continue;
        }
        qDebug()<<"In group "<<group;
        QTreeWidgetItem *parent = new QTreeWidgetItem;
        parent->setText(0, group);
        m_treeWidget->addTopLevelItem(parent);
        QStringList filters = m_compo->nodesInGroup(group);
        qDebug()<<"\tFilters: "<<filters;
        foreach(QString filter, filters) {
            QTreeWidgetItem *item = new QTreeWidgetItem(parent);
            item->setText(0, filter);
        }
    }
}

Quasar::Composition * Window::createComposition(const QString &filter)
{
    delete m_compo;
    m_compo = new Quasar::Composition;

    m_inputNode = m_compo->createNode("/input/file");
    Q_ASSERT(m_inputNode);
    m_inputNode->setPropertyValue("filename", m_location);

    m_filterNode = m_compo->createNode(filter);
    Q_ASSERT(m_filterNode);
    Quasar::link(m_inputNode, "texture", m_filterNode, "texture");

    m_outputNode = m_compo->createNode("/output/render");
    Q_ASSERT(m_outputNode);
    Quasar::link(m_filterNode, "output", m_outputNode, "texture");

    return m_compo;
}

void Window::itemActivated(QTreeWidgetItem *item, int)
{
    if (!item || !item->parent()) {
        return;
    }
    QTreeWidgetItem *group = item->parent();

    QString filterName = QString("/%1/%2").arg(group->text(0)).arg(item->text(0));
    qDebug()<<"Selected filter "<<filterName;

    m_compo = createComposition(filterName);
    m_widget->setComposition(m_compo);
    m_editor->setNode(m_filterNode);

    QTimer::singleShot(0, m_widget, SLOT(update()));
}

void Window::selectFile()
{
    QString filename = QFileDialog::getOpenFileName(this, "Select file",
                                                    QDir::currentPath(),
                                                    tr("Image Files (*.png *.jpg *.JPG)"));
    if (!filename.isEmpty()) {
        m_location = filename;
        m_inputNode->setPropertyValue("filename", m_location);
        QFileInfo info(filename);
        m_fileButton->setText(info.fileName());
        QTimer::singleShot(0, m_widget, SLOT(update()));
    }
}
