#include "composition.h"

#include "filternode.h"
#include "texture.h"
#include "nodeloader.h"
#include "node.h"

#include <QGLFramebufferObject>
#include <QGLPixelBuffer>
#include <qgl.h>
#include <qset.h>
#include <qdebug.h>

#include <assert.h>

namespace Quasar
{

class Composition::Private
{
public:
    void prepareContext()
    {
        const QGLContext *ctx = QGLContext::currentContext();
        if (!ctx) {
            qDebug()<<"Pixel buffer context";
            localContext = new QGLPixelBuffer(1000, 1000);
        }
    }
    void makeCurrent()
    {
        if (localContext) {
            qDebug()<<"X Pixel buffer context";
            localContext->makeCurrent();
            glClearColor(0.3, 0.3, 0.3, 1);
            glViewport(0, 0, 1000, 1000);
            glMatrixMode(GL_PROJECTION);
            glLoadIdentity();
            glOrtho(0, 1000, 0, 1000, -999999, 999999);
            glMatrixMode(GL_MODELVIEW);
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        }
    }
    void doneCurrent()
    {
        if (localContext)
             localContext->doneCurrent();
    }
    void layoutGraph();
    void schedule();
    void execute()
    {
        QSet<void*> processedNodes;
        QList<Node*> nodesToProcess = processingNodes;
        qDebug("Execution starts");
        QRectF surfaceRect;
        //size computation
        foreach(Node *node, nodesToProcess) {
            if (node->isGraphicsNode()) {
                GraphicsNode *gn = qobject_cast<GraphicsNode*>(node);
                surfaceRect |= gn->domainOfDefinition(surfaceRect);
            }
        }
        qDebug()<<"\tdomain of definition is "<<surfaceRect;
        Q_ASSERT(!surfaceRect.isNull());
        //surface allocation
        QGLFramebufferObject *fbo = 0;
        foreach(Node *node, nodesToProcess) {
            if (node->isRendererNode()) {
                RendererNode *rn = qobject_cast<RendererNode*>(node);
                QGLFramebufferObject *currentFbo =
                    rn->framebufferObject();
                if (!currentFbo ||
                    currentFbo->width() != surfaceRect.width() ||
                    currentFbo->height() != surfaceRect.height()) {
                    if (!fbo) {
                        fbo = new QGLFramebufferObject(surfaceRect.width(),
                                                       surfaceRect.height());
                    }
                    rn->setFramebufferObject(fbo);
                }
            }
        }
        while (!nodesToProcess.isEmpty()) {
            nodesToProcess = processNodes(nodesToProcess, processedNodes);
        }
    }
    inline void process(Node *node)
    {
        node->process();
    }
    inline QList<Node*> processNodes(const QList<Node*> &nodesToProcess,
                                     QSet<void*> &processedNodes)
    {
        QList<Node*> toProcess;
        foreach(Node *node, nodesToProcess) {
            if (!processedNodes.contains(node)) {
                if (dependenciesOk(node, processedNodes)) {
                    process(node);
                    processedNodes.insert(node);

                    const QMultiHash<QString, Node::Connection> outputs =
                        node->outputNodes();
                    QMultiHash<QString, Node::Connection>::const_iterator itr;
                    for (itr = outputs.constBegin(); itr != outputs.constEnd();
                         ++itr) {
                        toProcess.append(itr.value().node);
                    }
                } else {
                    qDebug()<<"rescheduling node "<<node;
                    toProcess.append(node);
                }
            }
        }
        return toProcess;
    }
    inline bool dependenciesOk(Node *n, QSet<void*> &processedNodes)
    {
        const QHash<QString, Node::Connection> &inputs = n->inputNodes();
        QHash<QString, Node::Connection>::const_iterator itr;
        for (itr = inputs.constBegin(); itr != inputs.constEnd(); ++itr) {
            if (!processedNodes.contains(itr.value().node))
                return false;
        }
        return true;
    }
    void init()
    {
        localContext = 0;
    }

    QList<Node*> createdNodes;
    bool         dirty;
    QList<Node*> topLevelNodes;
    QList<Node*> resultNodes;
    QList<Node*> processingNodes;
    Texture      result;

    QGLPixelBuffer   *localContext;
};

void Composition::Private::layoutGraph()
{
#if 0
    if (!createdNodes.isEmpty()) {
        foreach(Node *node, createdNodes) {
            if (node->inputNodes().isEmpty())
                topLevelNodes.append(node);
            if (node->outputNodes().isEmpty())
                resultNodes.append(node);
        }
        createdNodes = QList<Node*>();
    }

    //if we have a forced size then propagate
    // that through the children
    foreach(Node *node, resultNodes) {
    }
    //use the domain of definition to compute
    // computational nodes
    foreach(Node *node, topLevelNodes) {
    }
#else
    assert(processingNodes.isEmpty());
    if (!createdNodes.isEmpty()) {
        foreach(Node *node, createdNodes) {
            node->prepare();
            processingNodes.append(node);
        }
        createdNodes = QList<Node*>();
    }
#endif
}

void Composition::Private::schedule()
{
}

Composition::Composition(QWidget *parent)
    : QObject(parent),
    d(new Private)
{
    d->init();
}


Composition::Composition(QGraphicsItem *parent)
    : d(new Private)
{
    d->init();
}

Composition::~Composition()
{
    delete d;
}


QSize Composition::size() const
{
    return QSize();
}


QString Composition::toXml() const
{
    return QString();
}

bool Composition::fromXml(const QString &)
{
    return false;
}

void Composition::addNode(Node *node, Node *parent)
{
    if (!parent)
        d->createdNodes.append(node);
    else {
        int parentIdx = d->createdNodes.indexOf(parent);
        ++parentIdx;
        if (parentIdx > d->createdNodes.size())
            parentIdx = d->createdNodes.size();

        d->createdNodes.insert(parentIdx, node);
    }
}

void Quasar::Composition::removeNode(Node *node)
{
    d->createdNodes.removeAll(node);
}

Node * Composition::createNode(const QString &url)
{
    Node *node = NodeLoader::self()->createNode(url);
    d->createdNodes.append(node);
    return node;
}

void Composition::execute()
{
    d->makeCurrent();
    d->execute();
    d->doneCurrent();
}

Texture Composition::renderToTexture()
{
    d->layoutGraph();
    d->schedule();
    qDebug()<<"d->result = "<<d->resultNodes.count();
    foreach(Node *node, d->resultNodes) {
        QList<Property> props = node->properties();
        foreach(Property prop, props) {
            if (prop.kind == Quasar::Property::OutputProperty) {
                if (prop.type == QVariant::UserType) {
                    Texture tex = qvariant_cast<Texture>(prop.value);
                    return tex;
                }
            }
        }
    }
    return Texture();
}

QStringList Composition::groups()
{
    return NodeLoader::self()->availableGroups();
}

QStringList Composition::nodesInGroup(const QString &group)
{
    return NodeLoader::self()->filtersInGroup(group);
}

void Composition::prepare()
{
    d->prepareContext();
    d->makeCurrent();
    d->layoutGraph();
    d->doneCurrent();
}

}


#include "moc_composition.cpp"
