#define GL_GLEXT_PROTOTYPES
#include "filternode.h"

#include "filternode_p.h"
#include "mesh.h"

#include <QGLFramebufferObject>
#include <QtGui/QLabel>
#include <QDebug>

#include <GL/glext.h>
#include <GL/gl.h>


namespace Quasar
{

FilterNodePrivate::FilterNodePrivate()
    : ShaderNodePrivate(),
      mesh(0)
{
    Quasar::Property tin = {
        Quasar::Property::InputProperty,

        QString::fromLatin1("texture"),
        QVariant::UserType,
        QVariant(),
        QString::fromLatin1("Input texture"),

        QVariant(),
        QVariant(),
        QVariant(),
    };
    properties.insert(QString::fromLatin1("texture"), tin);

    Quasar::Property out = {
        Quasar::Property::OutputProperty,

        QString::fromLatin1("texture"),
        QVariant::UserType,
        QVariant(),
        QString::fromLatin1("Processed texture"),

        QVariant(),
        QVariant(),
        QVariant(),
    };
    properties.insert(QString::fromLatin1("output"), out);
}

FilterNode::FilterNode(const QString &name)
    : ShaderNode(name, *(new FilterNodePrivate()))
{
}

bool FilterNode::isFilterNode() const
{
    return true;
}

bool FilterNode::execute()
{
    Q_D(FilterNode);
    if (!m_fbo) {
        return false;
    }
    m_fbo->bind();
    glClear(GL_COLOR_BUFFER_BIT |
            GL_DEPTH_BUFFER_BIT);
    glUseProgram(programId());
    aboutToExecute();
    Q_ASSERT(d->mesh);
    d->mesh->render();
    glUseProgram(0);
    m_fbo->release();
    Texture out(m_fbo->texture());
    outputReady(QLatin1String("output"), out);

#if SAVE_TEXTURE
    {
        QImage img = out.toImage();
        img.save("yoX.png");
    }
#endif

    return true;
}

void FilterNode::aboutToExecute()
{
}

void FilterNode::updateFilterSize(const Texture &master, int meshSize)
{
    Q_D(FilterNode);
    if (!d->mesh || d->mesh->texture().id() != master.id()) {
        delete d->mesh;
        d->mesh = new Mesh(master, meshSize);
    }
}

void FilterNode::setShaderSources(const QString &vs, const QString &fs)
{
    Q_D(FilterNode);
    d->vs = vs;
    d->fs = fs;
}

void FilterNode::prepare()
{
    Q_D(FilterNode);
    loadShaders(d->vs, d->fs);
    QVariant res = propertyValue("texture");
    Texture tex = qvariant_cast<Texture>(res);
    updateFilterSize(tex, 2);
}

void FilterNode::propertyChanged(const QString &name, const QVariant &val)
{
    if (name == "texture") {
        Texture tex = qvariant_cast<Texture>(val);
        updateFilterSize(tex, 2);
    } else if (hasProperty(name, Property::InputProperty)) {
        glUseProgram(programId());
        QByteArray latin1 = name.toLatin1();
        const char *nameC = latin1.constData();
        switch(val.type()) {
        case QVariant::Int: {
            int i = qvariant_cast<int>(val);
            setShaderUniform(nameC, i);
        }
            break;
        case QVariant::Double: {
            double d = qvariant_cast<double>(val);
            setShaderUniform(nameC, (float)d);
        }
            break;
        case QVariant::Color: {
            QColor c = qvariant_cast<QColor>(val);
            vec4 vec;
            vec.x = c.redF();
            vec.y = c.greenF();
            vec.z = c.blueF();
            vec.w = c.alphaF();
            setShaderUniform(nameC, vec);
        }
            break;
        case QVariant::List: {
            QVariantList l = qvariant_cast<QVariantList>(val);
            /* FIXME */
            unkownPropertyChanged(name, val);
        }
            break;
        case QVariant::Matrix: {
            QMatrix m = qvariant_cast<QMatrix>(val);
            QTransform t(m);
            setShaderMatrixUniform(nameC, t);
        }
            break;
        case QVariant::Transform: {
            QTransform t = qvariant_cast<QTransform>(val);
            setShaderMatrixUniform(nameC, t);
        }
            break;
        case QVariant::Point: {
            QPoint m = qvariant_cast<QPoint>(val);
            /* FIXME */
            unkownPropertyChanged(name, val);
        }
            break;
        case QVariant::PointF: {
            QPointF pt = qvariant_cast<QPointF>(val);
            vec2 vec;
            vec.x = pt.x();
            vec.y = pt.y();
            setShaderUniform(nameC, vec);
        }
            break;
        case QVariant::Rect: {
            QRect m = qvariant_cast<QRect>(val);
            /* FIXME */
            unkownPropertyChanged(name, val);
        }
            break;
        case QVariant::RectF: {
            QRectF r = qvariant_cast<QRectF>(val);
            vec4 vec;
            vec.x = r.x();
            vec.y = r.y();
            vec.z = r.width();
            vec.w = r.height();
            setShaderUniform(nameC, vec);
        }
            break;
        case QVariant::Size: {
            QSize m = qvariant_cast<QSize>(val);
            /* FIXME */
            unkownPropertyChanged(name, val);
        }
            break;
        case QVariant::SizeF: {
            QSizeF s = qvariant_cast<QSizeF>(val);
            vec2 vec;
            vec.x = s.width();
            vec.y = s.height();
            setShaderUniform(nameC, vec);
        }
            break;
        case QVariant::UInt: {
            uint i = qvariant_cast<uint>(val);
            setShaderUniform(nameC, (int)i);
        }
            break;
        case QVariant::UserType: {
            /* FIXME */
            unkownPropertyChanged(name, val);
        }
            break;
        default:
            unkownPropertyChanged(name, val);
            break;
        }
        glUseProgram(0);
    }
}

void FilterNode::unkownPropertyChanged(const QString &, const QVariant &)
{
}

}

#include "moc_filternode.cpp"
