#ifndef QUASAR_NODE_H
#define QUASAR_NODE_H

#include <QObject>
#include <QHash>
#include <QList>
#include <QVariant>
#include <QString>

namespace Quasar
{
    struct Property {
        enum PropertyKind {
            InputProperty,
            OutputProperty
        };
        PropertyKind   kind;
        QString        name;
        QVariant::Type type;
        QVariant       value;
        QString        description;

        QVariant       defaultValue;
        QVariant       min;
        QVariant       max;
    };

    class NodePrivate;

    class Node : public QObject
    {
        Q_OBJECT
        Q_DECLARE_PRIVATE(Node)
    public:
        enum ExecutionRestrain {
            NoRestrain           = 0,
            CpuRestrain          = 1 << 0,
            GpuRestrain          = 1 << 1,
            MainThreadRestrain   = 1 << 2
        };
    public:
        struct Connection {
            Connection()
                : node(0)
            {}
            Connection(const QString &k, Node *n)
                : key(k), node(n)
            {}
            QString key;
            Node *node;
        };
    public:
        Node(const QString &name);
        ~Node();

        QString name() const;

        bool isDirty() const;

        bool isDependentOn(Node *node) const;
        bool isIndependentOf(Node *node) const;

        virtual bool isFilterNode() const;
        virtual bool isGraphicsNode() const;
        virtual bool isRendererNode() const;

        QList<Node*> children() const;
        QList<Node*> parents() const;

        QList<Property> properties() const;
        QList<QString> inputKeys() const;
        QList<QString> outputKeys() const;
        QList<Node*> receiversFor(const QString &prop) const;
        Node *senderTo(const QString &prop) const;
        bool hasProperty(const QString &key,
                         Property::PropertyKind k = Property::InputProperty) const;
        QVariant::Type propertyType(const QString &key) const;
        const Property property(const QString &key) const;
        QVariant propertyValue(const QString &key) const;

        bool link(const QString &outKey,
                  Node *other, const QString &inKey);
        bool unlink(const QString &outKey=QString(),
                    Node *other=0, const QString &inKey=QString());

        bool hasInputs() const;
        bool hasOutputs() const;
        QMultiHash<QString, Connection> outputNodes() const;
        QList<Connection> outputNodes(const QString &key) const;
        QHash<QString, Node::Connection>  inputNodes() const;

        virtual ExecutionRestrain executionRestrain() const;

    public Q_SLOTS:
        bool setPropertyValue(const QString &key,
                              const QVariant &value);
        virtual void prepare();
        virtual bool process();

    Q_SIGNALS:
        void warning(const QString &warning);

    protected:
        virtual bool execute() =0;
        virtual void propertyChanged(const QString &name,
                                     const QVariant &val);
    protected:
        void addProperty(const QString &key,
                         const Property &prop);
        bool outputReady(const QString &key,
                         const QVariant &var);
        void informReceivers();

    protected:
        Node(const QString &name, NodePrivate &dd);
    protected:
        NodePrivate *d_ptr;
    private:
        Q_DISABLE_COPY(Node)
    };

    bool link(Node *src, const QString &outKey,
              Node *dst, const QString &inKey);

    bool unlink(Node *src, const QString &outKey,
                Node *dst, const QString &inKey);
}


#endif
