#include "nodeloader.h"

#include "brightnessnode_p.h"
#include "fadenode_p.h"
#include "gaussianblur_p.h"
#include "imageinput_p.h"
#include "imageoutput_p.h"
#include "renderoutput_p.h"
#include "saturationnode_p.h"
#include "transformnode_p.h"
#include "edgedetection_p.h"

#include <QtCore/qdebug.h>

namespace Quasar
{

Q_GLOBAL_STATIC(NodeLoader, loader)

template <class T>
class TemplateCreator : public NodeCreator
{
public:
    virtual Node *createNode()
    {
        return new T;
    }
};


NodeLoader::NodeLoader()
{
    loadBuiltins();
}

NodeLoader::~NodeLoader()
{
    qDeleteAll(m_nodes);
}

NodeLoader * NodeLoader::self()
{
    return loader();
}

Node * NodeLoader::createNode(const QString &url)
{
    NodeCreator *creator = m_nodes[url];
    if (creator)
        return creator->createNode();
    return 0;
}

QStringList NodeLoader::availableGroups() const
{
    return m_groups.keys();
}

QStringList NodeLoader::filtersInGroup(const QString &group) const
{
    return m_groups[group];
}

QStringList NodeLoader::availableFilters() const
{
    return m_nodes.keys();
}

void NodeLoader::loadBuiltins()
{
    registerNode("/blur/gaussian", new TemplateCreator<GaussianBlurNode>());
    registerNode("/color/brightness", new TemplateCreator<BrightnessNode>());
    registerNode("/color/edgedetect", new TemplateCreator<EdgeDetectionNode>());
    registerNode("/color/fade", new TemplateCreator<FadeNode>());
    registerNode("/color/saturation", new TemplateCreator<SaturationNode>());
    registerNode("/geometry/transform",
                 new TemplateCreator<TransformNode>());
    registerNode("/input/file", new TemplateCreator<ImageInputNode>());
    registerNode("/output/image", new TemplateCreator<ImageOutputNode>());
    registerNode("/output/render", new TemplateCreator<RenderOutputNode>());
}

void Quasar::NodeLoader::registerNode(const QString &name,
                                      NodeCreator *creator)
{
    QStringList lst = name.split('/', QString::SkipEmptyParts);
    //qDebug()<<lst;
    m_nodes.insert(name, creator);
    QStringList algos = m_groups[lst[0]];
    algos << lst[1];
    m_groups.insert(lst[0], algos);
}

}



