#define GL_GLEXT_PROTOTYPES
#include "shadernode.h"

#include "shadernode_p.h"

#include <QtGui/QLabel>
#include <QDebug>

#include <GL/glext.h>
#include <GL/gl.h>

namespace Quasar
{

static inline void
fillGLMatrix(const QTransform &t, GLfloat *mat)
{
    mat[0]  = t.m11(); mat[1]  = t.m12(); mat[2]  = 0; mat[3]  = t.m13();
    mat[4]  = t.m21(); mat[5]  = t.m22(); mat[6]  = 0; mat[7]  = t.m23();
    mat[8]  = 0;       mat[9]  = 0;       mat[10] = 0; mat[11] = 0;
    mat[12] = t.m31(); mat[13] = t.m32(); mat[14] = 0; mat[15] = t.m33();
}

ShaderNodePrivate::ShaderNodePrivate()
    : NodePrivate(),
      fragShader(0),
      vertShader(0),
      prog(0)
{
}

bool ShaderNodePrivate::checkLinkedProgram(GLuint prog)
{
    //Q_Q(Node);
    GLint stat;
    glGetProgramiv(prog, GL_LINK_STATUS, &stat);
    if (!stat) {
        GLchar log[1000];
        GLsizei len;
        glGetProgramInfoLog(prog, 1000, &len, log);
        QString errorStr = QString::fromLatin1(log);
        fprintf(stderr, "Linker error:\n%s\n", log);
        //emit q->warning(log);
        return false;
    }
    return true;
}

bool ShaderNodePrivate::loadAndCompileShader(GLuint shader, const char *text)
{
    //Q_Q(Node);
    GLint stat;

    qDebug()<<"Loading shader "<<shader;
    qDebug()<<text;
    qDebug()<<"----------------";
    glShaderSource(shader, 1, (const GLchar **) &text, NULL);

    glCompileShader(shader);

    glGetShaderiv(shader, GL_COMPILE_STATUS, &stat);
    if (!stat) {
        GLchar log[1000];
        GLsizei len;
        glGetShaderInfoLog(shader, 1000, &len, log);
        QString errorStr = QString::fromLatin1(log);
        qDebug() << "Problem compiling shader: "<<errorStr;
        //emit q->warning(errorStr);
        return false;
    }
    return true;
}

ShaderNode::ShaderNode(const QString &name)
    : RendererNode(name, *(new ShaderNodePrivate()))
{
}

bool ShaderNode::loadShaders(const QString &xvs, const QString &xfs)
{
    Q_D(ShaderNode);
    QString fs = xfs;
    QString vs = xvs;

    if (vs.isEmpty() && fs.isEmpty() &&
        d->currentVs.isEmpty() && d->currentFs.isEmpty())
    {
        d->prog = 0;
        return true;
    }

    d->fragShader = glCreateShader(GL_FRAGMENT_SHADER);
    if (!d->loadAndCompileShader(d->fragShader,
                                 fs.toLatin1().constData())) {
        fs = d->currentFs;
        d->loadAndCompileShader(d->fragShader,
                                fs.toLatin1().constData());
    }

    d->vertShader = glCreateShader(GL_VERTEX_SHADER);
    if (!d->loadAndCompileShader(d->vertShader,
                                 vs.toLatin1().constData())) {
        vs = d->currentVs;
        d->loadAndCompileShader(d->vertShader,
                                vs.toLatin1().constData());
    }

    d->prog = glCreateProgram();
    glAttachShader(d->prog, d->fragShader);
    glAttachShader(d->prog, d->vertShader);
    glLinkProgram(d->prog);
    bool linkedOK = d->checkLinkedProgram(d->prog);
    glUseProgram(d->prog);

    if (linkedOK) {
        programChanged(d->prog);
        d->currentVs = vs;
        d->currentFs = fs;
    } else {
        if (fs != d->currentFs || vs != d->currentVs)
            linkedOK = loadShaders(d->currentVs, d->currentFs);
    }
    return linkedOK;
}

GLuint ShaderNode::vertexShaderId() const
{
    Q_D(const ShaderNode);
    return d->vertShader;
}

GLuint ShaderNode::fragmentShaderId() const
{
    Q_D(const ShaderNode);
    return d->fragShader;
}

GLuint ShaderNode::programId() const
{
    Q_D(const ShaderNode);
    return d->prog;
}

void ShaderNode::programChanged(GLuint)
{
}

bool ShaderNode::setShaderUniform(const char *name, int val)
{
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;
    glUniform1iARB(loc, val);
    return true;
}

bool ShaderNode::setShaderUniform(const char *name, float val)
{
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;
    glUniform1fARB(loc, val);
    return true;
}

bool ShaderNode::setShaderUniform(const char *name, const vec2 &val)
{
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;
    glUniform2fARB(loc, val.x, val.y);
    return true;
}

bool ShaderNode::setShaderUniform(const char *name, const vec3 &val)
{
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;
    glUniform3fARB(loc, val.x, val.y, val.z);
    return true;
}

bool ShaderNode::setShaderUniform(const char *name, const vec4 &val)
{
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;
    glUniform4fARB(loc, val.x, val.y, val.z, val.w);
    return true;
}

bool ShaderNode::setShaderUniform(const char *name, const QVector<float> &vals)
{
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;
    glUniform1fvARB(loc, vals.count(), vals.data());
    return true;
}

bool ShaderNode::setShaderUniform(const char *name, const QVector<vec2> &vals)
{
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;
    glUniform2fvARB(loc, vals.count(), (const GLfloat*)vals.data());
    return true;
}

bool ShaderNode::setShaderUniform(const char *name, const QVector<vec3> &vals)
{
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;
    glUniform3fvARB(loc, vals.count(), (const GLfloat*)vals.data());
    return true;
}

bool ShaderNode::setShaderUniform(const char *name, const QVector<vec4> &vals)
{
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;
    glUniform4fvARB(loc, vals.count(), (const GLfloat*)vals.data());
    return true;
}

int ShaderNode::uniformLocation(const char *name) const
{
    GLint loc = glGetUniformLocationARB(programId(), name);
    if (loc < 0) {
        qWarning()<<"Uniform \""<<name<<"\" doesn't exist!";
    }
    return loc;
}

ShaderNode::ShaderNode(const QString &name, ShaderNodePrivate &dd)
    : RendererNode(name, dd)
{
}

bool ShaderNode::setShaderMatrixUniform(const char *name,
                                        const QTransform &trans)
{
    GLfloat matrix[16];
    GLint loc = uniformLocation(name);
    if (loc < 0)
        return false;

    fillGLMatrix(trans, matrix);
    glUniformMatrix4fvARB(loc, 1, GL_FALSE, matrix);

    return true;
}

}


#include "moc_shadernode.cpp"
