/*  This file is part of the KDE project
    Copyright (C) 2008 Matthias Kretz <kretz@kde.org>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) version 3.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#define GL_GLEXT_PROTOTYPES
#include "phononinputnode.h"
#include "phononinputnode_p.h"

#include "texture.h"

#include <QImage>
#include <QtCore/QCoreApplication>
#include <QtCore/QFile>
#include <QtGui/QImage>
#include <QtGui/QLabel>
#include <QtGui/QPixmap>
#include <QtOpenGL/QGLFramebufferObject>
#include <qdebug.h>
#include <phonon/experimental/abstractvideodataoutput.h>
#include <phonon/experimental/videoframe2.h>

#include <GL/glext.h>
#include <GL/gl.h>

Q_DECLARE_METATYPE(Phonon::Experimental::VideoFrame2)

using Phonon::Experimental::AbstractVideoDataOutput;
using Phonon::Experimental::VideoFrame2;

namespace Quasar
{

PhononInputNodePrivate::PhononInputNodePrivate()
    : fragShader(0),
    program(0),
    fbo(0)
{
}

PhononInputNode::PhononInputNode()
    : GraphicsNode("Phonon Input"),
    d_ptr(new PhononInputNodePrivate)
{
    d_ptr->q_ptr = this;
    d_ptr->currentFormat = VideoFrame2::Format_RGB888;

    Quasar::Property out = {
        Quasar::Property::OutputProperty,

        QString::fromLatin1("texture"),
        QVariant::UserType,
        QVariant(),
        QString::fromLatin1("Processed texture"),

        QVariant(),
        QVariant(),
        QVariant(),
    };
    addProperty(QLatin1String("output"), out);

    setAllowedFormats(QSet<VideoFrame2::Format>() << VideoFrame2::Format_RGB888 << VideoFrame2::Format_YV12 << VideoFrame2::Format_YUY2);
}

PhononInputNode::~PhononInputNode()
{
    delete d_ptr;
}

QRectF PhononInputNode::domainOfDefinition(const QRectF &) const
{
    Q_D(const PhononInputNode);
    if (d->fbo) {
        return QRectF(0.0, 0.0, d->fbo->width(), d->fbo->height());
    }
    return QRectF(0.0, 0.0, 1.0, 1.0); // FIXME: I'd like to return a null rect, but Quasar doesn't like that
}

void PhononInputNode::prepare()
{
    Q_D(PhononInputNode);
    setRunning(true);
    d->loadProgram();
}

inline void PhononInputNodePrivate::checkFormat(VideoFrame2::Format f)
{
    if (currentFormat != f) {
        currentFormat = f;
        loadProgram();
    }
}

bool PhononInputNode::execute()
{
    Q_D(PhononInputNode);
    if (!d->fbo || d->currentFormat == VideoFrame2::Format_Invalid) {
        return false;
    }

    glPushAttrib(GL_VIEWPORT_BIT);
    qDebug() << "setting fbo viewport to" << d->fbo->size();
    glViewport(0, 0, d->fbo->width(), d->fbo->height());

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glOrtho(0, d->fbo->width(), 0, d->fbo->height(), -999999, 999999);
    glMatrixMode(GL_MODELVIEW);

    d->fbo->bind();

    switch (d->currentFormat) {
    case VideoFrame2::Format_Invalid:
        abort();
        break;
    case VideoFrame2::Format_RGB888:
        // need this only to flip the image
        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, d->texture.id());
        glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(0, 1); glVertex3f(0, 0, 0);
        glTexCoord2f(0, 0); glVertex3f(0, d->fbo->height(), 0);
        glTexCoord2f(1, 1); glVertex3f(d->fbo->width(), 0, 0);
        glTexCoord2f(1, 0); glVertex3f(d->fbo->width(), d->fbo->height(), 0);
        glEnd();
        break;
    case VideoFrame2::Format_YV12:
        glEnable(GL_FRAGMENT_PROGRAM_ARB);
        glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, d->program);

        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, d->texture.id());
        glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(0, 2.0f/3.0f); glVertex3f(0, 0, 0);
        glTexCoord2f(0, 0); glVertex3f(0, d->fbo->height(), 0);
        glTexCoord2f(1, 2.0f/3.0f); glVertex3f(d->fbo->width(), 0, 0);
        glTexCoord2f(1, 0); glVertex3f(d->fbo->width(), d->fbo->height(), 0);
        glEnd();

        glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, 0);
        glDisable(GL_FRAGMENT_PROGRAM_ARB);
        break;
    case VideoFrame2::Format_YUY2:
        glUseProgram(d->program);
        glUniform1fARB(d->oneOverFrameWidthParameter, 1.0f / d->texture.width());

        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D, d->texture.id());
        glBegin(GL_TRIANGLE_STRIP);
        glTexCoord2f(0, 1); glVertex3f(0, 0, 0);
        glTexCoord2f(0, 0); glVertex3f(0, d->fbo->height(), 0);
        glTexCoord2f(d->texture.width(), 1); glVertex3f(d->fbo->width(), 0, 0);
        glTexCoord2f(d->texture.width(), 0); glVertex3f(d->fbo->width(), d->fbo->height(), 0);
        glEnd();

        glUseProgram(0);
        break;
    }

    d->fbo->release();

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);

    glPopAttrib();

    Texture out(d->fbo->texture());
    outputReady(QLatin1String("output"), out);
    return true;
}

void PhononInputNodePrivate::loadProgram()
{
    switch (currentFormat) {
    case VideoFrame2::Format_Invalid:
        break;
    case VideoFrame2::Format_RGB888:
        qDebug() << "loading RGB shader";
        if (glIsProgram(program)) {
            glDeleteProgram(program);
        } else if (glIsProgramARB(program)) {
            glDeleteProgramsARB(0, &program);
        }
        program = 0;
        break;
    case VideoFrame2::Format_YV12:
        qDebug() << "loading YV12 shader";
        {
            if (!glIsProgramARB(program)) {
                if (glIsProgram(program)) {
                    glDeleteProgram(program);
                }
                glGenProgramsARB(1, &program);
            }
            const QByteArray fragShaderSource(
                    "!!ARBfp1.0\n"
                    "PARAM c[3] = { { 1, 0.5, 0, 0.0625 },\n"
                    "                { 0.7927399, -0.53291321, 0, 0.66666669 },\n"
                    "                { -0.21324158, 2.1123962, 1.1643829 } };\n"
                    "TEMP tmp;\n"
                    "MAD tmp, fragment.texcoord[0].xyxy, c[0].xxyy, c[1].zzzw;\n"
                    "TEX tmp.x, tmp.xyzw, texture[0], 2D;\n"
                    "TEX tmp.y, tmp.zwzw, texture[0], 2D;\n"
                    "ADD tmp.zw, tmp.zwzw, c[0].yzyz;\n"
                    "TEX tmp.z, tmp.zwzw, texture[0], 2D;\n"
                    "ADD tmp.xyz, tmp, -c[0].wyyy;\n"
                    "MUL tmp.yw, tmp.y, c[2].xxxy;\n"
                    "MAD tmp.yzw, tmp.z, c[1].xyxz, tmp.xyzw;\n"
                    "MAD result.color.xyz, tmp.x, c[2].z, tmp.zyww;\n"
                    "END\n");
            glEnable(GL_FRAGMENT_PROGRAM_ARB);
            glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, program);
            glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, fragShaderSource.size(), fragShaderSource.constData());
            if (glGetError() == GL_INVALID_OPERATION) {
                glDisable(GL_FRAGMENT_PROGRAM_ARB);
                qDebug() << (const char *)glGetString(GL_PROGRAM_ERROR_STRING_ARB);
                glDeleteProgramsARB(1, &program);
                program = 0;
                return;
            }
            qDebug() << "loaded fragment program:\n" << fragShaderSource;
        }
        break;
    case VideoFrame2::Format_YUY2:
        qDebug() << "loading YUY2 shader";
        {
            if (!glIsShader(fragShader)) {
                fragShader = glCreateShader(GL_FRAGMENT_SHADER);
            }
            if (!glIsProgram(program)) {
                if (glIsProgramARB(program)) {
                    glDeleteProgramsARB(1, &program);
                }
                program = glCreateProgram();
            }
            const char *fragShaderSource =
                    "#version 120\n"
                    "uniform sampler2D yuvTex;\n"
                    "uniform float oneOverFrameWidth;\n"
                    "\n"
                    "void main()\n"
                    "{\n"
                    "    vec4 coord = gl_TexCoord[0].xyxx;\n"
                    "\n"
                    "    // Cb sample is at offset 0 if step == 0 and at offset -1 if step == 1\n"
                    "    coord.z -= step(1.0, mod(coord.x, 2.0));\n"
                    "\n"
                    "    // Cr sample is always right of the Cb sample\n"
                    "    coord.w = coord.z + 1.0;\n"
                    "\n"
                    "    // normalize s coordinate back to 0..1\n"
                    "    coord.xzw *= oneOverFrameWidth;\n"
                    "\n"
                    "    vec3 yuv;\n"
                    "    yuv.r = texture2D(yuvTex, coord.xy).r;\n"
                    "    yuv.g = texture2D(yuvTex, coord.zy).a;\n"
                    "    yuv.b = texture2D(yuvTex, coord.wy).a;\n"
                    "\n"
                    "    yuv -= vec3(0.0625, 0.5, 0.5);\n"
                    "    gl_FragColor = vec4(mat3x3(\n"
                    "            1.1643829345703125,  1.1643829345703125, 1.1643829345703125,\n"
                    "            0.0               , -0.2132415771484375, 2.112396240234375,\n"
                    "            1.7927398681640625, -0.5329132080078125, 0.0\n"
                    "            ) * yuv, 1.0);\n"
                    "}\n";
            GLint stat;
            glShaderSource(fragShader, 1, &fragShaderSource, NULL);
            glCompileShader(fragShader);
            glGetShaderiv(fragShader, GL_COMPILE_STATUS, &stat);
            if (!stat) {
                GLchar log[1000];
                GLsizei len;
                glGetShaderInfoLog(fragShader, 1000, &len, log);
                QString errorStr = QString::fromLatin1(log);
                qDebug() << "Problem compiling shader: "<<errorStr;
            }
            glAttachShader(program, fragShader);
            glLinkProgram(program);

            glGetProgramiv(program, GL_LINK_STATUS, &stat);
            if (!stat) {
                GLchar log[1000];
                GLsizei len;
                glGetProgramInfoLog(program, 1000, &len, log);
                qDebug() << "Linker error:\n" << log;
            } else {
                glUseProgram(program);
                GLuint texLoc = glGetUniformLocationARB(program, "yuvTex");
                glUniform1iARB(texLoc, 0);
                oneOverFrameWidthParameter = glGetUniformLocationARB(program, "oneOverFrameWidth");
            }
        }
        break;
    }
}

void PhononInputNode::frameReady(const VideoFrame2 &frame)
{
    Q_D(PhononInputNode);
    d->mutex.lock();
    d->frameToProcess = frame;
    QCoreApplication::postEvent(this, new QEvent(QEvent::User));
    d->wait.wait(&d->mutex, 1000);
    d->mutex.unlock();
}

bool PhononInputNode::event(QEvent *e)
{
    if (e->type() != QEvent::User) {
        return GraphicsNode::event(e);
    }

    Q_D(PhononInputNode);
    d->mutex.lock();
    // we need to at least upload the texture since the frame data pointers will be invalidated
    // right after this function returns.
    // processing can wait

    d->checkFormat(d->frameToProcess.format);
    GLuint texId;
    switch (d->frameToProcess.format) {
    case VideoFrame2::Format_Invalid:
        d->wait.wakeAll();
        d->mutex.unlock();
        return true;
    case VideoFrame2::Format_RGB888:
        if (d->texture.isNull()) {
            glGenTextures(1, &texId);
            glBindTexture(GL_TEXTURE_2D, texId);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
            glTexImage2D(GL_TEXTURE_2D, 0, 3, d->frameToProcess.width, d->frameToProcess.height, 0, GL_RGB, GL_UNSIGNED_BYTE, d->frameToProcess.data0.constData());
            d->texture = Texture(texId);
        } else {
            d->texture.updateRect(QRect(0, 0, d->frameToProcess.width, d->frameToProcess.height), d->frameToProcess.data0, GL_RGB);
        }
        break;
    case VideoFrame2::Format_YV12:
        if (d->texture.isNull()) {
            glGenTextures(1, &texId);
            glBindTexture(GL_TEXTURE_2D, texId);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
            glTexImage2D(GL_TEXTURE_2D, 0, 1, d->frameToProcess.width, (d->frameToProcess.height >> 1) + d->frameToProcess.height, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
            d->texture = Texture(texId);
        }
        d->texture.updateRect(QRect(0, 0, d->frameToProcess.width, d->frameToProcess.height), d->frameToProcess.data0, GL_LUMINANCE);
        d->texture.updateRect(QRect(0, d->frameToProcess.height, d->frameToProcess.width >> 1, d->frameToProcess.height >> 1), d->frameToProcess.data1, GL_LUMINANCE);
        d->texture.updateRect(QRect(d->frameToProcess.width >> 1, d->frameToProcess.height, d->frameToProcess.width >> 1, d->frameToProcess.height >> 1), d->frameToProcess.data2, GL_LUMINANCE);
        break;
    case VideoFrame2::Format_YUY2:
        if (d->texture.isNull()) {
            glGenTextures(1, &texId);
            glBindTexture(GL_TEXTURE_2D, texId);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
            glTexImage2D(GL_TEXTURE_2D, 0, 2, d->frameToProcess.width, d->frameToProcess.height, 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, 0);
            d->texture = Texture(texId);
        }
        d->texture.updateRect(QRect(0, 0, d->frameToProcess.width, d->frameToProcess.height), d->frameToProcess.data0, GL_LUMINANCE_ALPHA);
        break;
    }
    if (!d->fbo || d->fbo->width() != d->frameToProcess.width || d->fbo->height() != d->frameToProcess.height) {
        delete d->fbo;
        d->fbo = new QGLFramebufferObject(d->frameToProcess.width, d->frameToProcess.height);
        qDebug() << Q_FUNC_INFO << "created QGLFramebufferObject with size" << d->fbo->size();
    }
    d->wait.wakeAll();
    d->mutex.unlock();
    QMetaObject::invokeMethod(this, "needUpdate", Qt::QueuedConnection);
    return true;
}

void PhononInputNode::endOfMedia()
{
}

}
