/* This file is part of Strigi Desktop Search
 *
 * Copyright (C) 2008,2009 Jos van den Oever <jos@vandenoever.info>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "jstream.h"
// Strigi
#include <strigi/archivereader.h>
#include <strigi/fileinputstream.h>
// KDE
#include <kmimetype.h>
#include <kdebug.h>
#include <kcomponentdata.h>
#include <kio/udsentry.h>
// Qt
#include <QFileInfo>
#include <QDirIterator>
#include <QDateTime>

using namespace KIO;
using namespace std;
using namespace Strigi;

class QFileStreamOpener : public StreamOpener {
public:
    ~QFileStreamOpener() {}
    StreamBase<char>* openStream(const string& url);
    int stat(const string& url, EntryInfo& e);
};
StreamBase<char>*
QFileStreamOpener::openStream(const string& url) {
    StreamBase<char>* stream = new FileInputStream(url.c_str());
    if (stream->status() != Ok) {
        delete stream;
        stream = 0;
    }
    return stream;
}
int
QFileStreamOpener::stat(const string& url, EntryInfo& e) {
    QFileInfo f(url.c_str());
    if (!f.exists()) {
        return -1;
    }
    e.type = EntryInfo::Unknown;
    if (f.isFile()) {
        e.type = EntryInfo::File;
    }
    if (f.isDir()) {
        e.type = EntryInfo::Dir;
    }
    e.size = f.size();
    e.mtime = f.lastModified().toTime_t();
    QByteArray filename = f.fileName().toUtf8();
    e.filename.assign((const char*)filename, filename.size());

    return 0;
}

kio_jstreamProtocol::kio_jstreamProtocol(const QByteArray& pool_socket,
        const QByteArray& app_socket)
    : SlaveBase("kio_jstream", pool_socket, app_socket) {
    kDebug(7182) << "kio_jstreamProtocol::kio_jstreamProtocol()" << endl;
    reader = new ArchiveReader();
    opener = new QFileStreamOpener();
    reader->addStreamOpener(opener);
}
kio_jstreamProtocol::~kio_jstreamProtocol() {
    kDebug(7182) << "kio_jstreamProtocol::~kio_jstreamProtocol()" << endl;
    delete reader;
    delete opener;
}
void
kio_jstreamProtocol::get(const KUrl& url) {
    kDebug(7182) << "kio_jstream::get(const KUrl& url)" << endl ;
    const QString path(url.path(KUrl::RemoveTrailingSlash));
    const string stdpath(path.toUtf8());
    QFileInfo f(path);

    if (f.exists()) {
        redirectToLocal(url);
        return;
    }

    StreamBase<char>* in = reader->openStream(stdpath);
    if (in == 0) {
        error( KIO::ERR_CANNOT_OPEN_FOR_READING, url.path() );
        return;
    }

    const char* buf;
    int32_t nread = in->read(buf, 1024, 0);
    QByteArray d;
    if (nread > 0) {
        mimeType(KMimeType::findByContent(d)->name());
        d = QByteArray::fromRawData(buf, nread);
        data(d);
        d.clear();
        nread = in->read(buf, 1, 0);
    }
    while (nread > 0) {
        d = QByteArray::fromRawData(buf, nread);
        data(d);
        d.clear();
        nread = in->read(buf, 1, 0);
    }
    data(QByteArray()); // empty array means we're done sending the data
    finished();
}
UDSEntry
QFileInfoToUDSEntry(const QFileInfo& fi, bool forcedir = false) {
    UDSEntry entry;
    entry.insert(KIO::UDSEntry::UDS_NAME, fi.fileName());
    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,
       (forcedir || fi.isDir()) ?S_IFDIR :S_IFREG);
    entry.insert(KIO::UDSEntry::UDS_SIZE, fi.size());
    entry.insert(KIO::UDSEntry::UDS_ACCESS, fi.permissions());
    entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME,
       fi.lastModified().toTime_t());
    return entry;
}
UDSEntry
kio_jstreamProtocol::statQFileInfo(const QFileInfo& f) const {
    UDSEntry entry;
    bool isArchive = false;
    if (f.isFile()) {
        string path(f.filePath().toUtf8());
        ArchiveReader::DirLister dl = reader->dirEntries(path);
        EntryInfo e;
        isArchive = dl.nextEntry(e);
    }
    entry = QFileInfoToUDSEntry(f, isArchive);
    return entry;
}
UDSEntry
entryInfoToUDSEntry(const EntryInfo& e) {
    UDSEntry entry;
    entry.insert(KIO::UDSEntry::UDS_NAME, QString::fromUtf8(e.filename.c_str(),
        e.filename.length()));
    entry.insert(KIO::UDSEntry::UDS_SIZE, e.size);
    entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, e.mtime);
    int perm = 0444;
    int type = 0;
    if (e.type & EntryInfo::Dir) {
        type |= S_IFDIR;
        perm |= 0111;
    }
    if (e.type & EntryInfo::File) {
        type |= S_IFREG;
    }
    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, type);
    entry.insert(KIO::UDSEntry::UDS_ACCESS, perm);
    return entry;
}
void
kio_jstreamProtocol::mkdir(const KUrl& url, int permissions) {
    const QString path(url.path(KUrl::RemoveTrailingSlash));
    const QFileInfo f(path);
    if (!f.dir().mkdir(f.fileName())) {
        error(ERR_COULD_NOT_MKDIR, path);
        return;
    }
    if (permissions != -1) {
        if (!QFile(path).setPermissions(QFile::Permissions(permissions))) {
            error(ERR_CANNOT_CHMOD, path);
            return;
        }
    }
    finished();
}
void
kio_jstreamProtocol::stat(const KUrl& url)
{
    const QString path(url.path(KUrl::RemoveTrailingSlash));
    const string stdpath(path.toUtf8());
    const QFileInfo f(path);

    if (f.exists()) {
        if (f.isDir() || !reader->isArchive(stdpath)) {
            redirectToLocal(url);
        } else {
            statEntry(QFileInfoToUDSEntry(f, true));
            finished();
        }
    } else {
        EntryInfo e;
        if (reader->stat(stdpath, e) == 0) {
            statEntry(entryInfoToUDSEntry(e));
            finished();
        } else {
            error( KIO::ERR_CANNOT_OPEN_FOR_READING, path);
        }
    }
}
void
kio_jstreamProtocol::redirectToLocal(const KUrl& url) {
    KUrl redir(url);
    redir.setProtocol("file");
    redirection(redir);
    finished();
}
void
kio_jstreamProtocol::mimetype(const KUrl& url)
{
    kDebug(7182) << "kio_jstream::mimetype(const KUrl& url)" << endl;
    if (QFileInfo(url.path()).exists()) {
        redirectToLocal(url);
    } else {
        finished();
    }
}
void
kio_jstreamProtocol::listDir(const KUrl& url) {
    kDebug(7182) << "kio_jstream::listDir(const KUrl& url)" << endl ;
    const QString path(url.path(KUrl::RemoveTrailingSlash));
    const QFileInfo f(path);

    UDSEntry entry;
    if (f.isDir()) {
        QDirIterator d(url.path(),
            QDir::Dirs | QDir::Files | QDir::Hidden | QDir::NoSymLinks);
        while (d.hasNext()) {
            d.next();
            listEntry(statQFileInfo(d.fileInfo()), false);
        }
        listEntry(entry, true);
        finished();
        return;
    }

    const string stdpath(path.toUtf8());
    ArchiveReader::DirLister dl = reader->dirEntries(stdpath);
    EntryInfo e;
    while (dl.nextEntry(e)) {
        entry = entryInfoToUDSEntry(e);
        listEntry(entry, false);
    }
    listEntry(entry, true);
    finished();
}

extern "C"
{
    int KDE_EXPORT kdemain(int argc, char **argv)
    {
        KComponentData componentData( "kio_jstream" );
        
        kDebug(7182) << "*** Starting kio_jstream " << endl;
        
        if (argc != 4) {
            kDebug(7182) << "Usage: kio_jstream  protocol domain-socket1 domain-socket2" << endl;
            exit(-1);
        }
        
        kio_jstreamProtocol slave(argv[2], argv[3]);
        slave.dispatchLoop();
        
        kDebug(7182) << "*** kio_jstream Done" << endl;
        return 0;
    }
} 
